@@ -14,189 +14,141 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef __KATE_MAINWINDOW_H__
 #define __KATE_MAINWINDOW_H__
 
 #include "katemain.h"
-#include "../interfaces/mainwindow.h"
-#include "../interfaces/toolviewmanager.h"
-
-#include "kateviewmanager.h"
-#include "kateprojectmanager.h"
+#include "katemdi.h"
 
 #include <kate/view.h>
 #include <kate/document.h>
 
 #include <kparts/part.h>
-#include <kparts/dockmainwindow.h>
-#include <kmdimainfrm.h>
 
-#include <qguardedptr.h>
-
-#include <scriptmanager.h>
 #include <kaction.h>
 
+class KateTabWidget;
 class GrepTool;
 
+namespace Kate {
+  class MainWindow;
+  class ToolViewManager;
+}
+
 class KFileItem;
 class KRecentFilesAction;
 class DCOPObject;
 
-class KateMainWindow : public KMdiMainFrm, virtual public KParts::PartBase
+class KateExternalToolsMenuAction;
+
+class KateMainWindow : public KateMDI::MainWindow, virtual public KParts::PartBase
 {
   Q_OBJECT
 
   friend class KateConfigDialog;
-  friend class KateApp;
   friend class KateViewManager;
-  friend class KateView;
-  friend class KateDocument;
 
   public:
-    KateMainWindow ( KateDocManager *_docManager, KatePluginManager *_pluginManager,
-                     KateProjectManager *projectMan, KMdi::MdiMode guiMode );
+    /**
+     * Construct the window and restore it's state from given config if any
+     * @param sconfig session config for this window, 0 if none
+     * @param sgroup session config group to use
+     */
+    KateMainWindow (KConfig *sconfig, const QString &sgroup);
+
+    /**
+     * Destruct the nice window
+     */
     ~KateMainWindow();
 
+  /**
+   * Accessor methodes for interface and child objects
+   */
+  public:
     Kate::MainWindow *mainWindow () { return m_mainWindow; }
     Kate::ToolViewManager *toolViewManager () { return m_toolViewManager; }
 
-    Kate::Project *activeProject () { return m_project; }
+    KateViewManager *viewManager () { return m_viewManager; }
 
-    /**
-     * Creates a new project file at give url of given type + opens it
-     * @param type projecttype
-     * @param filename name of the new project file
-     * @return Project new created project object
-     */
-     Kate::Project *createProject (const QString &type, const QString &name, const QString &filename);
-
-    /**
-     * @param filename name of the project file
-     * @return Project opened project
-     */
-    Kate::Project *openProject (const QString &filename);
-
-    void activateProject (Kate::Project *project);
+    DCOPObject *dcopObject () { return m_dcop; }
 
+  /**
+   * various methodes to get some little info out of this
+   */
+  public:
     /** Returns the URL of the current document.
      * anders: I add this for use from the file selector. */
     KURL activeDocumentUrl();
 
-    DCOPObject *dcopObject () { return m_dcop; }
-
-    DCOPObject *m_dcop;
-
-    // console
-    KateConsole *console;
-
-    // management items
-    KateDocManager *m_docManager;
-    KateViewManager *m_viewManager;
-    KateProjectManager *m_projectManager;
-
-    // should be protected, and kateviewmanager a friend class.
-    KRecentFilesAction *fileOpenRecent;
-
-    KateFileList *filelist;
-    class KateProjectList *projectlist;
-    class KateProjectViews *projectviews;
-    KateFileSelector *fileselector;
-
-  private:
-    uint myID;
-    bool syncKonsole;
-    bool modNotification;
-
-  public:
-    bool notifyMod() const { return modNotification; }
     uint mainWindowNumber () const { return myID; }
 
-  protected:
-    KatePluginManager *m_pluginManager;
+    /**
+     * Prompts the user for what to do with files that are modified on disk if any.
+     * This is optionally run when the window receives focus, and when the last
+     * window is closed.
+     * @return true if no documents are modified on disk, or all documents were
+     * handled by the dialog; otherwise (the dialog was canceled) false.
+     */
+    bool showModOnDiskPrompt();
 
-  private:
-    QGuardedPtr<Kate::Project> m_project;
-    uint m_projectNumber;
-    QGuardedPtr<Kate::View> activeView;
-
-    KAction *closeCurrentViewSpace;
-
-    KAction *goNext;
-    KAction *goPrev;
-
-    KAction *saveProject;
-    KAction *closeProject;
-    KRecentFilesAction *recentProjects;
-
-    KActionMenu* documentOpenWith;
-
-    KAction *gotoLine;
-    KAction* windowNext;
-    KAction* windowPrev;
-
-    QPopupMenu *documentMenu;
-
-    KToggleAction* settingsShowFilelist;
-    KToggleAction* settingsShowFileselector;
-    KToggleAction* showFullScreenAction;
-
-    KAction* settingsConfigure;
-
-    KActionMenu *scriptMenu;
-    KScriptManager* kscript;
-
-    class KateExternalToolsMenuAction *externalTools;
-
-  public slots:
-    void newWindow ();
-
-    void slotConfigure();
-
-    void slotOpenWithMenuAction(int idx);
-
-    void slotPipeToConsole ();
-
- private:
-     GrepTool * greptool;
-
-  public slots:
-    void slotGrepToolItemSelected ( const QString &filename, int linenumber );
-    void runScript( int menuItemId);
-    void slotMail();
+    /**
+     * central tabwidget ;)
+     * @return tab widget
+     */
+    KateTabWidget *tabWidget ();
 
   public:
     void readProperties(KConfig *config);
     void saveProperties(KConfig *config);
     void saveGlobalProperties( KConfig* sessionConfig );
 
+  public:
+    bool queryClose_internal();
+
   private:
     void setupMainWindow();
     void setupActions();
-    void setupScripts();
     bool queryClose();
 
-    void readOptions(KConfig *);
-    void saveOptions(KConfig *);
+    /**
+     * read some global options from katerc
+     */
+    void readOptions();
+
+    /**
+     * save some global options to katerc
+     */
+    void saveOptions();
 
     void dragEnterEvent( QDragEnterEvent * );
     void dropEvent( QDropEvent * );
 
+  /**
+   * slots used for actions in the menus/toolbars
+   * or internal signal connections
+   */
   private slots:
+    void newWindow ();
+
+    void slotConfigure();
+
+    void slotOpenWithMenuAction(int idx);
+
+    void slotGrepToolItemSelected ( const QString &filename, int linenumber );
+    void slotMail();
+
     void slotFileQuit();
     void slotEditToolbars();
-    void slotDocumentChanged();
     void slotWindowActivated ();
     void slotUpdateOpenWith();
     void documentMenuAboutToShow();
     void slotDropEvent(QDropEvent *);
     void editKeys();
     void mSlotFixOpenWithMenu();
-    void slotGoNext();
-    void slotGoPrev();
 
     void fileSelected(const KFileItem *file);
 
@@ -206,51 +158,58 @@ class KateMainWindow : public KMdiMainFrm, virtual public KParts::PartBase
     void slotDocumentCreated (Kate::Document *doc);
     void updateCaption (Kate::Document *doc);
 
-  public:
-    void openURL (const QString &name=0L);
-
-    static KMdi::MdiMode defaultMode;
-
-  protected:
-    bool eventFilter( QObject*, QEvent * );
-    static uint uniqueID;
-    Kate::MainWindow *m_mainWindow;
-    Kate::ToolViewManager *m_toolViewManager;
-
-  public:
-    Kate::ViewManager *viewManager () {return m_viewManager->viewManager(); }
-    KateViewManager *kateViewManager () { return m_viewManager; }
-
-  public: //ToolViewManager stuff
-    KMdiToolViewAccessor *addToolView(KDockWidget::DockPosition position, QWidget *widget, const QPixmap &icon, const QString &sname, const QString &tabToolTip = 0, const QString &tabCaption = 0);
-
-    bool removeToolView(QWidget *);
-    bool removeToolView(KMdiToolViewAccessor *);
-
-    bool showToolView(QWidget *);
-    bool showToolView(KMdiToolViewAccessor *);
-
-    bool hideToolView(QWidget *);
-    bool hideToolView(KMdiToolViewAccessor *);
-
-  private slots:
     void pluginHelp ();
     void slotFullScreen(bool);
 
-  // slots for the project GUI actions: new/open/save/close
-  public slots:
-    void slotProjectNew ();
-    void slotProjectOpen ();
-    void slotProjectSave ();
-    void slotProjectClose ();
-
-    // recent files
-    void openConstURLProject (const KURL&);
+  public:
+    void openURL (const QString &name=0L);
 
   private slots:
-    void projectDeleted (uint projectNumber);
+    void updateGrepDir (bool visible);
+
+  protected:
+    bool event( QEvent * );
+
+  private slots:
+    void slotDocumentCloseAll();
+
+  private:
+    static uint uniqueID;
+    uint myID;
+
+    Kate::MainWindow *m_mainWindow;
+    Kate::ToolViewManager *m_toolViewManager;
+
+    bool syncKonsole;
+    bool modNotification;
+
+    DCOPObject *m_dcop;
+
+    // console
+    KateConsole *console;
+
+    // management items
+    KateViewManager *m_viewManager;
+
+    KRecentFilesAction *fileOpenRecent;
+
+    KateFileList *filelist;
+    KateFileSelector *fileselector;
+
+    KActionMenu* documentOpenWith;
+
+    QPopupMenu *documentMenu;
+
+    KToggleAction* settingsShowFilelist;
+    KToggleAction* settingsShowFileselector;
+
+    KateExternalToolsMenuAction *externalTools;
+    GrepTool * greptool;
+    bool m_modignore, m_grrr;
+
+    KateTabWidget *m_tabWidget;
 };
 
 #endif
 
-
+// kate: space-indent on; indent-width 2; replace-tabs on;
