@@ -1,7 +1,7 @@
 /**
  * crypto.cpp
  *
- * Copyright (c) 2000-2003 George Staikos <staikos@kde.org>
+ * Copyright (c) 2000-2005 George Staikos <staikos@kde.org>
  *               2000 Carsten Pfeiffer <pfeiffer@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 //
@@ -31,44 +31,44 @@
 #include <pwd.h>
 #include <unistd.h>
 
-#include <qfile.h>
-#include <qlayout.h>
-#include <qvbox.h>
-#include <qvgroupbox.h>
-#include <qlabel.h>
 #include <qbuttongroup.h>
-#include <qhgroupbox.h>
-#include <qhbuttongroup.h>
-#include <qvbuttongroup.h>
-#include <qradiobutton.h>
-#include <qwhatsthis.h>
-#include <qpushbutton.h>
-#include <qfileinfo.h>
 #include <qcheckbox.h>
-#include <kcombobox.h>
+#include <qfile.h>
+#include <qfileinfo.h>
+#include <qframe.h>
+#include <qhbuttongroup.h>
+#include <qhgroupbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qpushbutton.h>
+#include <qradiobutton.h>
 #include <qregexp.h>
+#include <qvbox.h>
+#include <qvbuttongroup.h>
+#include <qvgroupbox.h>
+#include <qwhatsthis.h>
 
-#include <kfiledialog.h>
-#include <klineedit.h>
+#include <kaboutdata.h>
+#include <kcombobox.h>
 #include <kconfig.h>
-#include <kglobal.h>
-#include <kstandarddirs.h>
-#include <klocale.h>
+#include <kdatepicker.h>
+#include <kdebug.h>
 #include <kdialog.h>
+#include <kfiledialog.h>
+#include <kgenericfactory.h>
+#include <kglobal.h>
+#include <klineedit.h>
+#include <klocale.h>
+#include <kmdcodec.h>
 #include <kmessagebox.h>
 #include <kpassdlg.h>
-#include <kseparator.h>
-#include <kdatepicker.h>
-#include <kurllabel.h>
-#include <kmdcodec.h>
-#include <kgenericfactory.h>
-#include <kurlrequester.h>
+#include <kprocess.h>
 #include <kpushbutton.h>
-
-#include <qframe.h>
-
-
-#include <kdebug.h>
+#include <kresolver.h>
+#include <kseparator.h>
+#include <kstandarddirs.h>
+#include <kurllabel.h>
+#include <kurlrequester.h>
 
 #include <config.h>
 #ifdef HAVE_SSL
@@ -90,8 +90,8 @@
 #include "crypto.h"
 #include "certexport.h"
 #include "kdatetimedlg.h"
-#include <kaboutdata.h>
-#include <kprocess.h>
+
+using namespace KNetwork;
 
 typedef KGenericFactory<KCryptoConfig, QWidget> KryptoFactory;
 K_EXPORT_COMPONENT_FACTORY( kcm_crypto, KryptoFactory("kcmcrypto") )
@@ -121,8 +121,8 @@ QString CipherItem::configName() const
 
 
 
-OtherCertItem::OtherCertItem( QListView *view, QString& sub, bool perm, int policy, QDateTime exp, KCryptoConfig *module )
-    : QListViewItem( view, QString::null ), _sub(sub), _exp(exp), _perm(perm), _policy(policy)
+OtherCertItem::OtherCertItem( QListView *view, const QString& sub, const QString& md5, bool perm, int policy, QDateTime exp, KCryptoConfig *module )
+    : QListViewItem( view, QString::null ), _sub(sub), _md5(md5), _exp(exp), _perm(perm), _policy(policy)
 
 {
     m_module = module;
@@ -216,16 +216,6 @@ QString CAItem::configName() const
 
 
 
-static KAboutData* aboutData() {
-     KAboutData *about =
-     new KAboutData(I18N_NOOP("kcmcrypto"), I18N_NOOP("KDE Crypto Control Module"),
-                    0, 0, KAboutData::License_GPL,
-                    I18N_NOOP("(c) 2000 - 2001 George Staikos"));
-     about->addAuthor("George Staikos", 0, "staikos@kde.org");
-     about->addAuthor("Carsten Pfeiffer", 0, "pfeiffer@kde.org");
-     return about;
-}
-
 KCryptoConfig::KCryptoConfig(QWidget *parent, const char *name, const QStringList &)
   : KCModule(KryptoFactory::instance(), parent, name)
 {
@@ -233,6 +223,10 @@ QGridLayout *grid;
 QBoxLayout *top = new QVBoxLayout(this);
 QString whatstr;
 
+  setQuickHelp( i18n("<h1>Crypto</h1> This module allows you to configure SSL for"
+     " use with most KDE applications, as well as manage your personal"
+     " certificates and the known certificate authorities."));
+
   ___lehack = false;
   otherCertDelList.setAutoDelete(true);
   yourCertDelList.setAutoDelete(true);
@@ -241,9 +235,13 @@ QString whatstr;
 
   _signers = new KSSLSigners;
 
-#if KDE_IS_VERSION( 3, 2, 90 )
-  setAboutData( ::aboutData() );
-#endif
+     KAboutData *about =
+     new KAboutData(I18N_NOOP("kcmcrypto"), I18N_NOOP("KDE Crypto Control Module"),
+                    0, 0, KAboutData::License_GPL,
+                    I18N_NOOP("(c) 2000 - 2001 George Staikos"));
+     about->addAuthor("George Staikos", 0, "staikos@kde.org");
+     about->addAuthor("Carsten Pfeiffer", 0, "pfeiffer@kde.org");
+     setAboutData( about );
 
   ///////////////////////////////////////////////////////////////////////////
   // Create the GUI here - there are currently a total of 6 tabs.
@@ -449,6 +447,7 @@ QString whatstr;
   grid = new QGridLayout(tabYourSSLCert, 16, 6, KDialog::marginHint(), KDialog::spacingHint() );
 
   yourSSLBox = new QListView(tabYourSSLCert);
+  yourSSLBox->setAllColumnsShowFocus(true);
   whatstr = i18n("This list box shows which certificates of yours KDE"
                 " knows about. You can easily manage them from here.");
   QWhatsThis::add(yourSSLBox, whatstr);
@@ -550,6 +549,7 @@ QString whatstr;
 
   grid->addMultiCellWidget(new QLabel(i18n("Host authentication:"), tabAuth), 5, 5, 0, 1);
   hostAuthList = new QListView(tabAuth);
+  hostAuthList->setAllColumnsShowFocus(true);
   grid->addMultiCellWidget(hostAuthList, 6, 13, 0, 5);
   hostAuthList->addColumn(i18n("Host"));
   hostAuthList->addColumn(i18n("Certificate"));
@@ -566,7 +566,7 @@ QString whatstr;
   hostCertBG = new QHButtonGroup(i18n("Action"), tabAuth);
   hostSend = new QRadioButton(i18n("Send"), hostCertBG);
   hostPrompt = new QRadioButton(i18n("Prompt"), hostCertBG);
-  hostDont = new QRadioButton(i18n("Don't send"), hostCertBG);
+  hostDont = new QRadioButton(i18n("Do not send"), hostCertBG);
   grid->addMultiCellWidget(hostCertBG, 16, 16, 0, 5);
 
   authAdd = new QPushButton(i18n("Ne&w"), tabAuth);
@@ -606,6 +606,7 @@ QString whatstr;
   oGrid = grid = new QGridLayout(tabOtherSSLCert, 21, 6, KDialog::marginHint(), KDialog::spacingHint());
 
   otherSSLBox = new QListView(tabOtherSSLCert);
+  otherSSLBox->setAllColumnsShowFocus(true);
   connect(otherSSLBox, SIGNAL(selectionChanged()), SLOT(slotOtherCertSelect()));
   whatstr = i18n("This list box shows which site and person certificates KDE"
                 " knows about. You can easily manage them from here.");
@@ -720,6 +721,7 @@ QString whatstr;
   grid = new QGridLayout(tabSSLCA, 11, 8, KDialog::marginHint(), KDialog::spacingHint());
 
   caList = new QListView(tabSSLCA);
+  caList->setAllColumnsShowFocus(true);
   whatstr = i18n("This list box shows which certificate authorities KDE"
                  " knows about. You can easily manage them from here.");
   QWhatsThis::add(caList, whatstr);
@@ -928,23 +930,23 @@ void KCryptoConfig::load()
   QStringList groups = policies->groupList();
 
   otherSSLBox->clear();
-  for (QStringList::Iterator i = groups.begin();
-                             i != groups.end();
-                             ++i) {
-    if ((*i).isEmpty() || *i == "<default>") continue;
+  for (QStringList::Iterator i = groups.begin(); i != groups.end(); ++i) {
+    if ((*i).isEmpty() || *i == "<default>" || *i == "General") continue;
     policies->setGroup(*i);
-    new OtherCertItem(otherSSLBox, *i,
-                      policies->readBoolEntry("Permanent", true),
-                      policies->readNumEntry("Policy", 3),
-                      policies->readDateTimeEntry("Expires"), this );
+    KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
+    if (cert) {
+      new OtherCertItem(otherSSLBox, cert->getSubject(), *i,
+                        policies->readBoolEntry("Permanent", true),
+                        policies->readNumEntry("Policy", 3),
+                        policies->readDateTimeEntry("Expires"), this );
+      delete cert;
+    }
   }
 
   groups = pcerts->groupList();
 
   yourSSLBox->clear();
-  for (QStringList::Iterator i = groups.begin();
-                             i != groups.end();
-                             ++i) {
+  for (QStringList::Iterator i = groups.begin(); i != groups.end(); ++i) {
     if ((*i).isEmpty() || *i == "<default>") continue;
     pcerts->setGroup(*i);
     YourCertItem *j = new YourCertItem(yourSSLBox,
@@ -986,7 +988,7 @@ void KCryptoConfig::load()
     else if (authcfg->readBoolEntry("prompt", false) == true)
        aa = KSSLCertificateHome::AuthPrompt;
     HostAuthItem *j = new HostAuthItem(hostAuthList,
-                                       *i,
+                                       KResolver::domainToAscii(*i),
                                        authcfg->readEntry("certificate"),
                                        this );
     j->setAction(aa);
@@ -1101,7 +1103,6 @@ void KCryptoConfig::save()
                                    i18n("SSLv3 Ciphers"));
   // SSL Policies code
   for (OtherCertItem *x = otherCertDelList.first(); x != 0; x = otherCertDelList.next()) {
-     //policies->deleteGroup(x->configName());
      KSSLX509Map cert(x->configName());
      QString thisCN = cert.getValue("CN");
      _cc.removeByCN(thisCN);
@@ -1195,7 +1196,7 @@ void KCryptoConfig::save()
         static_cast<HostAuthItem *>(hostAuthList->firstChild());
                                                               x;
              x = static_cast<HostAuthItem *>(x->nextSibling())) {
-     authcfg->setGroup(x->configName());
+     authcfg->setGroup(KResolver::domainToAscii(x->configName()));
      authcfg->writeEntry("certificate", x->getCertName());
      authcfg->writeEntry("prompt", (x->getAction() == KSSLCertificateHome::AuthPrompt));
      authcfg->writeEntry("send", (x->getAction() == KSSLCertificateHome::AuthSend));
@@ -1260,41 +1261,25 @@ void KCryptoConfig::defaults()
   emit changed(true);
 }
 
-QString KCryptoConfig::quickHelp() const
-{
-  return i18n("<h1>Crypto</h1> This module allows you to configure SSL for"
-     " use with most KDE applications, as well as manage your personal"
-     " certificates and the known certificate authorities.");
-}
-
-#if !KDE_IS_VERSION( 3, 2, 90 )
-const KAboutData* KCryptoConfig::aboutData() const {
-  return ::aboutData();
-}
-#endif
-
-
 void KCryptoConfig::genCAList() {
-
    _signers->regenerate();
 
 }
 
 void KCryptoConfig::slotSelectCipher(int id) {
-
     switch(id) {
-		case 1:
-			cwCompatible();
-			break;
-		case 2:
-			cwUS();
-			break;
-		case 3:
-			cwExp();
-			break;
-		case 4:
-			cwAll();
-	}
+	    case 1:
+		    cwCompatible();
+		    break;
+	    case 2:
+		    cwUS();
+		    break;
+	    case 3:
+		    cwExp();
+		    break;
+	    case 4:
+		    cwAll();
+    }
 }
 
 void KCryptoConfig::cwCompatible() {
@@ -1379,7 +1364,7 @@ void KCryptoConfig::cwAll() {
 void KCryptoConfig::slotExportCert() {
 OtherCertItem *x = static_cast<OtherCertItem *>(otherSSLBox->selectedItem());
    if (x) {
-     policies->setGroup(x->getSub());
+     policies->setGroup(x->getMD5());
      KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
      if (cert) {
         KCertExport kce;
@@ -1412,7 +1397,7 @@ void KCryptoConfig::slotVerifyCert() {
 OtherCertItem *x = static_cast<OtherCertItem *>(otherSSLBox->selectedItem());
   if (!x) return;
 
-  policies->setGroup(x->getSub());
+  policies->setGroup(x->getMD5());
   KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
 
   if (!cert) {
@@ -1521,7 +1506,7 @@ QString iss = QString::null;
       cacheGroup->setEnabled(true);
       cachePerm->setEnabled(true);
       cacheUntil->setEnabled(true);
-      policies->setGroup(x->getSub());
+      policies->setGroup(x->getMD5());
 
       KSSLCertificate *cert = KSSLCertificate::fromString(policies->readEntry("Certificate", QString::null).local8Bit());
 
@@ -1618,7 +1603,7 @@ TryImportPassAgain:
    cert = KSSLPKCS12::loadCertFile(certFile, QString(pass));
 
    if (!cert) {
-      rc = KMessageBox::warningYesNo(this, i18n("The certificate file could not be loaded. Try a different password?"), i18n("SSL"));
+      rc = KMessageBox::warningYesNo(this, i18n("The certificate file could not be loaded. Try a different password?"), i18n("SSL"),i18n("Try"),i18n("Do Not Try"));
       if (rc == KMessageBox::Yes) goto TryImportPassAgain;
       return;
    }
@@ -1636,8 +1621,8 @@ TryImportPassAgain:
                                                             i;
              i = static_cast<YourCertItem *>(i->nextSibling())) {
       if (i->configName() == name) {
-         rc = KMessageBox::warningYesNo(this, i18n("A certificate with that name already exists. Are you sure that you wish to replace it?"), i18n("SSL"));
-         if (rc == KMessageBox::No) {
+         rc = KMessageBox::warningContinueCancel(this, i18n("A certificate with that name already exists. Are you sure that you wish to replace it?"), i18n("SSL"), i18n("Replace"));
+         if (rc == KMessageBox::Cancel) {
             delete cert;
             return;
          }
@@ -1764,26 +1749,27 @@ QString iss;
    iss = pkcs->getCertificate()->getIssuer();
    ySubject->setValues(x ? x->getName() : QString(QString::null));
    yIssuer->setValues(iss);
-         QPalette cspl;
-         KSSLCertificate *cert = pkcs->getCertificate();
-         cspl = yValidFrom->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) < cert->getQDTNotBefore()) {
-            cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
-         } else {
-            cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
-         }
-         yValidFrom->setPalette(cspl);
+   QPalette cspl;
+   KSSLCertificate *cert = pkcs->getCertificate();
+   cspl = yValidFrom->palette();
+   if (QDateTime::currentDateTime(Qt::UTC) < cert->getQDTNotBefore()) {
+	   cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
+   } else {
+	   cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
+   }
+   yValidFrom->setPalette(cspl);
 
-         cspl = yValidUntil->palette();
-         if (QDateTime::currentDateTime(Qt::UTC) > cert->getQDTNotAfter()) {
-            cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
-         } else {
-            cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
-         }
-         yValidUntil->setPalette(cspl);
+   cspl = yValidUntil->palette();
+   if (QDateTime::currentDateTime(Qt::UTC) > cert->getQDTNotAfter()) {
+	   cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
+   } else {
+	   cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
+   }
+   yValidUntil->setPalette(cspl);
 
-         yValidFrom->setText(cert->getNotBefore());
-         yValidUntil->setText(cert->getNotAfter());
+   yValidFrom->setText(cert->getNotBefore());
+   yValidUntil->setText(cert->getNotAfter());
+   yHash->setText(cert->getMD5DigestText());
    yourSSLUnlock->setEnabled(false);
    delete pkcs;
 }
@@ -1857,17 +1843,21 @@ QCString oldpass = "";
    }
 
    if (pkcs) {
-      QCString pass;
       x->setPassCache(oldpass);
       slotYourUnlock();
-      int i = KPasswordDialog::getNewPassword(pass,
-                                   i18n("Enter the new certificate password"));
+      KPasswordDialog *kpd = new KPasswordDialog(KPasswordDialog::NewPassword, false, 0, this);
+      kpd->setPrompt(i18n("Enter the new certificate password"));
+      kpd->setAllowEmptyPasswords(true);
+
+      int i = kpd->exec();
       if (i == KPasswordDialog::Accepted) {
+         QCString pass = kpd->password();
          pkcs->changePassword(QString(oldpass), QString(pass));
          x->setPKCS(pkcs->toString());
          x->setPassCache(pass);
          configChanged();
       }
+      delete kpd;
       delete pkcs;
    }
 }
@@ -2047,7 +2037,7 @@ void KCryptoConfig::slotCAImport() {
 
 void KCryptoConfig::offerImportToKMail( const QString& certFile )
 {
-    if ( KMessageBox::questionYesNo( this, i18n( "Do you want to make this certificate available to KMail as well?" ) ) == KMessageBox::Yes ) {
+    if ( KMessageBox::questionYesNo( this, i18n( "Do you want to make this certificate available to KMail as well?" ), QString::null, i18n("Make Available"), i18n("Do Not Make Available") ) == KMessageBox::Yes ) {
        KProcess proc;
        proc << "kleopatra";
        proc << "--import-certificate";
@@ -2071,8 +2061,8 @@ CAItem *x = static_cast<CAItem *>(caList->selectedItem());
 
 void KCryptoConfig::slotCARestore() {
 
- int rc = KMessageBox::warningYesNo(this, i18n("This will revert your certificate signers database to the KDE default.\nThis operation cannot be undone.\nAre you sure you wish to continue?"), i18n("SSL"));
-      if (rc == KMessageBox::No) {
+ int rc = KMessageBox::warningContinueCancel(this, i18n("This will revert your certificate signers database to the KDE default.\nThis operation cannot be undone.\nAre you sure you wish to continue?"), i18n("SSL"),i18n("Revert"));
+      if (rc == KMessageBox::Cancel) {
           return;
       }
 
