@@ -18,7 +18,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include <stdlib.h>
@@ -340,6 +340,7 @@ bool KShortURIFilter::filterURI( KURIFilterData& data ) const
   if( isLocalFullPath && !exists )
   {
     exists = ( stat( QFile::encodeName(path).data() , &buff ) == 0 );
+
     if ( !exists ) {
       // Support for name filter (/foo/*.txt), see also KonqMainWindow::detectNameFilter
       // If the app using this filter doesn't support it, well, it'll simply error out itself
@@ -513,8 +514,12 @@ QString KShortURIFilter::configName() const
 void KShortURIFilter::configure()
 {
   KConfig config( name() + QFL1("rc"), false, false );
-  m_strDefaultProtocol = config.readEntry( "DefaultProtocol", QFL1("http://") );
   m_bVerbose = config.readBoolEntry( "Verbose", false );
+
+  if ( m_bVerbose )
+    kdDebug() << "KShortURIFilter::configure: Config reload request..." << endl;
+
+  m_strDefaultProtocol = config.readEntry( "DefaultProtocol", QFL1("http://") );
   EntryMap patterns = config.entryMap( QFL1("Pattern") );
   const EntryMap protocols = config.entryMap( QFL1("Protocol") );
   config.setGroup("Type");
