@@ -6,29 +6,31 @@
 //
 // Ported to kcontrol2 by Geert Jansen.
 
-#include <qlayout.h>
+#include <qcheckbox.h>
+#include <qcombobox.h>
+#include <qdir.h>
+#include <qgroupbox.h>
 #include <qhbox.h>
 #include <qlabel.h>
-#include <qwhatsthis.h>
-#include <qtooltip.h>
-#include <qcheckbox.h>
-#include <qsettings.h>
-#include <qcombobox.h>
-#include <qgroupbox.h>
-#include <knuminput.h>
+#include <qlayout.h>
 #include <qpushbutton.h>
-#include <qdir.h>
+#include <qsettings.h>
+#include <qtooltip.h>
+#include <qwhatsthis.h>
 
 #include <dcopclient.h>
-#include <kapplication.h>
-#include <ksimpleconfig.h>
-#include <kipc.h>
-#include <kstandarddirs.h>
-#include <kmessagebox.h>
-#include <kgenericfactory.h>
-#include <stdlib.h>
-#include "../krdb/krdb.h"
 
+#include <kaccelmanager.h>
+#include <kapplication.h>
+#include <kgenericfactory.h>
+#include <kipc.h>
+#include <kmessagebox.h>
+#include <knuminput.h>
+#include <ksimpleconfig.h>
+#include <kstandarddirs.h>
+#include <stdlib.h>
+
+#include "../krdb/krdb.h"
 #include "fonts.h"
 #include "fonts.moc"
 
@@ -110,8 +112,7 @@ static const char *aa_vbgr_xpm[]={
 "aaaaaaaaaaaa",
 "aaaaaaaaaaaa"};
 
-static QPixmap aaPixmaps[]={ QPixmap(aa_rgb_xpm), QPixmap(aa_bgr_xpm),
-                             QPixmap(aa_vrgb_xpm), QPixmap(aa_vbgr_xpm) };
+static QPixmap aaPixmaps[]={ QPixmap(aa_rgb_xpm), QPixmap(aa_bgr_xpm), QPixmap(aa_vrgb_xpm), QPixmap(aa_vbgr_xpm) };
 
 /**** DLL Interface ****/
 typedef KGenericFactory<KFonts, QWidget> FontFactory;
@@ -134,6 +135,7 @@ FontUseItem::FontUseItem(
     _rckey(key),
     _default(default_fnt)
 {
+  KAcceleratorManager::setNoAccel( this );
   setTitle( name );
   readFont();
 }
@@ -205,7 +207,7 @@ FontAASettings::FontAASettings(QWidget *parent)
                 changesMade(false)
 {
   QWidget     *mw=new QWidget(this);
-  QGridLayout *layout=new QGridLayout(mw, 1, 1, KDialog::marginHint(), KDialog::spacingHint());
+  QGridLayout *layout=new QGridLayout(mw, 1, 1, 0, KDialog::spacingHint());
 
   excludeRange=new QCheckBox(i18n("E&xclude range:"), mw),
   layout->addWidget(excludeRange, 0, 0);
@@ -265,7 +267,7 @@ FontAASettings::FontAASettings(QWidget *parent)
 #endif
 }
 
-void FontAASettings::load()
+bool FontAASettings::load()
 {
   double     from, to;
 #ifdef HAVE_FONTCONFIG
@@ -323,9 +325,11 @@ void FontAASettings::load()
 #endif
 
   enableWidgets();
+
+  return xft.getAntiAliasing();
 }
 
-bool FontAASettings::save()
+bool FontAASettings::save( bool useAA )
 {
 #ifdef HAVE_FONTCONFIG
   KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType|KXftConfig::HintStyle);
@@ -333,9 +337,10 @@ bool FontAASettings::save()
   KXftConfig xft(KXftConfig::ExcludeRange|KXftConfig::SubPixelType);
 #endif
   KConfig    kglobals("kdeglobals", false, false);
-
   kglobals.setGroup("General");
 
+  xft.setAntiAliasing( useAA );
+
   if(excludeRange->isChecked())
     xft.setExcludeRange(excludeFrom->value(), excludeTo->value());
   else
@@ -486,8 +491,7 @@ static QCString desktopConfigName()
 }
 
 KFonts::KFonts(QWidget *parent, const char *name, const QStringList &)
-    :   KCModule(FontFactory::instance(), parent, name),
-        _changed(false)
+    :   KCModule(FontFactory::instance(), parent, name)
 {
   QStringList nameGroupKeyRc;
 
@@ -504,17 +508,17 @@ KFonts::KFonts(QWidget *parent, const char *name, const QStringList &)
 
   // Keep in sync with kdelibs/kdecore/kglobalsettings.cpp
 
-  QFont f0("helvetica", 12);
-  QFont f1("courier", 12);
-  QFont f2("helvetica", 10);
-  QFont f3("helvetica", 12, QFont::Bold);
-  QFont f4("helvetica", 11);
+  QFont f0("Sans Serif", 10);
+  QFont f1("Monospace", 10);
+  QFont f2("Sans Serif", 10);
+  QFont f3("Sans Serif", 9, QFont::Bold);
+  QFont f4("Sans Serif", 10);
 
-  f0.setPointSize(12);
-  f1.setPointSize(12);
+  f0.setPointSize(10);
+  f1.setPointSize(10);
   f2.setPointSize(10);
-  f3.setPointSize(12);
-  f4.setPointSize(11);
+  f3.setPointSize(9);
+  f4.setPointSize(10);
 
   defaultFontList << f0 << f1 << f2 << f0 << f3 << f4 << f0;
 
@@ -613,11 +617,7 @@ KFonts::KFonts(QWidget *parent, const char *name, const QStringList &)
 
    connect(cbAA, SIGNAL(clicked()), SLOT(slotUseAntiAliasing()));
 
-   useAA = QSettings().readBoolEntry("/qt/useXft");
-   useAA_original = useAA;
-
-   cbAA->setChecked(useAA);
-   aaSettingsButton->setEnabled(useAA);
+   load();
 }
 
 KFonts::~KFonts()
@@ -628,7 +628,6 @@ KFonts::~KFonts()
 
 void KFonts::fontSelected()
 {
-  _changed = true;
   emit changed(true);
 }
 
@@ -637,49 +636,26 @@ void KFonts::defaults()
   for ( int i = 0; i < (int) fontUseList.count(); i++ )
     fontUseList.at( i )->setDefault();
 
-  useAA = false;
+  useAA = true;
   cbAA->setChecked(useAA);
   aaSettings->defaults();
-  _changed = true;
   emit changed(true);
 }
 
-QString KFonts::quickHelp() const
-{
-    return i18n( "<h1>Fonts</h1> This module allows you to choose which"
-      " fonts will be used to display text in KDE. You can select not only"
-      " the font family (for example, <em>helvetica</em> or <em>times</em>),"
-      " but also the attributes that make up a specific font (for example,"
-      " <em>bold</em> style and <em>12 points</em> in height.)<p>"
-      " Just click the \"Choose\" button that is next to the font you want to"
-      " change."
-      " You can ask KDE to try and apply font and color settings to non-KDE"
-      " applications as well. See the \"Style\" control module for more"
-			" information.");
-}
-
 void KFonts::load()
 {
   for ( uint i = 0; i < fontUseList.count(); i++ )
     fontUseList.at( i )->readFont();
 
-  useAA = QSettings().readBoolEntry("/qt/useXft");
-  useAA_original = useAA;
+  useAA_original = useAA = aaSettings->load();
   kdDebug(1208) << "AA:" << useAA << endl;
   cbAA->setChecked(useAA);
 
-  aaSettings->load();
-
-  _changed = true;
   emit changed(false);
 }
 
 void KFonts::save()
 {
-  if ( !_changed )
-    return;
-
-  _changed = false;
 
   for ( FontUseItem* i = fontUseList.first(); i; i = fontUseList.next() )
       i->writeFont();
@@ -698,16 +674,11 @@ void KFonts::save()
   config->sync();
   delete config;
 
-  QSettings().writeEntry("/qt/useXft", useAA);
-
-  if (useAA)
-    QSettings().writeEntry("/qt/enableXft", useAA);
-
   KIPC::sendMessageAll(KIPC::FontChanged);
 
   kapp->processEvents(); // Process font change ourselves
 
-  if(aaSettings->save() || (useAA != useAA_original) ) {
+  if(aaSettings->save( useAA ) || (useAA != useAA_original) ) {
     KMessageBox::information(this,
       i18n(
         "<p>You have changed anti-aliasing related settings. This change will only affect newly started applications.</p>"
@@ -720,10 +691,6 @@ void KFonts::save()
   emit changed(false);
 }
 
-int KFonts::buttons()
-{
-  return KCModule::Help | KCModule::Default | KCModule::Apply;
-}
 
 void KFonts::slotApplyFontDiff()
 {
@@ -735,7 +702,6 @@ void KFonts::slotApplyFontDiff()
   {
     for ( int i = 0; i < (int) fontUseList.count(); i++ )
       fontUseList.at( i )->applyFontDiff( font,fontDiffFlags );
-    _changed = true;
     emit changed(true);
   }
 }
@@ -743,7 +709,6 @@ void KFonts::slotApplyFontDiff()
 void KFonts::slotUseAntiAliasing()
 {
     useAA = cbAA->isChecked();
-    _changed = true;
     emit changed(true);
 }
 
@@ -751,7 +716,6 @@ void KFonts::slotCfgAa()
 {
   if(aaSettings->exec())
   {
-    _changed = true;
     emit changed(true);
   }
 }
