@@ -20,7 +20,7 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 ////////////////////////////////////////////////////////////////////////////////
 // (C) Craig Drummond, 2001, 2002, 2003, 2004
@@ -29,6 +29,9 @@
 #include "XConfig.h"
 #include "FontEngine.h"
 #include "kxftconfig.h"
+#include <ksavefile.h>
+#include <qdatastream.h>
+#include <qtextstream.h>
 #include <fstream>
 #include <string.h>
 #include <qdir.h>
@@ -45,8 +48,6 @@ extern "C" unsigned int kfi_getPid(const char *proc, unsigned int ppid);
 
 #define UNSCALED ":unscaled"
 
-using namespace std;
-
 namespace KFI
 {
 
@@ -397,8 +398,8 @@ static char * getItem(char **start, char **end, const char *key, unsigned int &s
 
 bool CXConfig::processX11(bool read)
 {
-    ifstream x11(QFile::encodeName(itsFileName));
-    bool     ok=false;
+    std::ifstream x11(QFile::encodeName(itsFileName));
+    bool          ok=false;
 
     if(x11)
     {
@@ -406,8 +407,8 @@ bool CXConfig::processX11(bool read)
 
         bool closed=false;
 
-        x11.seekg(0, ios::end);
-        unsigned int size=(streamoff) x11.tellg();
+        x11.seekg(0, std::ios::end);
+        unsigned int size=(std::streamoff) x11.tellg();
 
         if(read)
             itsPaths.clear();
@@ -418,7 +419,7 @@ bool CXConfig::processX11(bool read)
 
             if(buffer)
             {
-                x11.seekg(0, ios::beg);
+                x11.seekg(0, std::ios::beg);
                 x11.read(buffer, size);
 
                 if(x11.good())
@@ -454,9 +455,10 @@ bool CXConfig::processX11(bool read)
                         {
                             Misc::createBackup(itsFileName);
 
-                            ofstream of(QFile::encodeName(itsFileName));
+                            KSaveFile   out(itsFileName);
+                            QDataStream *stream=out.dataStream();
 
-                            if(of)
+                            if(stream)
                             {
                                 char  *from=buffer,
                                       *modStart=NULL,
@@ -476,39 +478,38 @@ bool CXConfig::processX11(bool read)
 
                                 if(!foundFt && modStart && modEnd && modStart<filesStart) // Then write mod section first...
                                 {
-                                    of.write(from, modEnd-from);
+                                    stream->writeBytes(from, modEnd-from);
                                     if(!foundFt)
-                                        of << "    Load \"freetype\"\n";    // CPD TODO: Which is better xtt of freetype? Perhaps check locale?
-                                    of.write(modEnd, endSectionMarkerLen);
+                                        *stream << "    Load \"freetype\"\n";    // CPD TODO: Which is better xtt of freetype? Perhaps check locale?
+                                    stream->writeBytes(modEnd, endSectionMarkerLen);
                                     from=modEnd+endSectionMarkerLen;
                                 }
 
-                                of.write(from, filesEnd-from);
+                                stream->writeBytes(from, filesEnd-from);
 
                                 for(path=itsPaths.first(); path; path=itsPaths.next())
                                     if(TPath::DIR!=path->type || Misc::dExists(path->dir))
                                     {
-                                        of << "    FontPath \t\"";
-                                        of << QFile::encodeName(Misc::xDirSyntax(path->dir));
+                                        *stream << "    FontPath \t\"";
+                                        *stream << QFile::encodeName(Misc::xDirSyntax(path->dir));
                                         if(path->unscaled)
-                                            of << UNSCALED;
-                                        of << "\"\n";
+                                            *stream << UNSCALED;
+                                        *stream << "\"\n";
                                     }
 
-                                of.write(filesEnd, endSectionMarkerLen);
+                                stream->writeBytes(filesEnd, endSectionMarkerLen);
                                 from=filesEnd+endSectionMarkerLen;
 
                                 if(!foundFt && modStart && modEnd && modStart>filesStart) // Then write mod section last...
                                 {
-                                    of.write(from, modEnd-from);
+                                    stream->writeBytes(from, modEnd-from);
                                     if(!foundFt)
-                                        of << "    Load \"freetype\"\n";
-                                    of.write(modEnd, endSectionMarkerLen);
+                                        *stream << "    Load \"freetype\"\n";
+                                    stream->writeBytes(modEnd, endSectionMarkerLen);
                                     from=modEnd+endSectionMarkerLen;
                                 }
                                 if(((unsigned int)(from-buffer))<size)
-                                    of.write(from, size-(from-buffer));
-                                of.close();
+                                    stream->writeBytes(from, size-(from-buffer));
                                 ok=true;
                             }
                         }
@@ -612,8 +613,8 @@ static char * getXfsPath(char *buffer, unsigned int &totalSize, unsigned int off
 
 bool CXConfig::processXfs(bool read)
 {
-    ifstream xfs(QFile::encodeName(itsFileName));
-    bool     ok=false;
+    std::ifstream xfs(QFile::encodeName(itsFileName));
+    bool          ok=false;
  
     if(xfs)
     {
@@ -621,8 +622,8 @@ bool CXConfig::processXfs(bool read)
 
         bool closed=false;
  
-        xfs.seekg(0, ios::end);
-        unsigned int size= (streamoff) xfs.tellg();
+        xfs.seekg(0, std::ios::end);
+        unsigned int size= (std::streamoff) xfs.tellg();
 
         if(read)
             itsPaths.clear();
@@ -633,7 +634,7 @@ bool CXConfig::processXfs(bool read)
  
             if(buffer)
             {
-                xfs.seekg(0, ios::beg);
+                xfs.seekg(0, std::ios::beg);
                 xfs.read(buffer, size);
  
                 if(xfs.good())
@@ -693,30 +694,30 @@ bool CXConfig::processXfs(bool read)
                                                     {
                                                         Misc::createBackup(itsFileName);
 
-                                                        ofstream of(QFile::encodeName(itsFileName));
- 
-                                                        if(of)
+                                                        KSaveFile   out(itsFileName);
+                                                        QDataStream *stream=out.dataStream();
+
+                                                        if(stream)
                                                         {
                                                             bool  first=true;
                                                             TPath *p=NULL;
 
-                                                            of.write(buffer, cat-buffer);
-                                                            of << ' ';
+                                                            stream->writeBytes(buffer, cat-buffer);
+                                                            *stream << ' ';
                                                             for(p=itsPaths.first(); p; p=itsPaths.next())
                                                                 if(Misc::dExists(p->dir))
                                                                 {
                                                                     if(!first)
                                                                     {
-                                                                        of << ',';
-                                                                        of << endl;
+                                                                        *stream << ',';
+                                                                        *stream << '\n';
                                                                     }
-                                                                    of << QFile::encodeName(Misc::xDirSyntax(p->dir));
+                                                                    *stream << QFile::encodeName(Misc::xDirSyntax(p->dir));
                                                                     if(p->unscaled)
-                                                                        of << UNSCALED;
+                                                                        *stream << UNSCALED;
                                                                     first=false;
                                                                 }
-                                                            of.write(cat, size-(cat-buffer));
-                                                            of.close();
+                                                            stream->writeBytes(cat, size-(cat-buffer));
                                                             ok=true;
                                                         }
                                                     }
@@ -848,12 +849,6 @@ bool CXConfig::createFontsDotDir(const QString &dir, CFontEngine &fe)
                                              if(-1==fdir.findIndex(entry))
                                                  fdir.append(entry);
                                              addedFd=addedFs=true;
-#ifndef HAVE_FONTCONFIG
-                                             if(CFontEngine::isATtf(QFile::encodeName(fInfo->fileName())) &&
-                                                CEncodings::constTTSymbol==*it &&
-                                                !symbolFamilies.contains(family))
-                                                 symbolFamilies.append(family);
-#endif
                                          }
                                      }
                                 }
@@ -880,17 +875,17 @@ bool CXConfig::createFontsDotDir(const QString &dir, CFontEngine &fe)
         // removed)
         if(addedFd || fdir.count()!=fd.xlfdCount() || (0==fdir.count() && 0!=fd.xlfdCount()))
         {
-            ofstream fontsDotDir(QFile::encodeName(QString(ds+"fonts.dir")));
 
-            if(fontsDotDir)
+            KSaveFile   out(ds+"fonts.dir");
+            QTextStream *stream=out.textStream();
+
+            if(stream)
             {
                 QStringList::Iterator sIt;
 
-                fontsDotDir << fdir.count() << endl;
+                *stream << fdir.count() << endl;
                 for(sIt=fdir.begin(); sIt!=fdir.end(); ++sIt)
-                    fontsDotDir << (*sIt).local8Bit() << endl;
-
-                fontsDotDir.close();
+                    *stream << (*sIt).local8Bit() << endl;
             }
             else
                 status=false;
@@ -898,17 +893,16 @@ bool CXConfig::createFontsDotDir(const QString &dir, CFontEngine &fe)
 
         if(status && (addedFs || fscale.count()!=fs.xlfdCount()))
         {
-            ofstream fontsDotScale(QFile::encodeName(QString(ds+"fonts.scale")));
+            KSaveFile   out(ds+"fonts.scale");
+            QTextStream *stream=out.textStream();
 
-            if(fontsDotScale)
+            if(stream)
             {
                 QStringList::Iterator sIt;
 
-                fontsDotScale << fscale.count() << endl;
+                *stream << fscale.count() << endl;
                 for(sIt=fscale.begin(); sIt!=fscale.end(); ++sIt)
-                    fontsDotScale << (*sIt).local8Bit() << endl;
-
-                fontsDotScale.close();
+                    *stream << (*sIt).local8Bit() << endl;
             }
             else
                 status=false;
@@ -934,7 +928,7 @@ CXConfig::TPath::EType CXConfig::TPath::getType(const QString &d)
 CXConfig::CFontsFile::CFontsFile(const char *file)
                     : itsXlfdCount(0)
 {
-    ifstream f(file);
+    std::ifstream f(file);
 
     itsEntries.setAutoDelete(true);
 
