@@ -15,20 +15,20 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  */
+
 #include <stdlib.h>
 
 #include <qbuttongroup.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
-#include <qheader.h>
 #include <qlabel.h>
 #include <qpushbutton.h>
-#include <qradiobutton.h>
 #include <qslider.h>
 #include <qtooltip.h>
 #include <qtimer.h>
 
 #include <kapplication.h>
+#include <kdebug.h>
 #include <klocale.h>
 #include <knuminput.h>
 #include <kpanelextension.h>
@@ -36,7 +36,9 @@
 #include <kstandarddirs.h>
 #include <kwin.h>
 
+#include "main.h"
 #include "../background/bgrender.h"
+
 #include "positiontab_impl.h"
 #include "positiontab_impl.moc"
 
@@ -48,11 +50,10 @@ extern const int maxX = 150;
 extern const int maxY = 114;
 extern const int margin = 1;
 
-PositionTab::PositionTab(KickerConfig *kcmKicker, const char* name)
-  : PositionTabBase(kcmKicker, name),
+PositionTab::PositionTab(QWidget *parent, const char* name)
+  : PositionTabBase(parent, name),
     m_pretendPanel(0),
     m_desktopPreview(0),
-    m_kcm(kcmKicker),
     m_panelInfo(0),
     m_panelPos(PosBottom),
     m_panelAlign(AlignLeft)
@@ -130,15 +131,30 @@ PositionTab::PositionTab(KickerConfig *kcmKicker, const char* name)
     connect(m_customSlider, SIGNAL(valueChanged(int)), SIGNAL(changed()));
     connect(m_customSpinbox, SIGNAL(valueChanged(int)), SIGNAL(changed()));
 
-    m_desktopPreview = new KBackgroundRenderer(0);
-    connect(m_desktopPreview, SIGNAL(imageDone(int)), SLOT(slotBGPreviewReady(int)));
+    m_desktopPreview = new KVirtualBGRenderer(0);
+    connect(m_desktopPreview, SIGNAL(imageDone(int)),
+            SLOT(slotBGPreviewReady(int)));
 
-    connect(m_kcm, SIGNAL(extensionInfoChanged()), SLOT(infoUpdated()));
-    connect(m_kcm, SIGNAL(extensionAdded(extensionInfo*)), SLOT(extensionAdded(extensionInfo*)));
-    connect(m_kcm, SIGNAL(extensionChanged(const QString&)), SLOT(extensionChanged(const QString&)));
-    connect(m_kcm, SIGNAL(extensionAboutToChange(const QString&)), SLOT(extensionAboutToChange(const QString&)));
-    connect(m_panelSize, SIGNAL(activated(int)), SLOT(sizeChanged(int)));
-    connect(m_panelSize, SIGNAL(activated(int)), SIGNAL(changed()));
+    connect(KickerConfig::the(), SIGNAL(extensionInfoChanged()),
+            SLOT(infoUpdated()));
+    connect(KickerConfig::the(), SIGNAL(extensionAdded(ExtensionInfo*)),
+            SLOT(extensionAdded(ExtensionInfo*)));
+    connect(KickerConfig::the(), SIGNAL(extensionRemoved(ExtensionInfo*)),
+            SLOT(extensionRemoved(ExtensionInfo*)));
+    connect(KickerConfig::the(), SIGNAL(extensionChanged(const QString&)),
+            SLOT(extensionChanged(const QString&)));
+    connect(KickerConfig::the(), SIGNAL(extensionAboutToChange(const QString&)),
+            SLOT(extensionAboutToChange(const QString&)));
+    // position tab tells hiding tab about extension selections and vice versa
+    connect(KickerConfig::the(), SIGNAL(hidingPanelChanged(int)),
+            SLOT(jumpToPanel(int)));
+    connect(m_panelList, SIGNAL(activated(int)),
+            KickerConfig::the(), SIGNAL(positionPanelChanged(int)));
+
+    connect(m_panelSize, SIGNAL(activated(int)),
+            SLOT(sizeChanged(int)));
+    connect(m_panelSize, SIGNAL(activated(int)),
+            SIGNAL(changed()));
 }
 
 PositionTab::~PositionTab()
@@ -149,15 +165,15 @@ PositionTab::~PositionTab()
 void PositionTab::load()
 {
     m_panelInfo = 0;
-    m_kcm->populateExtensionInfoList(m_panelList);
+    KickerConfig::the()->populateExtensionInfoList(m_panelList);
     m_panelsGroupBox->setHidden(m_panelList->count() < 2);
 
-    switchPanel(0);
+    switchPanel(KickerConfig::the()->currentPanelIndex());
     m_desktopPreview->setPreview(m_pretendDesktop->size());
     m_desktopPreview->start();
 }
 
-void PositionTab::extensionAdded(extensionInfo* info)
+void PositionTab::extensionAdded(ExtensionInfo* info)
 {
     m_panelList->insertItem(info->_name);
     m_panelsGroupBox->setHidden(m_panelList->count() < 2);
@@ -166,7 +182,7 @@ void PositionTab::extensionAdded(extensionInfo* info)
 void PositionTab::save()
 {
     storeInfo();
-    m_kcm->saveExtentionInfo();
+    KickerConfig::the()->saveExtentionInfo();
 }
 
 void PositionTab::defaults()
@@ -192,13 +208,14 @@ void PositionTab::defaults()
 
     // update the magic drawing
     lengthenPanel(-1);
-    switchPanel(0);
+    switchPanel(KickerConfig::the()->currentPanelIndex());
 }
 
 void PositionTab::sizeChanged(int which)
 {
-    m_customSlider->setEnabled(which == KPanelExtension::SizeCustom);
-    m_customSpinbox->setEnabled(which == KPanelExtension::SizeCustom);
+    bool custom = which == KPanelExtension::SizeCustom;
+    m_customSlider->setEnabled(custom);
+    m_customSpinbox->setEnabled(custom);
 }
 
 void PositionTab::movePanel(int whichButton)
@@ -292,7 +309,7 @@ void PositionTab::movePanel(int whichButton)
 	   setPositionButtons();
 	   return;
 	}
-        m_panelAlign = AlignRight;
+        m_panelAlign = kapp->reverseLayout() ? AlignLeft : AlignRight;
         m_panelPos = PosBottom;
     }
     else if (pushed == locationRightTop)
@@ -470,6 +487,8 @@ void PositionTab::panelDimensionsChanged()
 
 void PositionTab::slotBGPreviewReady(int)
 {
+    m_pretendDesktop->setBackgroundPixmap(*m_desktopPreview->pixmap());
+#if 0
     KPixmap pm;
     if (QPixmap::defaultDepth() < 15)
     {
@@ -481,17 +500,18 @@ void PositionTab::slotBGPreviewReady(int)
     }
 
     m_pretendDesktop->setBackgroundPixmap(pm);
+#endif
 }
 
 void PositionTab::switchPanel(int panelItem)
 {
     blockSignals(true);
-    extensionInfo* panelInfo = (m_kcm->extensionsInfo())[panelItem];
+    ExtensionInfo* panelInfo = (KickerConfig::the()->extensionsInfo())[panelItem];
 
     if (!panelInfo)
     {
         m_panelList->setCurrentItem(0);
-        panelInfo = (m_kcm->extensionsInfo())[panelItem];
+        panelInfo = (KickerConfig::the()->extensionsInfo())[panelItem];
 
         if (!panelInfo)
         {
@@ -506,30 +526,26 @@ void PositionTab::switchPanel(int panelItem)
 
     m_panelInfo = panelInfo;
 
-    m_panelSize->removeItem(4);
-    if (m_panelInfo->_customSizeMin == m_panelInfo->_customSizeMax)
+    // because this changes when panels come and go, we have 
+    // to be overly pedantic and remove the custom item every time and
+    // decide to add it back again, or not
+    m_panelSize->removeItem(KPanelExtension::SizeCustom);
+    if (m_panelInfo->_customSizeMin != m_panelInfo->_customSizeMax)
     {
-        m_customSlider->setEnabled(false);
-        m_customSpinbox->setEnabled(false);
-    }
-    else
-    {
-        m_panelSize->insertItem(i18n("Custom"), 4);
-        m_customSlider->setEnabled(false);
-        m_customSpinbox->setEnabled(false);
+        m_panelSize->insertItem(i18n("Custom"), KPanelExtension::SizeCustom);
     }
 
     if (m_panelInfo->_size >= KPanelExtension::SizeCustom ||
         (!m_panelInfo->_useStdSizes &&
          m_panelInfo->_customSizeMin != m_panelInfo->_customSizeMax)) // compat
     {
-        m_panelSize->setCurrentItem(4);
-        m_customSlider->setEnabled(true);
-        m_customSpinbox->setEnabled(true);
+        m_panelSize->setCurrentItem(KPanelExtension::SizeCustom);
+        sizeChanged(KPanelExtension::SizeCustom);
     }
     else
     {
         m_panelSize->setCurrentItem(m_panelInfo->_size);
+        sizeChanged(0);
     }
     m_panelSize->setEnabled(m_panelInfo->_useStdSizes);
 
@@ -540,7 +556,6 @@ void PositionTab::switchPanel(int panelItem)
     m_customSpinbox->setMinValue(m_panelInfo->_customSizeMin);
     m_customSpinbox->setMaxValue(m_panelInfo->_customSizeMax);
     m_customSpinbox->setValue(m_panelInfo->_customSize);
-
     m_sizeGroup->setEnabled(m_panelInfo->_resizeable);
     m_panelPos = m_panelInfo->_position;
     m_panelAlign = m_panelInfo->_alignment;
@@ -620,7 +635,7 @@ void PositionTab::infoUpdated()
 
 void PositionTab::extensionAboutToChange(const QString& configPath)
 {
-    extensionInfo* extension = (m_kcm->extensionsInfo())[m_panelList->currentItem()];
+    ExtensionInfo* extension = (KickerConfig::the()->extensionsInfo())[m_panelList->currentItem()];
     if (extension && extension->_configPath == configPath)
     {
         storeInfo();
@@ -629,7 +644,7 @@ void PositionTab::extensionAboutToChange(const QString& configPath)
 
 void PositionTab::extensionChanged(const QString& configPath)
 {
-    extensionInfo* extension = (m_kcm->extensionsInfo())[m_panelList->currentItem()];
+    ExtensionInfo* extension = (KickerConfig::the()->extensionsInfo())[m_panelList->currentItem()];
     if (extension && extension->_configPath == configPath)
     {
         m_panelInfo = 0;
@@ -650,7 +665,7 @@ void PositionTab::storeInfo()
     {
         m_panelInfo->_size = m_panelSize->currentItem();
     }
-    else // if (m_sizeCustom->isChecked())
+    else
     {
         m_panelInfo->_size = KPanelExtension::SizeCustom;
         m_panelInfo->_customSize = m_customSlider->value();
@@ -700,14 +715,14 @@ void PositionTab::showIdentify()
     }
 }
 
-void PositionTab::removeExtension(extensionInfo* info)
+void PositionTab::extensionRemoved(ExtensionInfo* info)
 {
     int count = m_panelList->count();
-    int extensionCount = m_kcm->extensionsInfo().count();
+    int extensionCount = KickerConfig::the()->extensionsInfo().count();
     int index = 0;
     for (; index < count && index < extensionCount; ++index)
     {
-        if (m_kcm->extensionsInfo()[index] == info)
+        if (KickerConfig::the()->extensionsInfo()[index] == info)
         {
             break;
         }
@@ -723,3 +738,8 @@ void PositionTab::removeExtension(extensionInfo* info)
     }
 }
 
+void PositionTab::jumpToPanel(int index)
+{
+    m_panelList->setCurrentItem(index);
+    switchPanel(index);
+}
