@@ -1,6 +1,4 @@
 /*
-    $Id: knotify.cpp,v 1.70 2003/11/08 23:43:51 mkretz Exp $
-
     Copyright (C) 2000,2002 Carsten Pfeiffer <pfeiffer@kde.org>
 
     This program is free software; you can redistribute it and/or
@@ -15,14 +13,15 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 
 */
 
-#include <qcheckbox.h>
+#include <qbuttongroup.h>
 #include <qlabel.h>
 #include <qlayout.h>
+#include <qradiobutton.h>
 #include <qslider.h>
 #include <qvbox.h>
 
@@ -39,6 +38,7 @@
 
 
 #include "knotify.h"
+#include "playersettings.h"
 
 static const int COL_FILENAME = 1;
 
@@ -53,6 +53,17 @@ KCMKNotify::KCMKNotify(QWidget *parent, const char *name, const QStringList & )
 {
     setButtons( Help | Default | Apply );
 
+    setQuickHelp( i18n("<h1>System Notifications</h1>"
+                "KDE allows for a great deal of control over how you "
+                "will be notified when certain events occur. There are "
+                "several choices as to how you are notified:"
+                "<ul><li>As the application was originally designed."
+                "<li>With a beep or other noise."
+                "<li>Via a popup dialog box with additional information."
+                "<li>By recording the event in a logfile without "
+                "any additional visual or audible alert."
+                "</ul>"));
+
     QVBoxLayout *layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );
 
     QLabel *label = new QLabel( i18n( "Event source:" ), this );
@@ -73,6 +84,15 @@ KCMKNotify::KCMKNotify(QWidget *parent, const char *name, const QStringList & )
     connect( m_notifyWidget->m_playerButton, SIGNAL( clicked() ),
              SLOT( slotPlayerSettings()));
 
+    KAboutData* ab = new KAboutData(
+        "kcmknotify", I18N_NOOP("KNotify"), "3.0",
+        I18N_NOOP("System Notification Control Panel Module"),
+        KAboutData::License_GPL, "(c) 2002 Carsten Pfeiffer", 0, 0 );
+    ab->addAuthor( "Carsten Pfeiffer", 0, "pfeiffer@kde.org" );
+    ab->addCredit( "Charles Samuels", I18N_NOOP("Original implementation"),
+	       "charles@altair.dhs.org" );
+    setAboutData( ab );
+
     load();
 }
 
@@ -136,10 +156,10 @@ void KCMKNotify::load()
         KGlobal::dirs()->findAllResources("data", "*/eventsrc", false, true );
 
     QStringList::ConstIterator it = fullpaths.begin();
-    for ( ; it != fullpaths.end(); ++it) 
+    for ( ; it != fullpaths.end(); ++it)
         m_notifyWidget->addApplicationEvents( *it );
 
-    ApplicationList& allApps = m_notifyWidget->allApps();
+    ApplicationList allApps = m_notifyWidget->allApps();
     allApps.sort();
     m_notifyWidget->setEnabled( !allApps.isEmpty() );
 
@@ -149,7 +169,7 @@ void KCMKNotify::load()
 
     KConfig config( "knotifyrc", true, false );
     config.setGroup( "Misc" );
-    QString appDesc = config.readEntry( "LastConfiguredApp" );
+    QString appDesc = config.readEntry( "LastConfiguredApp", "KDE System Notifications" );
     if ( !appDesc.isEmpty() )
         m_appCombo->setCurrentItem( appDesc );
 
@@ -171,38 +191,6 @@ void KCMKNotify::save()
     emit changed( false );
 }
 
-QString KCMKNotify::quickHelp() const
-{
-    return i18n("<h1>System Notifications</h1>"
-                "KDE allows for a great deal of control over how you "
-                "will be notified when certain events occur. There are "
-                "several choices as to how you are notified:"
-                "<ul><li>As the application was originally designed."
-                "<li>With a beep or other noise."
-                "<li>Via a popup dialog box with additional information."
-                "<li>By recording the event in a logfile without "
-                "any additional visual or audible alert."
-                "</ul>");
-}
-
-const KAboutData *KCMKNotify::aboutData() const
-{
-    static KAboutData* ab = 0;
-
-    if(!ab)
-    {
-        ab = new KAboutData(
-            "kcmnotify", I18N_NOOP("KNotify"), "3.0",
-            I18N_NOOP("System Notification Control Panel Module"),
-            KAboutData::License_GPL, "(c) 2002 Carsten Pfeiffer", 0, 0 );
-        ab->addAuthor( "Carsten Pfeiffer", 0, "pfeiffer@kde.org" );
-        ab->addCredit( "Charles Samuels", I18N_NOOP("Original implementation"),
-                       "charles@altair.dhs.org" );
-    }
-
-    return ab;
-}
-
 ///////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////
 
@@ -212,45 +200,43 @@ PlayerSettingsDialog::PlayerSettingsDialog( QWidget *parent, bool modal )
 {
     QFrame *frame = makeMainWidget();
 
-    QVBoxLayout *topLayout = new QVBoxLayout( frame, 0, 
+    QVBoxLayout *topLayout = new QVBoxLayout( frame, 0,
         KDialog::spacingHint() );
 
-    QHBoxLayout *hbox = new QHBoxLayout( topLayout, KDialog::spacingHint() );
-    cbExternal = new QCheckBox( i18n("Use e&xternal player: "), frame );
-    reqExternal = new KURLRequester( frame );
-    reqExternal->completionObject()->setMode( KURLCompletion::ExeCompletion );
-    connect( cbExternal, SIGNAL( toggled( bool )),
-             SLOT( externalToggled( bool )));
-    hbox->addWidget( cbExternal );
-    hbox->addWidget( reqExternal );
-
-    hbox = new QHBoxLayout( topLayout, KDialog::spacingHint() );
-    volumeLabel = new QLabel( i18n( "&Volume: " ), frame );
-    volumeSlider = new QSlider( frame );
-    volumeSlider->setOrientation( Horizontal );
-    volumeSlider->setRange( 0, 100 );
-    volumeLabel->setBuddy( volumeSlider );
-    hbox->addWidget( volumeLabel );
-    hbox->addWidget( volumeSlider );
+    m_ui = new PlayerSettingsUI(frame);
+    topLayout->addWidget(m_ui);
 
     load();
     dataChanged = false;
     enableButton(Apply, false);
-    connect( cbExternal, SIGNAL( toggled( bool ) ), this, SLOT( slotChanged() ) );
-    connect( volumeSlider, SIGNAL( valueChanged ( int ) ), this, SLOT( slotChanged() ) );
-    connect( reqExternal, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotChanged() ) );
+
+    connect( m_ui->cbExternal, SIGNAL( toggled( bool ) ), this, SLOT( externalToggled( bool ) ) );
+    connect( m_ui->grpPlayers, SIGNAL( clicked( int ) ), this, SLOT( slotChanged() ) );
+    connect( m_ui->volumeSlider, SIGNAL( valueChanged ( int ) ), this, SLOT( slotChanged() ) );
+    connect( m_ui->reqExternal, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotChanged() ) );
 }
 
 void PlayerSettingsDialog::load()
 {
     KConfig config( "knotifyrc", true, false );
     config.setGroup( "Misc" );
-    cbExternal->setChecked( config.readBoolEntry( "Use external player",
-                                                  false ));
-    reqExternal->setURL( config.readPathEntry( "External player" ));
-    reqExternal->setEnabled( cbExternal->isChecked() );
-    volumeSlider->setValue( config.readNumEntry( "Volume", 100 ) );
-    volumeSlider->parentWidget()->setEnabled( !cbExternal->isChecked() );
+    bool useExternal = config.readBoolEntry( "Use external player", false );
+    m_ui->cbExternal->setChecked( useExternal );
+    m_ui->reqExternal->setURL( config.readPathEntry( "External player" ) );
+    m_ui->volumeSlider->setValue( config.readNumEntry( "Volume", 100 ) );
+
+    if ( !m_ui->cbExternal->isChecked() )
+    {
+        config.setGroup( "StartProgress" );
+        if ( config.readBoolEntry( "Use Arts", true ) )
+        {
+            m_ui->cbArts->setChecked( true );
+        }
+        else
+        {
+            m_ui->cbNone->setChecked( true );
+        }
+    }
 }
 
 void PlayerSettingsDialog::save()
@@ -258,9 +244,29 @@ void PlayerSettingsDialog::save()
     // see kdelibs/arts/knotify/knotify.cpp
     KConfig config( "knotifyrc", false, false );
     config.setGroup( "Misc" );
-    config.writePathEntry( "External player", reqExternal->url() );
-    config.writeEntry( "Use external player", cbExternal->isChecked() );
-    config.writeEntry( "Volume", volumeSlider->value() );
+
+    config.writePathEntry( "External player", m_ui->reqExternal->url() );
+    config.writeEntry( "Use external player", m_ui->cbExternal->isChecked() );
+    config.writeEntry( "Volume", m_ui->volumeSlider->value() );
+
+    config.setGroup( "StartProgress" );
+
+    if ( m_ui->cbNone->isChecked() )
+    {
+        // user explicitly says "no sound!"
+        config.writeEntry( "Use Arts", false );
+    }
+    else if ( m_ui->cbArts->isChecked() )
+    {
+        // use explicitly said to use aRts so we turn it back on
+        // we don't want to always set this to the value of
+        // m_ui->cbArts->isChecked() since we don't want to
+        // turn off aRts support just because they also chose
+        // an external player
+        config.writeEntry( "Use Arts", true );
+        config.writeEntry( "Arts Init", true ); // reset it for the next time
+    }
+
     config.sync();
 }
 
@@ -282,6 +288,7 @@ void PlayerSettingsDialog::slotOk()
         slotApply();
     KDialogBase::slotOk();
 }
+
 void PlayerSettingsDialog::slotChanged()
 {
     dataChanged = true;
@@ -290,12 +297,10 @@ void PlayerSettingsDialog::slotChanged()
 
 void PlayerSettingsDialog::externalToggled( bool on )
 {
-    reqExternal->setEnabled( on );
-    volumeSlider->setEnabled( !on );
-    volumeLabel->setEnabled( !on );
-
     if ( on )
-        reqExternal->setFocus();
+        m_ui->reqExternal->setFocus();
+    else
+        m_ui->reqExternal->clearFocus();
 }
 
 #include "knotify.moc"
