@@ -13,18 +13,22 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __desktop_h__
 #define __desktop_h__
 
 #include <kcmodule.h>
-#include <qslider.h>
 
 class QSpinBox;
 class QLabel;
+class QCheckBox;
 class KLineEdit;
+class KIntNumInput;
+
+// if you change this, update also the number of keyboard shortcuts in kwin/kwinbindings.cpp
+static const int maxDesktops = 20;
 
 class KDesktopConfig : public KCModule
 {
@@ -37,18 +41,15 @@ class KDesktopConfig : public KCModule
   void save();
   void defaults();
   
-  int buttons();
-  QString quickHelp() const;
-
  protected slots:
   void slotValueChanged(int);
-  void slotTextChanged(const QString&);
     
  private:
-  QSlider    *_numSlider;
-  QLabel     *_numLabel;
-  QLabel     *_nameLabel[16];
-  KLineEdit  *_nameInput[16];
+  KIntNumInput *_numInput;
+  QLabel       *_nameLabel[maxDesktops];
+  KLineEdit    *_nameInput[maxDesktops];
+  QCheckBox    *_wheelOption;
+  bool         _wheelOptionImmutable;
 };
 
 #endif
