@@ -14,21 +14,24 @@
 
 
 #include <qstring.h>
-#include <kurl.h>
+
 #include <kcmodule.h>
 #include <kio/global.h>
+#include <kurl.h>
 
-class QPushButton;
-class KConfig;
 class QCheckBox;
 class QComboBox;
-class KURLRequester;
+class QPushButton;
+
+class KConfig;
 class KListView;
+class KURLRequester;
+
 namespace KIO { class Job; }
 
 //-----------------------------------------------------------------------------
 // The "Path" Tab contains :
-// The paths for Desktop, Trash and Autostart
+// The paths for Desktop, Autostart and Documents
 
 class DesktopPathConfig : public KCModule
 {
@@ -38,16 +41,13 @@ public:
         virtual void load();
         virtual void save();
         virtual void defaults();
-        virtual QString quickHelp() const;
 
 private slots:
-	void changed();
     void slotEntries( KIO::Job * job, const KIO::UDSEntryList& list);
 
 private:
         // Desktop Paths
         KURLRequester *urDesktop;
-        KURLRequester *urTrash;
         KURLRequester *urAutostart;
         KURLRequester *urDocument;
 
