@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000 by Matthias Kalle Dalheimer <kalle@kde.org>
+ *               2004 by Olivier Goffart  <ogoffart @ tiscalinet.be>
  *
  * Licensed under the Artistic License.
  */
@@ -23,6 +24,7 @@
 #include <kdialogbase.h>
 #include <kstdaccel.h>
 #include <kcolorbutton.h>
+#include <klistviewsearchline.h>
 
 #include <qtimer.h>
 #include <qwidgetstack.h>
@@ -43,6 +45,7 @@
 #include <dcopref.h>
 #include <qvbox.h>
 #include <qimage.h>
+#include <qheader.h>
 
 #include <kdebug.h>
 #include <kkeydialog.h>
@@ -75,7 +78,7 @@ public:
 		if(!e) return 0;
 		return e->text().toInt();
 	}
-	
+
 private:
 	QMap<int,KLineEdit*> m_widgets;
 };
@@ -115,31 +118,31 @@ DCOPBrowserApplicationItem::DCOPBrowserApplicationItem
 
 
 	/* Get the icon:  we use the icon from a mainwindow in that class.
-	    a lot of applications has a app-mainwindow#1 object, but others can still have 
+	    a lot of applications has a app-mainwindow#1 object, but others can still have
 	    a main window with another name. In that case, we search for a main window with the qt object.
 	 * Why don't search with qt dirrectly?
-	    simply because some application that have a 'mainwindow#1' doesn't have a qt object.  And, for 
+	    simply because some application that have a 'mainwindow#1' doesn't have a qt object.  And, for
 	    reason i don't know, some application return stanges result. Some konqueror instance are returning
-	    konqueror-mainwindow#3 while only the #1 exists,  I already seen the same problem with konsole 
-	 * All calls are async to don't block the GUI if the clients does not reply immediatly 
+	    konqueror-mainwindow#3 while only the #1 exists,  I already seen the same problem with konsole
+	 * All calls are async to don't block the GUI if the clients does not reply immediatly
 	 */
-     
-	QRegExp rx( "([^\\-]+)");  // remove the possible processus id   
+
+	QRegExp rx( "([^\\-]+)");  // remove the possible processus id
 	rx.search(app_);           //   konqueror-123  => konqueror-mainwindow#1
 	QString mainWindowName= rx.cap(1) + "-mainwindow#1" ;
-	
+
 	QByteArray data;
 	int callId=kapp->dcopClient()->callAsync( app_, mainWindowName.utf8(), "icon()", data, this, SLOT(retreiveIcon(int, const QCString&, const QByteArray&)));
-	
+
 	if(!callId)
 	{
 		//maybe there is another mainwindow registered with another name.
 		QByteArray data;
 		QDataStream arg(data, IO_WriteOnly);
 		arg << QCString( "MainWindow" );
-		
+
 		kapp->dcopClient()->callAsync( app_, "qt", "find(QCString)", data, this, SLOT(slotGotWindowName(int, const QCString&, const QByteArray& )));
-	} 
+	}
 }
 
   void
@@ -187,7 +190,7 @@ void DCOPBrowserApplicationItem::slotGotWindowName(int /*callId*/, const QCStrin
 		if(!mainWindowName.isEmpty())
 		{
 			QByteArray data;
-			kapp->dcopClient()->callAsync( app_, mainWindowName.utf8(), "icon()", data, 
+			kapp->dcopClient()->callAsync( app_, mainWindowName.utf8(), "icon()", data,
 				this, SLOT(retreiveIcon(int, const QCString&, const QByteArray&)));
 		}
 	}
@@ -199,7 +202,7 @@ void DCOPBrowserApplicationItem::retreiveIcon(int /*callId*/,const QCString& /*r
 	QPixmap returnQPixmap;
 	reply >> returnQPixmap;
 	if(!returnQPixmap.isNull())
-		setPixmap(0, QPixmap(returnQPixmap.convertToImage().smoothScale(16,16)) );  
+		setPixmap(0, QPixmap(returnQPixmap.convertToImage().smoothScale(16,16)) );
 	else
 		kdDebug() << "Unable to retreive the icon" << endl;
 }
@@ -285,8 +288,10 @@ KDCOPWindow::KDCOPWindow(QWidget *parent, const char * name)
   statusBar()->message(i18n("Welcome to the KDE DCOP browser"));
 
 	mainView = new kdcopview(this, "KDCOP");
+        mainView->kListViewSearchLine1->setListView( mainView->lv );
 	setCentralWidget(mainView);
 	mainView->lv->addColumn(i18n("Application"));
+	mainView->lv->header()->setStretchEnabled(true, 0);
 //	mainView->lv->addColumn(i18n("Interface"));
 //	mainView->lv->addColumn(i18n("Function"));
 	mainView->lv->setDragAutoScroll( FALSE );
@@ -310,7 +315,7 @@ KDCOPWindow::KDCOPWindow(QWidget *parent, const char * name)
   KStdAction::quit( this, SLOT( close() ), actionCollection() );
   KStdAction::copy( this, SLOT( slotCopy()), actionCollection() );
   KStdAction::keyBindings( guiFactory(), SLOT( configureShortcuts() ), actionCollection() );
-  
+
 
   (void) new KAction( i18n( "&Reload" ), "reload", KStdAccel::shortcut(KStdAccel::Reload), this, SLOT( slotReload() ), actionCollection(), "reload" );
 
@@ -459,7 +464,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         wl.append( e );
         e->setValidator( new QIntValidator( e ) );
       }
-      else if ( type == "unsigned"  || type == "uint" || type == "unsigned int" 
+      else if ( type == "unsigned"  || type == "uint" || type == "unsigned int"
              || type == "Q_UINT32" )
       {
         QLabel* n = new QLabel( names[i-1], frame );
@@ -485,7 +490,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         wl.append( e );
         e->setValidator( new QIntValidator( e ) );
       }
-      else if ( type == "ulong" || type == "unsigned long" || type == "unsigned long int" 
+      else if ( type == "ulong" || type == "unsigned long" || type == "unsigned long int"
              || type == "Q_UINT64" )
       {
         QLabel* n = new QLabel( names[i-1], frame );
@@ -497,6 +502,28 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         wl.append( e );
         e->setValidator( new QIntValidator( e ) );
       }
+      else if ( type == "short" || type == "short int" )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "long", frame );
+        grid->addWidget( l, i, 1 );
+        KLineEdit* e = new KLineEdit( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+        e->setValidator( new QIntValidator( e ) );
+      }
+      else if ( type == "ushort" || type == "unsigned short" || type == "unsigned short int"  )
+      {
+        QLabel* n = new QLabel( names[i-1], frame );
+        grid->addWidget( n, i, 0 );
+        QLabel* l = new QLabel( "unsigned short", frame );
+        grid->addWidget( l, i, 1 );
+        KLineEdit* e = new KLineEdit( frame );
+        grid->addWidget( e, i, 2 );
+        wl.append( e );
+        e->setValidator( new QIntValidator( e ) );
+      }
       else if ( type == "float" )
       {
         QLabel* n = new QLabel( names[i-1], frame );
@@ -631,7 +658,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
       {
         QLabel* n = new QLabel( names[i-1], frame );
         grid->addWidget( n, i, 0 );
-        QLabel* l = new QLabel( "KURL", frame );
+        QLabel* l = new QLabel( "QPixmap", frame );
         grid->addWidget( l, i, 1 );
         KURLRequester* e = new KURLRequester( frame );
         grid->addWidget( e, i, 2 );
@@ -683,6 +710,16 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
         KLineEdit* e = (KLineEdit*)wl.at( i );
         arg << e->text().toULong();
       }
+      else if( type == "short" || type == "short int" )
+      {
+        KLineEdit* e = (KLineEdit*)wl.at( i );
+        arg << e->text().toShort();
+      }
+      else if( type == "ushort" || type == "unsigned short" || type == "unsigned short int" )
+      {
+        KLineEdit* e = (KLineEdit*)wl.at( i );
+        arg << e->text().toUShort();
+      }
       else if ( type == "Q_UINT64" )
       {
         KLineEdit* e = ( KLineEdit* )wl.at( i );
@@ -752,7 +789,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
       else if( type == "QPixmap" )
       {
        KURLRequester* e= (KURLRequester*)wl.at( i );
-       arg << QPixmap(e->url());        
+       arg << QPixmap(e->url());
       }
       else
       {
@@ -864,7 +901,7 @@ void KDCOPWindow::slotCallFunction( QListViewItem* it )
 void KDCOPWindow::slotFillApplications()
 {
   KApplication::setOverrideCursor(waitCursor);
-  
+
   QCStringList apps = dcopClient->registeredApplications();
   QCString appId = dcopClient->appId();
 
@@ -912,7 +949,7 @@ bool KDCOPWindow::demarshal
     reply >> i;
     ret << QString::number(i);
   }
-  else if ( replyType == "uint" || replyType == "unsigned int" 
+  else if ( replyType == "uint" || replyType == "unsigned int"
          || replyType == "Q_UINT32" )
   {
     uint i;
