@@ -14,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
@@ -60,7 +60,6 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 : KMainWindow(parent, name)
 {
 	m_faxctrl = new FaxCtrl(this);
-	m_handler = 0;
 	m_quitAfterSend = false;
 	connect(m_faxctrl, SIGNAL(message(const QString&)), SLOT(slotMessage(const QString&)));
 	connect(m_faxctrl, SIGNAL(faxSent(bool)), SLOT(slotFaxSent(bool)));
@@ -70,12 +69,12 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 	m_files = new KListBox(mainw);
 	connect( m_files, SIGNAL( currentChanged( QListBoxItem* ) ), SLOT( slotCurrentChanged() ) );
 	m_upbtn = new KPushButton( mainw );
-	m_upbtn->setPixmap( SmallIcon( "up" ) );
+	m_upbtn->setIconSet( SmallIconSet( "up" ) );
 	QToolTip::add( m_upbtn, i18n( "Move up" ) );
 	connect( m_upbtn, SIGNAL( clicked() ), SLOT( slotMoveUp() ) );
 	m_upbtn->setEnabled( false );
 	m_downbtn = new KPushButton( mainw );
-	m_downbtn->setPixmap( SmallIcon( "down" ) );
+	m_downbtn->setIconSet( SmallIconSet( "down" ) );
 	QToolTip::add( m_downbtn, i18n( "Move down" ) );
 	connect( m_downbtn, SIGNAL( clicked() ), SLOT( slotMoveDown() ) );
 	m_downbtn->setEnabled( false );
@@ -100,7 +99,7 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 	QToolTip::add( m_abbtn, i18n( "Add fax number from addressbook" ) );
 	connect( m_abbtn, SIGNAL( clicked() ), SLOT( slotKab() ) );
 	m_delbtn = new KPushButton( mainw );
-	m_delbtn->setPixmap( SmallIcon( "trash" ) );
+	m_delbtn->setIconSet( SmallIconSet( "edittrash" ) );
 	QToolTip::add( m_delbtn, i18n( "Remove fax number" ) );
 	m_delbtn->setEnabled( false );
 	connect( m_delbtn, SIGNAL( clicked() ), SLOT( slotFaxRemove() ) );
@@ -178,7 +177,6 @@ KdeprintFax::KdeprintFax(QWidget *parent, const char *name)
 
 KdeprintFax::~KdeprintFax()
 {
-	delete m_handler;
 }
 
 void KdeprintFax::initActions()
@@ -230,8 +228,6 @@ void KdeprintFax::slotView()
 {
 	if (m_files->currentItem() >= 0)
 	{
-		if (!m_handler)
-			m_handler = new KFileOpenWithHandler;
 		new KRun(KURL( m_files->currentText() ));
 	}
 }
@@ -271,7 +267,7 @@ void KdeprintFax::slotKab()
 void KdeprintFax::addURL(KURL url)
 {
 	QString	target;
-	if (KIO::NetAccess::download(url,target))
+	if (KIO::NetAccess::download(url,target,this))
 	{
 		m_files->insertItem(KMimeType::pixmapForURL(url,0,KIcon::Small),target);
 		actionCollection()->action("file_remove")->setEnabled(true);
