@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 
@@ -25,13 +25,17 @@
 #include "startupid.h"
 #include "kdiconview.h"
 #include "minicli.h"
+#include "kdesktopsettings.h"
+#include "klaunchsettings.h"
 
 #include <string.h>
 #include <unistd.h>
 #include <kcolordrag.h>
+#include <kurldrag.h>
 
 #include <qdir.h>
 #include <qevent.h>
+#include <qtooltip.h>
 
 #include <netwm.h>
 #include <dcopclient.h>
@@ -67,49 +71,78 @@
 
 KRootWidget::KRootWidget() : QObject()
 {
-     m_bWheelSwitchesWorkspace = false;
      kapp->desktop()->installEventFilter(this);
+     kapp->desktop()->setAcceptDrops( true );
 }
 
 bool KRootWidget::eventFilter ( QObject *, QEvent * e )
 {
      if (e->type() == QEvent::MouseButtonPress)
      {
-       QMouseEvent *me = (QMouseEvent *) e;
+       QMouseEvent *me = static_cast<QMouseEvent *>(e);
        KRootWm::self()->mousePressed( me->globalPos(), me->button() );
        return true;
      }
-     else if (m_bWheelSwitchesWorkspace &&  e->type() == QEvent::Wheel && KWin::numberOfDesktops() > 1)
+     else if (e->type() == QEvent::Wheel)
      {
-       QWheelEvent *we = (QWheelEvent *) e;
-       int newDesk, curDesk = KWin::currentDesktop();
-       if( we->delta() < 0 )
-         newDesk = curDesk % KWin::numberOfDesktops() + 1;
-       else
-         newDesk = ( KWin::numberOfDesktops() + curDesk - 2 ) % KWin::numberOfDesktops() + 1;
-       KWin::setCurrentDesktop( newDesk );
+       QWheelEvent *we = static_cast<QWheelEvent *>(e);
+       emit wheelRolled(we->delta());
+       return true;
+     }
+     else if ( e->type() == QEvent::DragEnter )
+     {
+       QDragEnterEvent* de = static_cast<QDragEnterEvent *>( e );
+       bool b = !KGlobal::config()->isImmutable();
+       b &= !KGlobal::dirs()->isRestrictedResource( "wallpaper" );
+
+       bool imageURL = false;
+       if ( KURLDrag::canDecode( de ) )
+       {
+         KURL::List list;
+         KURLDrag::decode( de, list );
+         KURL url = list.first();
+         KMimeType::Ptr mime = KMimeType::findByURL( url );
+         if ( !KImageIO::type( url.path() ).isEmpty() ||
+              KImageIO::isSupported( mime->name(), KImageIO::Reading ) || mime->is( "image/svg+xml" ) )
+           imageURL = true;
+       }
+
+       b &= KColorDrag::canDecode( de ) || QImageDrag::canDecode( de ) || imageURL;
+       de->accept( b );
+       return true;
+     }
+     else if ( e->type() == QEvent::Drop )
+     {
+       QDropEvent* de = static_cast<QDropEvent*>( e );
+       if ( KColorDrag::canDecode( de ) ) 
+         emit colorDropEvent( de );
+       else if ( QImageDrag::canDecode( de ) )
+         emit imageDropEvent( de );
+       else if ( KURLDrag::canDecode( de ) ) {
+	 KURL::List list;
+         KURLDrag::decode( de, list );
+         KURL url = list.first();
+         emit newWallpaper( url );
+       }
        return true;
      }
      return false; // Don't filter.
 }
 
-void KRootWidget::setWheelSwitchEnabled( bool enable )
-{
-     m_bWheelSwitchesWorkspace = enable;
-}
-
 // -----------------------------------------------------------------------------
 #define DEFAULT_DELETEACTION 1
 
 // for multihead - from main.cc
 extern int kdesktop_screen_number;
 
+KDesktop::WheelDirection KDesktop::m_eWheelDirection = KDesktop::m_eDefaultWheelDirection;
+const char* KDesktop::m_wheelDirectionStrings[2] = { "Forward", "Reverse" };
+
 KDesktop::KDesktop( bool x_root_hack, bool auto_start, bool wait_for_kded ) :
     DCOPObject( "KDesktopIface" ),
     QWidget( 0L, "desktop", WResizeNoErase | ( x_root_hack ? (WStyle_Customize | WStyle_NoBorder) : 0) ),
     // those two WStyle_ break kdesktop when the root-hack isn't used (no Dnd)
-
-   startup_id( NULL )
+   startup_id( NULL ), m_waitForKicker(0)
 {
   m_bAutoStart = auto_start;
   m_bWaitForKded = wait_for_kded;
@@ -117,16 +150,16 @@ KDesktop::KDesktop( bool x_root_hack, bool auto_start, bool wait_for_kded ) :
   keys = 0; // created later
   KGlobal::locale()->insertCatalogue("kdesktop");
   KGlobal::locale()->insertCatalogue("libkonq"); // needed for apps using libkonq
+  KGlobal::locale()->insertCatalogue("libdmctl");
 
   setCaption( "KDE Desktop");
 
   setAcceptDrops(true); // WStyle_Customize seems to disable that
   m_pKwinmodule = new KWinModule( this );
-  updateWorkAreaTimer = new QTimer( this );
-  connect( updateWorkAreaTimer, SIGNAL( timeout() ),
-           this, SLOT( updateWorkArea() ) );
-  connect( m_pKwinmodule, SIGNAL( workAreaChanged() ),
-           this, SLOT( workAreaChanged() ) );
+
+  kapp->dcopClient()->setNotifications(true);
+  kapp->dcopClient()->connectDCOPSignal("kicker", "kicker", "desktopIconsAreaChanged(QRect, int)",
+                                        "KDesktopIface", "desktopIconsAreaChanged(QRect, int)", false);
 
   // Dont repaint on configuration changes during construction
   m_bInit = true;
@@ -182,10 +215,9 @@ KDesktop::initRoot()
   Display *dpy = qt_xdisplay();
   Window root = RootWindow(dpy, kdesktop_screen_number);
   XDefineCursor(dpy, root, cursor().handle());
-
-  KConfigGroup configGroup(KGlobal::config(), "General");
-  m_bDesktopEnabled = configGroup.readBoolEntry("Enabled", true);
-  if (!m_bDesktopEnabled && !m_pRootWidget)
+  
+  m_bDesktopEnabled = KDesktopSettings::desktopEnabled();
+  if ( !m_bDesktopEnabled && !m_pRootWidget )
   {
      hide();
      delete bgMgr;
@@ -195,20 +227,32 @@ KDesktop::initRoot()
      delete m_pIconView;
      m_pIconView = 0;
 
+     { // trigger creation of QToolTipManager, it does XSelectInput() on the root window
+     QWidget w;
+     QToolTip::add( &w, "foo" );
+     }
+     // NOTE: If mouse clicks stop working again, it's most probably something doing XSelectInput()
+     // on the root window after this, and setting it to some fixed value instead of adding its mask.
      XWindowAttributes attrs;
      XGetWindowAttributes(dpy, root, &attrs);
      XSelectInput(dpy, root, attrs.your_event_mask | ButtonPressMask);
 
      m_pRootWidget = new KRootWidget;
+     connect(m_pRootWidget, SIGNAL(wheelRolled(int)), this, SLOT(slotSwitchDesktops(int)));
+     connect(m_pRootWidget, SIGNAL(colorDropEvent(QDropEvent*)), this, SLOT(handleColorDropEvent(QDropEvent*)) );
+     connect(m_pRootWidget, SIGNAL(imageDropEvent(QDropEvent*)), this, SLOT(handleImageDropEvent(QDropEvent*)) );
+     connect(m_pRootWidget, SIGNAL(newWallpaper(const KURL&)), this, SLOT(slotNewWallpaper(const KURL&)) );
+
      // Geert Jansen: backgroundmanager belongs here
      // TODO tell KBackgroundManager if we change widget()
      bgMgr = new KBackgroundManager( m_pIconView, m_pKwinmodule );
+     bgMgr->setExport(1);
      connect( bgMgr, SIGNAL( initDone()), SLOT( backgroundInitDone()));
      if (!m_bInit)
      {
         delete KRootWm::self();
         KRootWm* krootwm = new KRootWm( this ); // handler for root menu (used by kdesktop on RMB click)
-        keys->setSlot("Lock Screen", krootwm, SLOT(slotLock()));
+        keys->setSlot("Lock Session", krootwm, SLOT(slotLock()));
         keys->updateConnections();
      }
   }
@@ -225,6 +269,8 @@ KDesktop::initRoot()
               this, SLOT( handleColorDropEvent( QDropEvent * ) ) );
      connect( m_pIconView, SIGNAL( newWallpaper( const KURL & ) ),
               this, SLOT( slotNewWallpaper( const KURL & ) ) );
+     connect( m_pIconView, SIGNAL( wheelRolled( int ) ),
+              this, SLOT( slotSwitchDesktops( int ) ) );
 
      // All the QScrollView/QWidget-specific stuff should go here, so that we can use
      // another qscrollview/widget instead of the iconview and use the same code
@@ -241,19 +287,55 @@ KDesktop::initRoot()
      // Geert Jansen: backgroundmanager belongs here
      // TODO tell KBackgroundManager if we change widget()
      bgMgr = new KBackgroundManager( m_pIconView, m_pKwinmodule );
+     bgMgr->setExport(1);
      connect( bgMgr, SIGNAL( initDone()), SLOT( backgroundInitDone()));
 
-     //kdDebug(1204) << "KDesktop constructor -> workAreaChanged" << endl;
-     workAreaChanged();
+     if (!m_bInit)
+        m_pIconView->initConfig( m_bInit );
+
+     // set the size of the area for desktop icons placement
+     {
+        QByteArray data, result;
+        QDataStream arg(data, IO_WriteOnly);
+        arg << kdesktop_screen_number;
+        QCString replyType;
+        QRect area;
+
+        if ( kapp->dcopClient()->call("kicker", "kicker", "desktopIconsArea(int)",
+                                       data, replyType, result, false, 2000) )
+        {
+          QDataStream res(result, IO_ReadOnly);
+          res >> area;
+
+          m_pIconView->updateWorkArea(area);
+        }
+        else
+          if ( m_bInit )
+          {
+            // if we failed to get the information from kicker wait a little - probably
+            // this is the KDE startup and kicker is simply not running yet
+            m_waitForKicker = new QTimer(this);
+            connect(m_waitForKicker, SIGNAL(timeout()), this, SLOT(slotNoKicker()));
+            m_waitForKicker->start(15000, true);
+          }
+          else  // we are not called from the ctor, so kicker should already run
+          {
+            area = kwinModule()->workArea(kwinModule()->currentDesktop());
+            m_pIconView->updateWorkArea(area);
+          }
+     }
+
      if (!m_bInit)
      {
-        m_pIconView->initConfig( m_bInit );
         m_pIconView->start();
         delete KRootWm::self();
         KRootWm* krootwm = new KRootWm( this ); // handler for root menu (used by kdesktop on RMB click)
-        keys->setSlot("Lock Screen", krootwm, SLOT(slotLock()));
+        keys->setSlot("Lock Session", krootwm, SLOT(slotLock()));
         keys->updateConnections();
      }
+   } else {
+     DCOPRef r( "ksmserver", "ksmserver" );
+     r.send( "resumeStartup" );
    }
 
    KWin::setType( winId(), NET::Desktop );
@@ -261,10 +343,20 @@ KDesktop::initRoot()
    KWin::setOnAllDesktops( winId(), true );
 }
 
+void KDesktop::slotNoKicker()
+{
+    kdDebug(1204) << "KDesktop::slotNoKicker ... kicker did not respond" << endl;
+    // up till now, we got no desktopIconsArea from kicker - probably
+    // it's not running, so use the area from KWinModule
+    m_pIconView->updateWorkArea(kwinModule()->workArea(kwinModule()->currentDesktop()));
+}
+
 void
 KDesktop::backgroundInitDone()
 {
     //kdDebug(1204) << "KDesktop::backgroundInitDone" << endl;
+    DCOPRef r( "ksmserver", "ksmserver" );
+    r.send( "resumeStartup" );
 
     // avoid flicker
     if (m_bDesktopEnabled)
@@ -284,6 +376,16 @@ KDesktop::slotStart()
   if (!m_bInit) return;
 
   kapp->dcopClient()->send( "ksplash", "", "upAndRunning(QString)", QString("kdesktop"));
+#ifdef Q_WS_X11
+  XEvent e;
+  e.xclient.type = ClientMessage;
+  e.xclient.message_type = XInternAtom( qt_xdisplay(), "_KDE_SPLASH_PROGRESS", False );
+  e.xclient.display = qt_xdisplay();
+  e.xclient.window = qt_xrootwin();
+  e.xclient.format = 8;
+  strcpy( e.xclient.data.b, "kdesktop" );
+  XSendEvent( qt_xdisplay(), qt_xrootwin(), False, SubstructureNotifyMask, &e );
+#endif
 
   // In case we started without database
   KImageIO::registerFormats();
@@ -362,9 +464,8 @@ void KDesktop::initConfig()
         keys->updateConnections();
     }
 
-    KConfig c( "klaunchrc", true );
-    c.setGroup( "FeedbackStyle" );
-    if( !c.readBoolEntry( "BusyCursor", true ))
+    KLaunchSettings::self()->readConfig();
+    if( !KLaunchSettings::busyCursor() )
     {
         delete startup_id;
         startup_id = NULL;
@@ -376,15 +477,14 @@ void KDesktop::initConfig()
         startup_id->configure();
     }
 
-    KConfig * config = KGlobal::config();
-    config->setGroup( "General" );
-    set_vroot = config->readBoolEntry( "SetVRoot", false );
+    set_vroot = KDesktopSettings::setVRoot();
     slotSetVRoot(); // start timer
 
-    if (m_pRootWidget) {
-        config->setGroup("Mouse Buttons");
-        m_pRootWidget->setWheelSwitchEnabled( config->readBoolEntry("WheelSwitchesWorkspace", false) );
-    }
+    m_bWheelSwitchesWorkspace = KDesktopSettings::wheelSwitchesWorkspace();
+
+    const char* forward_string = m_wheelDirectionStrings[Forward];
+    m_eWheelDirection =
+        (KDesktopSettings::wheelDirection() == forward_string) ? Forward : Reverse;
 }
 
 // -----------------------------------------------------------------------------
@@ -402,6 +502,11 @@ void KDesktop::slotExecuteCommand()
   Shows minicli
  */
 void KDesktop::popupExecuteCommand()
+{
+  popupExecuteCommand("");
+}
+
+void KDesktop::popupExecuteCommand(const QString& command)
 {
   if (m_bInit)
       return;
@@ -416,30 +521,30 @@ void KDesktop::popupExecuteCommand()
       m_miniCli->adjustSize(); // for the centering below
   }
 
+  if (!command.isEmpty())
+      m_miniCli->setCommand(command);
+
   // Move minicli to the current desktop
   NETWinInfo info( qt_xdisplay(), m_miniCli->winId(), qt_xrootwin(), NET::WMDesktop );
   int currentDesktop = kwinModule()->currentDesktop();
   if ( info.desktop() != currentDesktop )
       info.setDesktop( currentDesktop );
-  // Update the user timestamp. Minicli can be shown either by a keyboard shortcut,
-  // or by a DCOP call. In the case of the DCOP call, this is necessary, otherwise
-  // kwin's focus stealing prevention could refuse to make minicli the active window.
-  // This means the DCOP call is only for user scripting. In the case of keyboard
-  // shortcut, the shortcut itself is user action, so this doesn't really matter,
-  // it only makes sure minicli will be active even if kdesktop will be slow to react.
-  // (kdebase/kwin/README, section on focus stealing prevention)
-  kapp->updateUserTimestamp();
 
   if ( m_miniCli->isVisible() ) {
       KWin::forceActiveWindow( m_miniCli->winId() );
   } else {
       QRect rect = KGlobalSettings::desktopGeometry(QCursor::pos());
       m_miniCli->move(rect.x() + (rect.width() - m_miniCli->width())/2,
-                      rect.y() + (rect.height() - m_miniCli->height())/4);
-      m_miniCli->exec();
+                      rect.y() + (rect.height() - m_miniCli->height())/2);
+      m_miniCli->show(); // non-modal
   }
 }
 
+void KDesktop::slotSwitchUser()
+{
+     KRootWm::self()->slotSwitchUser();
+}
+
 void KDesktop::slotShowWindowList()
 {
      KRootWm::self()->slotWindowList();
@@ -523,6 +628,7 @@ void KDesktop::configure()
 {
     // re-read configuration and apply it
     KGlobal::config()->reparseConfiguration();
+    KDesktopSettings::self()->readConfig();
 
     // If we have done start() already, then re-configure.
     // Otherwise, start() will call initConfig anyway
@@ -533,8 +639,11 @@ void KDesktop::configure()
        KRootWm::self()->initConfig();
     }
 
-    keys->readSettings();
-    keys->updateConnections();
+    if (keys)
+    {
+       keys->readSettings();
+       keys->updateConnections();
+    }
 }
 
 void KDesktop::slotSettingsChanged(int category)
@@ -559,7 +668,6 @@ void KDesktop::slotIconChanged(int group)
     if ( group == KIcon::Desktop )
     {
         kdDebug(1204) << "KDesktop::slotIconChanged" << endl;
-        refreshIcons();
         refresh();
     }
 }
@@ -582,9 +690,9 @@ void KDesktop::refresh()
 #if 0
   m_bNeedRepaint |= 1;
   updateWorkArea();
-  refreshIcons();
 #endif
   kapp->dcopClient()->send( "kwin", "", "refresh()", "");
+  refreshIcons();
 }
 
 // -----------------------------------------------------------------------------
@@ -639,19 +747,27 @@ void KDesktop::closeEvent(QCloseEvent *e)
     e->ignore();
 }
 
-void KDesktop::workAreaChanged()
+void KDesktop::desktopIconsAreaChanged(const QRect &area, int screen)
 {
-    //kdDebug(1204) << "KDesktop::workAreaChanged() -> starting timer" << endl;
-    updateWorkAreaTimer->stop();
-    updateWorkAreaTimer->start( 100, TRUE );
+    // hurra! kicker is alive
+    if ( m_waitForKicker ) m_waitForKicker->stop();
+
+    if ( (screen == kdesktop_screen_number) && m_pIconView )
+        m_pIconView->updateWorkArea(area);
 }
 
-void KDesktop::updateWorkArea()
+void KDesktop::slotSwitchDesktops(int delta)
 {
-    if (m_pIconView)
+    if(m_bWheelSwitchesWorkspace && KWin::numberOfDesktops() > 1)
     {
-        QRect wr( kwinModule()->workArea( kwinModule()->currentDesktop() ) );
-        m_pIconView->updateWorkArea( wr );
+      int newDesk, curDesk = KWin::currentDesktop();
+
+      if( (delta < 0 && m_eWheelDirection == Forward) || (delta > 0 && m_eWheelDirection == Reverse) )
+        newDesk = curDesk % KWin::numberOfDesktops() + 1;
+      else
+        newDesk = ( KWin::numberOfDesktops() + curDesk - 2 ) % KWin::numberOfDesktops() + 1;
+
+      KWin::setCurrentDesktop( newDesk );
     }
 }
 
@@ -675,8 +791,9 @@ void KDesktop::handleColorDropEvent(QDropEvent * e)
 void KDesktop::handleImageDropEvent(QDropEvent * e)
 {
     KPopupMenu popup;
+    if ( m_pIconView )
     popup.insertItem(SmallIconSet("filesave"),i18n("&Save to Desktop..."), 1);
-    if (m_pIconView && m_pIconView->maySetWallpaper())
+    if ( ( m_pIconView && m_pIconView->maySetWallpaper() ) || m_pRootWidget )
        popup.insertItem(SmallIconSet("background"),i18n("Set as &Wallpaper"), 2);
     popup.insertSeparator();
     popup.insertItem(SmallIconSet("cancel"), i18n("&Cancel"));
@@ -706,7 +823,11 @@ void KDesktop::handleImageDropEvent(QDropEvent * e)
         KTempFile tmpFile(QString::null, filename);
         i.save(tmpFile.name(), "PNG");
         // We pass 0 as parent window because passing the desktop is not a good idea
-        KIO::NetAccess::copy(tmpFile.name(), KGlobalSettings::desktopPath() + "/" + filename, 0);
+        KURL src;
+        src.setPath( tmpFile.name() );
+        KURL dest( KDIconView::desktopURL() );
+        dest.addPath( filename );
+        KIO::NetAccess::copy( src, dest, 0 );
         tmpFile.unlink();
     }
     else if (result == 2)
@@ -726,7 +847,7 @@ void KDesktop::slotNewWallpaper(const KURL &url)
     // (called by KonqOperations)
     QString tmpFile;
     // We pass 0 as parent window because passing the desktop is not a good idea
-    KIO::NetAccess::download( url, tmpFile, 0 ); 
+    KIO::NetAccess::download( url, tmpFile, 0 );
     bgMgr->setWallpaper(tmpFile);
 }
 
@@ -742,9 +863,9 @@ void KDesktop::logout( KApplication::ShutdownConfirm confirm,
 {
     if( !kapp->requestShutDown( confirm, sdtype ) )
         // this i18n string is also in kicker/applets/run/runapplet
-        KMessageBox::error( this, i18n("Could not logout properly.\nThe session manager cannot "
+        KMessageBox::error( this, i18n("Could not log out properly.\nThe session manager cannot "
                                         "be contacted. You can try to force a shutdown by pressing "
-                                        "Ctrl+Alt+Backspace. Note, however, that your current session "
+                                        "Ctrl+Alt+Backspace; note, however, that your current session "
                                         "will not be saved with a forced shutdown." ) );
 }
 
@@ -779,10 +900,8 @@ void KDesktop::setVRoot( bool enable )
 
     set_vroot = enable;
     kdDebug(1204) << "setVRoot " << enable << endl;
-    KConfig * config = KGlobal::config();
-    KConfigGroupSaver gs( config, "General" );
-    config->writeEntry( "SetVRoot", set_vroot );
-    config->sync();
+    KDesktopSettings::setSetVRoot( set_vroot );
+    KDesktopSettings::writeConfig();
     slotSetVRoot();
 }
 
@@ -799,10 +918,8 @@ void KDesktop::setIconsEnabled( bool enable )
 
     m_bDesktopEnabled = enable;
     kdDebug(1204) << "setIcons " << enable << endl;
-    KConfig * config = KGlobal::config();
-    KConfigGroupSaver gs( config, "General" );
-    config->writeEntry( "Enabled", m_bDesktopEnabled );
-    config->sync();
+    KDesktopSettings::setDesktopEnabled( m_bDesktopEnabled );
+    KDesktopSettings::writeConfig();
     if (!enable) {
         delete m_pIconView;
         m_pIconView = 0;
@@ -812,7 +929,43 @@ void KDesktop::setIconsEnabled( bool enable )
 
 void KDesktop::desktopResized()
 {
-    resize( kapp->desktop()->size());
+    resize(kapp->desktop()->size());
+
+    if ( m_pIconView )
+    {
+        // the sequence of actions is important:
+        // remove all icons, resize desktop, tell kdiconview new iconsArea size
+        // tell kdiconview to reget all icons
+        m_pIconView->slotClear();
+        m_pIconView->resize(kapp->desktop()->size());
+
+        // get new desktopIconsArea from kicker
+        QByteArray data, result;
+        QDataStream arg(data, IO_WriteOnly);
+        arg << kdesktop_screen_number;
+        QCString replyType;
+        QRect area;
+
+        if ( kapp->dcopClient()->call("kicker", "kicker", "desktopIconsArea(int)",
+                                       data, replyType, result, false, 2000) )
+        {
+            QDataStream res(result, IO_ReadOnly);
+            res >> area;
+        }
+        else
+            area = kwinModule()->workArea(kwinModule()->currentDesktop());
+
+        m_pIconView->updateWorkArea(area);
+        m_pIconView->startDirLister();
+    }
+}
+
+void KDesktop::switchDesktops( int delta )
+{
+    bool old = m_bWheelSwitchesWorkspace;
+    m_bWheelSwitchesWorkspace = true;
+    slotSwitchDesktops(delta);
+    m_bWheelSwitchesWorkspace = old;
 }
 
 bool KDesktop::event(QEvent * e)
