@@ -24,8 +24,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <pwd.h>
@@ -44,6 +44,7 @@
 #include <qwhatsthis.h>
 #include <qstylesheet.h>
 
+#include <dcopclient.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
@@ -88,32 +89,14 @@ Minicli::Minicli( QWidget *parent, const char *name)
   m_dlg->cbCommand->setDuplicatesEnabled( false );
   m_dlg->cbCommand->setTrapReturnKey( true );
 
-  // IMPORTANT NOTE: the buttons in the minicli are QPushButtons, not KPushButtons
-  // this is because KPushButtons install a tooltip automatically which, via
-  // QTipManager's (properly written, even) event filter on the qApp, interferes
-  // with event filters installed on the root window (qApp->desktop()) which prevents
-  // the user from being able to access desktop menus in certain configurations
-  // (e.g. icons turned off)
-  // so the KGuiItems are handled manually and QPushButtons are users.
-  // see BR#110607
-
   // Options button...
-  KGuiItem guiItem = KGuiItem( i18n("&Options >>"), "configure" );
-  m_dlg->pbOptions->setIconSet(guiItem.iconSet());
-  m_dlg->pbOptions->setText(guiItem.text());
-  QWhatsThis::add(m_dlg->pbOptions, guiItem.whatsThis());
+  m_dlg->pbOptions->setGuiItem (KGuiItem( i18n("&Options >>"), "configure" ));
 
   // Run button...
-  guiItem = KGuiItem( i18n("&Run"), "run" );
-  m_dlg->pbRun->setIconSet(guiItem.iconSet());
-  m_dlg->pbRun->setText(guiItem.text());
-  QWhatsThis::add(m_dlg->pbRun, guiItem.whatsThis());
+  m_dlg->pbRun->setGuiItem (KGuiItem( i18n("&Run"), "run" ));
 
   // Cancel button...
-  guiItem = KStdGuiItem::cancel();
-  m_dlg->pbCancel->setIconSet(guiItem.iconSet());
-  m_dlg->pbCancel->setText(guiItem.text());
-  QWhatsThis::add(m_dlg->pbCancel, guiItem.whatsThis());
+  m_dlg->pbCancel->setGuiItem ( KStdGuiItem::cancel() );
 
   if (!kapp->authorize("shell_access"))
     m_dlg->pbOptions->hide();
@@ -138,8 +121,6 @@ Minicli::Minicli( QWidget *parent, const char *name)
 
   m_dlg->leUsername->setText("root");
 
-  setMaxCommandBoxWidth();
-
   // Main widget buttons...
   connect( m_dlg->pbRun, SIGNAL(clicked()), this, SLOT(accept()) );
   connect( m_dlg->pbCancel, SIGNAL(clicked()), this, SLOT(reject()) );
@@ -180,9 +161,9 @@ void Minicli::setCommand(const QString& command)
   }
 }
 
-void Minicli::setMaxCommandBoxWidth()
+QSize Minicli::sizeHint() const
 {
-  int maxWidth = qApp->desktop()->screenGeometry(this).width();
+  int maxWidth = qApp->desktop()->screenGeometry((QWidget*)this).width();
   if (maxWidth < 603)
   {
     // a sensible max for smaller screens
@@ -193,19 +174,13 @@ void Minicli::setMaxCommandBoxWidth()
     maxWidth = maxWidth * 2 / 5;
   }
 
-  m_dlg->cbCommand->setMaximumWidth(maxWidth);
+  return QSize(maxWidth, -1);
 }
 
 void Minicli::show()
 {
-  setMaxCommandBoxWidth();
   KWin::setState( winId(), NET::StaysOnTop );
-  setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
-  adjustSize();
-  setFixedSize(size());
   KDialog::show();
-  adjustSize();
-  setFixedSize(size());
 }
 
 void Minicli::loadConfig()
@@ -539,6 +514,7 @@ int Minicli::runCommand()
             KService::Ptr service = KService::serviceByDesktopName(cmd);
             if (service && service->isValid() && service->type() == "Application")
             {
+              notifyServiceStarted(service);
               KRun::run(*service, KURL::List());
               return 0;
             }
@@ -572,6 +548,7 @@ int Minicli::runCommand()
           KService::Ptr service = KService::serviceByDesktopName(cmd);
           if (service && service->isValid() && service->type() == "Application")
           {
+            notifyServiceStarted(service);
             KRun::run(*service, KURL::List());
             return 0;
           }
@@ -579,6 +556,7 @@ int Minicli::runCommand()
           service = KService::serviceByName(cmd);
           if (service && service->isValid() && service->type() == "Application")
           {
+            notifyServiceStarted(service);
             KRun::run(*service, KURL::List());
             return 0;
           }
@@ -602,6 +580,18 @@ int Minicli::runCommand()
   }
 }
 
+void Minicli::notifyServiceStarted(KService::Ptr service)
+{
+    // Inform other applications (like the quickstarter applet)
+    // that an application was started
+    QByteArray params;
+    QDataStream stream(params, IO_WriteOnly);
+    stream << "minicli" << service->storageId();
+    kdDebug() << "minicli appLauncher dcop signal: " << service->storageId() << endl;
+    KApplication::kApplication()->dcopClient()->emitDCOPSignal("appLauncher",
+        "serviceStartedByStorageId(QString,QString)", params);
+}
+
 void Minicli::slotCmdChanged(const QString& text)
 {
   bool isEmpty = text.isEmpty();
@@ -881,9 +871,7 @@ QString Minicli::calculate(const QString &exp)
 
 void Minicli::fontChange( const QFont & )
 {
-   setMaxCommandBoxWidth();
    adjustSize();
-   setFixedSize(size());
 }
 
 // vim: set et ts=2 sts=2 sw=2:
