@@ -14,7 +14,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
 
@@ -59,7 +59,7 @@ void Widgets::handleXGeometry(QWidget * dlg)
 bool Widgets::inputBox(QWidget *parent, const QString& title, const QString& text, const QString& init, QString &result)
 {
   bool ok;
-  QString str = KInputDialog::getText( title, text, init, &ok, parent );
+  QString str = KInputDialog::text( title, text, init, &ok, parent, 0, 0, QString::null );
   if ( ok )
     result = str;
   return ok;
