@@ -384,14 +384,8 @@ AccNoPass( const char *un, struct passwd *pw )
 	if (!*un)
 		return 0;
 
-	switch (cursource) {
-	case PWSRC_RELOGIN:
+	if (cursource != PWSRC_MANUAL)
 		return 1;
-	case PWSRC_AUTOLOGIN:
-		if (!strcmp( un, td->autoUser ))
-			return 1;
-		return 0;
-	}
 
 	for (hg = 0, fp = td->noPassUsers; *fp; fp++)
 		if (**fp == '@')
@@ -1143,6 +1137,8 @@ StartClient()
 		env = setEnv( env, "XDM_MANAGED", xma );
 		free( xma );
 	}
+	if (td->autoLock && cursource == PWSRC_AUTOLOGIN)
+		env = setEnv( env, "DESKTOP_LOCKED", "true" );
 	env = setEnv( env, "PATH", curuid ? td->userPath : td->systemPath );
 	env = setEnv( env, "SHELL", p->pw_shell );
 	env = setEnv( env, "HOME", p->pw_dir );
