@@ -1,4 +1,29 @@
 #! /usr/bin/perl -w
+#
+# Copyright 2004-2005 Oswald Buddenhagen <ossi@kde.org>
+#
+# Permission to use, copy, modify, distribute, and sell this software and its
+# documentation for any purpose is hereby granted without fee, provided that
+# the above copyright notice appear in all copies and that both that
+# copyright notice and this permission notice appear in supporting
+# documentation.
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+# IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+# OTHER DEALINGS IN THE SOFTWARE.
+#
+# Except as contained in this notice, the name of a copyright holders shall
+# not be used in advertising or otherwise to promote the sale, use or
+# other dealings in this Software without prior written authorization
+# from the copyright holders.
+#
 
 use strict;
 
@@ -23,7 +48,7 @@ sub dedb($)
 {
   my $t = shift;
   $t =~ s,</?(command|guilabel|quote)>,\",g;
-  $t =~ s,</?(acronym|envar|filename|option|systemitem( [^ ]+)?)>,,g;
+  $t =~ s,</?(acronym|envar|filename|option|systemitem( [^ >]+)?)>,,g;
   $t =~ s,<emphasis>([^<]+)</emphasis>,uc($1),ge;
   $t =~ s,&nbsp;, ,g;
   $t =~ s,&lt;,<,g;
@@ -211,7 +236,7 @@ while (<INFILE>) {
       if (/^\[(.*)\]$/) {
         defined($ex_sects{$1}) &&
           pegout("redefinition of example section [$1]");
-        push @ex_config, [$1, $comm, "", ""];
+        push @ex_config, [$1, dedb($comm), "", ""];
         $ex_sects{$1} = $#ex_config;
         $comm = "";
       } else {
@@ -230,11 +255,52 @@ while (<INFILE>) {
       last if (/^<\/docu>\n$/);
       $doc .= $_;
     }
+  } elsif (/^<legacy>$/) {
+    while (<INFILE>) {
+      next if (/^($|#)/);
+      my ($proc, $kif);
+      if (/^If: (.+)$/) {
+        $kif = $1;
+        getl();
+      } else {
+        $kif = "";
+      }
+      if (/^Proc: (.+)$/) {
+        $proc = $1;
+        getl();
+      } else {
+        pegout("expecting Proc keyword in legacy section");
+      }
+      my $nsrc = 0;
+      my $mcnt = 0;
+      while (/^Source: (.+)$/) {
+        my $src = $1;
+        if ($src =~ /^xdm:(.*)$/) {
+          my $what = $1;
+          my $dsp = ($what =~ s/^\*\.//);
+          my @oa = ([ "{ \"".$what."\", (char *)-1, 0, ".$proc." },", "XMO" ]);
+          add_cond($kif, $what, \@oa, \%ov_xm_conds);
+          $ov_xm[$dsp] .= $oa[0][0]." \\\n";
+        } elsif ($src =~ /^kdm:(.*)\/(.*)$/) {
+          my ($sec, $key) = ($1, $2);
+          my @oa = ([ "{ \"".$key."\", (char *)-1, 0, ".$proc." },", "KMO".($mcnt++) ]);
+          add_cond($kif, $key, \@oa, \%ov_km_conds);
+          $ov_km{$sec} .= $oa[0][0]." \\\n";
+        } else {
+          pegout("invalid legacy option '$_'");
+        }
+        $nsrc++;
+        getl();
+      }
+      $nsrc || pegout("no sources for legacy processor ".$proc);
+      last if (/^<\/legacy>$/);
+      pegout("unidentified section body '".$_."' in legacy section") if ($_);
+    }
   } else {
     next if (/^($|#)/);
     if (/^Key: (.+)$/) {
       my $key = $1;
-      $sect || pegout("definining key ".$key." outside any section");
+      $sect || pegout("defining key ".$key." outside any section");
       defined($key_names{$key}) &&
         pegout("redefinition of key ".$key." in section [".$sect."]");
       $key_names{$key} = "";
@@ -324,7 +390,7 @@ while (<INFILE>) {
       }
       my $nusers = 0;
       my ($vname, $kid, $xkid, $ctype, $cpptype, $cget, $cppget);
-      while(/^User: (.+)$/) {
+      while (/^User: (.+)$/) {
         my $user = $1;
         if ($user eq "dummy") {
           $vname = "dummy";
@@ -396,7 +462,7 @@ while (<INFILE>) {
       }
       $nusers || pegout("expecting User for key ".$key." in section [".$sect."]");
       my $ninsts = 0;
-      while(/^Instance: ?(.*)$/) {
+      while (/^Instance: ?(.*)$/) {
         my $inst = $1;
         if ($inst ne "-") {
           my $on = 1 - ($inst =~ s/^#//);
@@ -437,7 +503,7 @@ while (<INFILE>) {
         $prio = 0;
       }
       my $mcnt = 0;
-      while(/^Merge: (.+)$/) {
+      while (/^Merge: (.+)$/) {
         my $merge = $1;
         if ($merge =~ /^xdm(:([^\(]+))?(\((.+)\))?$/) {
           my ($what, $proc) = ($2, $4);
@@ -502,7 +568,7 @@ while (<INFILE>) {
           $desc ||
             print STDERR "Warning: key ".$key." in section [".$sect."] has empty Description\n";
           ($comm eq "&") &&
-            ($comm = dedb($desc));
+            ($comm = $desc);
           $desc = "<para>\n".$desc."</para>\n";
           if ($e_desc) {
             $e_desc = "<variablelist>\n".$e_desc."</variablelist>\n";
@@ -540,6 +606,7 @@ while (<INFILE>) {
       } else {
         $vname = "0";
       }
+	  $comm = dedb($comm);
       $comm =~ s/"/\\"/g;
       $comm =~ s/([^\n]*)\n/ \\\n\"# $1\\n\"/g;
       @oa = (
@@ -609,6 +676,8 @@ if (!$do_doc) {
 
 print OUTFILE
   "/* generated from $ARGV[0] by $0 - DO NOT EDIT! */\n\n".
+  "#ifndef CONFIG_DEFS\n".
+  "#define CONFIG_DEFS\n\n".
   $raw_out."\n\n".
   $ov_enum_defs."\n".
   $ov_defaults."\n\n";
@@ -738,6 +807,8 @@ print OUTFILE
   "KUpdSec kupsects[] = { \\\n".
   $ov_km_sect_refs.
   "};\n";
+print OUTFILE
+  "\n#endif /* CONFIG_DEFS */\n";
 
 } else {
 
