@@ -13,8 +13,8 @@
 
    You should have received a copy of the Lesser GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "dmctl.h"
@@ -22,6 +22,12 @@
 #ifdef Q_WS_X11
 
 #include <klocale.h>
+#include <dcopclient.h>
+
+#include <qregexp.h>
+
+#include <X11/Xauth.h>
+#include <X11/Xlib.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -31,7 +37,7 @@
 #include <fcntl.h>
 #include <errno.h>
 
-static enum { Dunno, NoDM, NewKDM, OldKDM/*, GDM*/ } DMType = Dunno;
+static enum { Dunno, NoDM, NewKDM, OldKDM, GDM } DMType = Dunno;
 static const char *ctl, *dpy;
 
 DM::DM() : fd( -1 )
@@ -46,6 +52,8 @@ DM::DM() : fd( -1 )
 			DMType = NewKDM;
 		else if ((ctl = ::getenv( "XDM_MANAGED" )) && ctl[0] == '/')
 			DMType = OldKDM;
+		else if (::getenv( "GDMSESSION" ))
+			DMType = GDM;
 		else
 			DMType = NoDM;
 	}
@@ -53,17 +61,25 @@ DM::DM() : fd( -1 )
 	default:
 		return;
 	case NewKDM:
+	case GDM:
 		if ((fd = ::socket( PF_UNIX, SOCK_STREAM, 0 )) < 0)
 			return;
 		sa.sun_family = AF_UNIX;
-		if ((ptr = strchr( dpy, ':' )))
-			ptr = strchr( ptr, '.' );
-		snprintf( sa.sun_path, sizeof(sa.sun_path),
-		          "%s/dmctl-%.*s/socket", ctl, ptr ? ptr - dpy : 512, dpy );
+		if (DMType == GDM)
+			strcpy( sa.sun_path, "/tmp/.gdm_socket" );
+		else {
+			if ((ptr = strchr( dpy, ':' )))
+				ptr = strchr( ptr, '.' );
+			snprintf( sa.sun_path, sizeof(sa.sun_path),
+			          "%s/dmctl-%.*s/socket",
+			          ctl, ptr ? ptr - dpy : 512, dpy );
+		}
 		if (::connect( fd, (struct sockaddr *)&sa, sizeof(sa) )) {
 			::close( fd );
 			fd = -1;
 		}
+		if (DMType == GDM)
+			GDMAuthenticate();
 		break;
 	case OldKDM:
 		{
@@ -90,9 +106,9 @@ DM::exec( const char *cmd )
 }
 
 /**
- * Execute a KDM remote control command.
+ * Execute a KDM/GDM remote control command.
  * @param cmd the command to execute. FIXME: undocumented yet.
- * @param ret the result buffer.
+ * @param buf the result buffer.
  * @return result:
  *  @li If true, the command was successfully executed.
  *   @p ret might contain addional results.
@@ -136,7 +152,8 @@ DM::exec( const char *cmd, QCString &buf )
 		len += tl;
 		if (buf[len - 1] == '\n') {
 			buf[len - 1] = 0;
-			if (len > 2 && buf[0] == 'o' && buf[1] == 'k' && buf[2] < 32)
+			if (len > 2 && (buf[0] == 'o' || buf[0] == 'O') &&
+			    (buf[1] == 'k' || buf[1] == 'K') && buf[2] <= 32)
 				ret = true;
 			break;
 		}
@@ -152,16 +169,21 @@ DM::canShutdown()
 
 	QCString re;
 
+	if (DMType == GDM)
+		return exec( "QUERY_LOGOUT_ACTION\n", re ) && re.find("HALT") >= 0;
+
 	return exec( "caps\n", re ) && re.find( "\tshutdown" ) >= 0;
 }
 
 void
 DM::shutdown( KApplication::ShutdownType shutdownType,
-              KApplication::ShutdownMode shutdownMode,
+              KApplication::ShutdownMode shutdownMode, /* NOT Default */
               const QString &bootOption )
 {
-	bool cap_ask;
+	if (shutdownType == KApplication::ShutdownTypeNone)
+		return;
 
+	bool cap_ask;
 	if (DMType == NewKDM) {
 		QCString re;
 		cap_ask = exec( "caps\n", re ) && re.find( "\tshutdown ask" ) >= 0;
@@ -171,10 +193,16 @@ DM::shutdown( KApplication::ShutdownType shutdownType,
 		cap_ask = false;
 	}
 	if (!cap_ask && shutdownMode == KApplication::ShutdownModeInteractive)
-		shutdownMode = KApplication::ShutdownModeDefault;
+		shutdownMode = KApplication::ShutdownModeForceNow;
 
-	if (shutdownType != KApplication::ShutdownTypeNone) {
-		QCString cmd( "shutdown\t" );
+	QCString cmd;
+	if (DMType == GDM) {
+		cmd.append( shutdownMode == KApplication::ShutdownModeForceNow ?
+		            "SET_LOGOUT_ACTION " : "SET_SAFE_LOGOUT_ACTION " );
+		cmd.append( shutdownType == KApplication::ShutdownTypeReboot ?
+		            "REBOOT\n" : "HALT\n" );
+	} else {
+		cmd.append( "shutdown\t" );
 		cmd.append( shutdownType == KApplication::ShutdownTypeReboot ?
 		            "reboot\t" : "halt\t" );
 		if (!bootOption.isNull())
@@ -185,8 +213,8 @@ DM::shutdown( KApplication::ShutdownType shutdownType,
 		            "forcenow\n" :
 		            shutdownMode == KApplication::ShutdownModeTryNow ?
 		            "trynow\n" : "schedule\n" );
-		exec( cmd.data() );
 	}
+	exec( cmd.data() );
 }
 
 bool
@@ -221,7 +249,8 @@ DM::bootOptions( QStringList &opts, int &defopt, int &current )
 void
 DM::setLock( bool on )
 {
-	exec( on ? "lock\n" : "unlock\n" );
+	if (DMType != GDM)
+		exec( on ? "lock\n" : "unlock\n" );
 }
 
 bool
@@ -230,6 +259,9 @@ DM::isSwitchable()
 	if (DMType == OldKDM)
 		return dpy[0] == ':';
 
+	if (DMType == GDM)
+		return exec( "QUERY_VT\n" );
+
 	QCString re;
 
 	return exec( "caps\n", re ) && re.find( "\tlocal" ) >= 0;
@@ -238,6 +270,9 @@ DM::isSwitchable()
 int
 DM::numReserve()
 {
+	if (DMType == GDM)
+		return 1; /* Bleh */
+
 	if (DMType == OldKDM)
 		return strstr( ctl, ",rsvd" ) ? 1 : -1;
 
@@ -252,7 +287,10 @@ DM::numReserve()
 void
 DM::startReserve()
 {
-	exec("reserve\n");
+	if (DMType == GDM)
+		exec("FLEXI_XSERVER\n");
+	else
+		exec("reserve\n");
 }
 
 bool
@@ -263,22 +301,39 @@ DM::localSessions( SessList &list )
 
 	QCString re;
 
-	if (!exec( "list\talllocal\n", re ))
-		return false;
-	QStringList sess = QStringList::split( QChar('\t'), re.data() + 3 );
-	for (QStringList::ConstIterator it = sess.begin(); it != sess.end(); ++it) {
-		QStringList ts = QStringList::split( QChar(','), *it, true );
-		SessEnt se;
-		se.display = ts[0];
-		if (ts[1][0] == '@')
-			se.from = ts[1].mid( 1 ), se.vt = 0;
-		else
-			se.vt = ts[1].mid( 2 ).toInt();
-		se.user = ts[2];
-		se.session = ts[3];
-		se.self = (ts[4].find( '*' ) >= 0);
-		se.tty = (ts[4].find( 't' ) >= 0);
-		list.append( se );
+	if (DMType == GDM) {
+		if (!exec( "CONSOLE_SERVERS\n", re ))
+			return false;
+		QStringList sess = QStringList::split( QChar(';'), re.data() + 3 );
+		for (QStringList::ConstIterator it = sess.begin(); it != sess.end(); ++it) {
+			QStringList ts = QStringList::split( QChar(','), *it, true );
+			SessEnt se;
+			se.display = ts[0];
+			se.user = ts[1];
+			se.vt = ts[2].toInt();
+			se.session = "<unknown>";
+			se.self = ts[0] == ::getenv( "DISPLAY" ); /* Bleh */
+			se.tty = false;
+			list.append( se );
+		}
+	} else {
+		if (!exec( "list\talllocal\n", re ))
+			return false;
+		QStringList sess = QStringList::split( QChar('\t'), re.data() + 3 );
+		for (QStringList::ConstIterator it = sess.begin(); it != sess.end(); ++it) {
+			QStringList ts = QStringList::split( QChar(','), *it, true );
+			SessEnt se;
+			se.display = ts[0];
+			if (ts[1][0] == '@')
+				se.from = ts[1].mid( 1 ), se.vt = 0;
+			else
+				se.vt = ts[1].mid( 2 ).toInt();
+			se.user = ts[2];
+			se.session = ts[3];
+			se.self = (ts[4].find( '*' ) >= 0);
+			se.tty = (ts[4].find( 't' ) >= 0);
+			list.append( se );
+		}
 	}
 	return true;
 }
@@ -297,8 +352,10 @@ DM::sess2Str2( const SessEnt &se, QString &user, QString &loc )
 					se.session == "<remote>" ?
 						i18n("X login on remote host") :
 						i18n("... host", "X login on %1").arg( se.session ) :
-				i18n("user: session type", "%1: %2")
-					.arg( se.user ).arg( se.session );
+				se.session == "<unknown>" ?
+					se.user :
+					i18n("user: session type", "%1: %2")
+						.arg( se.user ).arg( se.session );
 		loc =
 			se.vt ?
 				QString("%1, vt%2").arg( se.display ).arg( se.vt ) :
@@ -315,10 +372,63 @@ DM::sess2Str( const SessEnt &se )
 	return i18n("session (location)", "%1 (%2)").arg( user ).arg( loc );
 }
 
-void
+bool
 DM::switchVT( int vt )
 {
-	exec( QString("activate\tvt%1\n").arg(vt).latin1() );
+	if (DMType == GDM)
+		return exec( QString("SET_VT %1\n").arg(vt).latin1() );
+
+	return exec( QString("activate\tvt%1\n").arg(vt).latin1() );
+}
+
+void
+DM::lockSwitchVT( int vt )
+{
+	if (switchVT( vt ))
+		kapp->dcopClient()->send( "kdesktop", "KScreensaverIface", "lock()", "" );
+}
+
+void
+DM::GDMAuthenticate()
+{
+	FILE *fp;
+	const char *dpy, *dnum, *dne;
+	int dnl;
+	Xauth *xau;
+
+	dpy = DisplayString( QPaintDevice::x11AppDisplay() );
+	if (!dpy) {
+		dpy = ::getenv( "DISPLAY" );
+		if (!dpy)
+			return;
+	}
+	dnum = strchr( dpy, ':' ) + 1;
+	dne = strchr( dpy, '.' );
+	dnl = dne ? dne - dnum : strlen( dnum );
+
+	/* XXX should do locking */
+	if (!(fp = fopen( XauFileName(), "r" )))
+		return;
+
+	while ((xau = XauReadAuth( fp ))) {
+		if (xau->family == FamilyLocal &&
+		    xau->number_length == dnl && !memcmp( xau->number, dnum, dnl ) &&
+		    xau->data_length == 16 &&
+		    xau->name_length == 18 && !memcmp( xau->name, "MIT-MAGIC-COOKIE-1", 18 ))
+		{
+			QString cmd( "AUTH_LOCAL " );
+			for (int i = 0; i < 16; i++)
+				cmd += QString::number( (uchar)xau->data[i], 16 ).rightJustify( 2, '0');
+			cmd += "\n";
+			if (exec( cmd.latin1() )) {
+				XauDisposeAuth( xau );
+				break;
+			}
+		}
+		XauDisposeAuth( xau );
+	}
+
+	fclose (fp);
 }
 
 #endif // Q_WS_X11
