@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #ifndef KHC_FORMATTER_H
 #define KHC_FORMATTER_H
@@ -38,6 +38,8 @@ class Formatter
     virtual QString separator();
     virtual QString docTitle( const QString & );
     virtual QString sectionHeader( const QString & );
+    virtual QString paragraph( const QString & );
+    virtual QString title( const QString & );
     
     virtual QString processResult( const QString & );
 
