@@ -38,15 +38,17 @@ extern Windows* windows_handler;
 #define KHOTKEYS_DISABLE_COPY( cls ) private: cls( const cls& ); cls& operator=( const cls& )
 
 // CHECKME hmms :(
-bool khotkeys_active();
-void khotkeys_set_active( bool active_P );
+KDE_EXPORT bool khotkeys_active();
+KDE_EXPORT void khotkeys_set_active( bool active_P );
 
 QString get_menu_entry_from_path( const QString& path_P );
 
-void init_global_data( bool active_P, QObject* owner_P );
+KDE_EXPORT void init_global_data( bool active_P, QObject* owner_P );
 
 const char* const MENU_EDITOR_ENTRIES_GROUP_NAME = I18N_NOOP( "Menu Editor entries" );
 
+KDE_EXPORT bool haveArts();
+
 //***************************************************************************
 // Inline
 //***************************************************************************
