@@ -1,3 +1,5 @@
+// -*- c++ -*-
+
 /* kastasker.h
 **
 ** Copyright (C) 2001-2004 Richard Moore <rich@kde.org>
@@ -21,8 +23,8 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
@@ -51,38 +53,44 @@
 /*
 ** Bug reports and questions can be sent to kde-devel@kde.org
 */
-// -*- c++ -*-
-
 
 #ifndef KASTASKER_H
 #define KASTASKER_H
 
 #include "kasbar.h"
 
+#include <kdemacros.h>
+#include <taskmanager.h>
+
 class KConfig;
 class KPixmap;
+class KPopupMenu;
+class KAction;
+class KToggleAction;
 
-class TaskManager;
 class Task;
 class Startup;
 
 class KasTaskItem;
 class KasStartupItem;
 class KasGroupItem;
+class KasTasker;
+class KasGrouper;
 
 /**
  * A KasBar that provides a taskbar using the TaskManager API.
  *
  * @author Richard Moore, rich@kde.org
  */
-class KasTasker : public KasBar
+class KDE_EXPORT KasTasker : public KasBar
 {
     Q_OBJECT
+    Q_PROPERTY( bool isTopLevel READ isTopLevel )
+    Q_PROPERTY( bool showClock READ showClock )
+    Q_PROPERTY( bool showLoad READ showLoad )
 
 public:
-    /**
-     * Create a KasTasker widget.
-     */
+    /** Create a KasTasker widget. */
     KasTasker( Orientation o, QWidget* parent = 0, const char* name = 0, WFlags f = 0 );
 
     /**
@@ -92,184 +100,166 @@ public:
      */
     KasTasker( Orientation o, KasTasker *master,
 	       QWidget *parent=0, const char *name=0, WFlags f=0 );
-    /**
-     * Cleans up.
-     */
+
+    /** Cleans up. */
     virtual ~KasTasker();
 
-    /**
-     * Finds the item representing a task (if there is one).
-     */
-    KasTaskItem *findItem( Task * );
+    /** Factory method that returns the default menu for items in the bar. */
+    virtual KPopupMenu *contextMenu();
+
+    virtual KasTasker *createChildBar( Orientation o, QWidget *parent, const char *name=0 );
 
     /**
-     * Finds the item representing a startup (if there is one).
+     * Returns true if this is the top KasTasker. Note that it is possible for
+     * the top KasTasker to be a child of another KasBar derived class, so
+     * this can return a different result to KasBar::isTopLevel().
      */
-    KasStartupItem *findItem( Startup *s );
+    bool isTopLevel() const { return dynamic_cast<KasTasker *>( KasBar::master() ) ? true : false; }
+    KasTasker *master() const { return dynamic_cast<KasTasker *>( KasBar::master() ); }
 
-    /**
-     * Accessor for the min icon (singleton).
-     */
-    QBitmap *minIcon();
+    /** Finds the item representing a task (if there is one). */
+    KasTaskItem *findItem( Task::Ptr  );
 
-    /**
-     * Accessor for the max icon (singleton).
-     */
-    QBitmap *maxIcon();
+    /** Finds the item representing a startup (if there is one). */
+    KasStartupItem *findItem( Startup::Ptr s );
 
-    /**
-     * Accessor for the shade icon (singleton).
-     */
-    QBitmap *shadeIcon();
-
-    /**
-     * Accessor for the micro min icon (singleton).
-     */
-    QPixmap *microMinIcon();
-
-    /**
-     * Accessor for the micro max icon (singleton).
-     */
-    QPixmap *microMaxIcon();
-
-    /**
-     * Accessor for the micro shade icon (singleton).
-     */
-    QPixmap *microShadeIcon();
-
-    /**
-     * Returns true iff thumbnails are enabled.
-     */
+    /** Returns true iff thumbnails are enabled. */
     bool thumbnailsEnabled() const { return enableThumbs_; }
 
-    /**
-     * Returns the proportions of the window thumbnails.
-     */
+    /** Returns true iff thumbnails should be shown in the boxes instead of icons. */
+    bool embedThumbnails() const { return embedThumbs_; }
+
+    bool showClock() const { return showClock_; }
+    bool showLoad() const { return showLoad_; }
+
+    /** Returns the proportions of the window thumbnails. */
     double thumbnailSize() const { return thumbnailSize_; }
 
-    /**
-     * Returns true iff the startup notifier is enabled.
-     */
+    /** Returns true iff the startup notifier is enabled. */
     bool notifierEnabled() const { return enableNotifier_; }
 
-    /**
-     * Returns true iff the modified flag should be shown.
-     */
+    /** Returns true iff the modified flag should be shown. */
     bool showModified() const { return showModified_; }
 
+    /** Returns true iff a progress bar should be shown for progress windows. */
     bool showProgress() const { return showProgress_; }
 
-    /**
-     * Returns true iff windows from all desktops should be displays.
-     */
+    /** Returns true iff we should indicate when a window demands attention. */
+    bool showAttention() const { return showAttention_; }
+
+    /** Returns true iff windows from all desktops should be displayed. */
     bool showAllWindows() const { return showAllWindows_; }
 
-    /**
-     * Returns the delay between thumbnail updates (in seconds).
-     */
+    /** Returns the delay between thumbnail updates (in seconds). */
     int thumbnailUpdateDelay() const { return thumbUpdateDelay_; }
 
-    /**
-     * Returns true iff windows should be grouped together.
-     */
+    /** Returns true iff windows should be grouped together. */
     bool groupWindows() const { return groupWindows_; }
 
+    /** Returns true iff windows on inactive desktops should be grouped together. */
+    bool groupInactiveDesktops() const { return groupInactiveDesktops_; }
+
+    /** Returns true iff we should only show windows that are minimized. */
+    bool onlyShowMinimized() const { return onlyShowMinimized_; }
+
+    /** Returns true if this bar is floating. */
+    bool isStandAlone() const { return standalone_; }
+
     //
     // Internal stuff
     //
 
-    /**
-     * Returns the TaskManager object.
-     */
-    TaskManager *taskManager() const { return manager; }
-
     /**
      * Converts the item for a task into a group item to which additional
      * tasks can be added.
      */
-    KasGroupItem *convertToGroup( Task *t );
+    KasGroupItem *convertToGroup( Task::Ptr t );
 
-    /**
-     * Moves an item from a group into the main bar.
-     */
-    void moveToMain( KasGroupItem *gi, Task *t );
+    /** Moves an item from a group into the main bar. */
+    void moveToMain( KasGroupItem *gi, Task::Ptr t );
 
-    /**
-     * If the task should be part of a group then make it so.
-     */
-    KasItem *maybeAddToGroup( Task *t );
+    /** Moves all the items from a group into the main bar and removes the group. */
+    void moveToMain( KasGroupItem *gi );
 
 public slots:
-    /**
-     * Adds a task to the bar.
-     */
-    void addTask( Task * );
+    /** Adds a task to the bar. */
+    void addTask( Task::Ptr  );
 
-    /**
-     * Removes a task from the bar.
-     */
-    void removeTask( Task * );
+    /** Removes a task from the bar. */
+    void removeTask( Task::Ptr  );
 
-    /**
-     * Adds a startup item to the bar.
-     */
-    void addStartup( Startup * );
+    /** Adds a startup item to the bar. */
+    void addStartup( Startup::Ptr  );
 
-    /**
-     * Removes a startup item from the bar.
-     */
-    void removeStartup( Startup * );
+    /** Removes a startup item from the bar. */
+    void removeStartup( Startup::Ptr  );
 
     void refreshAll();
+    void refreshAllLater();
     void refreshIconGeometry();
 
     void setNotifierEnabled( bool enable );
+    void setShowModified( bool enable );
+    void setShowProgress( bool enable );
+    void setShowAttention( bool enable );
+
+    void setShowAllWindows( bool enable );
+    void setGroupWindows( bool enable );
+    void setGroupInactiveDesktops( bool enable );
+    void setOnlyShowMinimized( bool enable );
+
     void setThumbnailSize( double size );
     void setThumbnailSize( int percent );
     void setThumbnailsEnabled( bool enable );
-    void setShowModified( bool enable );
-    void setShowProgress( bool enable );
-    void setShowAllWindows( bool enable );
     void setThumbnailUpdateDelay( int secs );
-    void setGroupWindows( bool enable );
+    void setEmbedThumbnails( bool enable );
+
+    void setShowClock( bool enable );
+    void setShowLoad( bool enable );
 
     void showPreferences();
     void showAbout();
 
-    /**
-     * Sets the current KConfig object.
-     */
+    /** Sets the current KConfig object. */
     void setConfig( KConfig *config );
     KConfig *config() const { return conf; }
 
-
-    /**
-     * Reads the settings from the current KConfig.
-     */
+    /** Reads the settings from the current KConfig. */
     void readConfig();
+    void writeConfig();
+    void writeConfigLater();
+    void writeLayout();
 
-    /**
-     * Rereads the configuration of the master Kasbar.
-     */
-    void rereadMaster();
+    /** Writes the settings of this bar to the specified KConfig. */
+    void writeConfig( KConfig *conf );
+
+    void setStandAlone( bool enable );
+
+signals:
+    void showAllWindowsChanged( bool );
+    void groupWindowsChanged( bool );
+    void showClockChanged( bool );
+    void showLoadChanged( bool );
 
 protected slots:
-    /**
-     * Load settings from the specified configuration.
-     */
+    /** Load settings from the specified configuration. */
     void readConfig( KConfig *conf );
 
 private:
-    KasTasker *master_;
-    TaskManager *manager;
-    QBitmap *minPix;
-    QBitmap *maxPix;
-    QBitmap *shadePix;
-    QPixmap *microShadePix;
-    QPixmap *microMaxPix;
-    QPixmap *microMinPix;
+    KPopupMenu *menu;
+    KConfig *conf;
+    KasGrouper *grouper;
+    KToggleAction *toggleDetachedAction;
+    KToggleAction *showAllWindowsAction;
+    KToggleAction *groupWindowsAction;
+    KToggleAction *showClockAction;
+    KToggleAction *showLoadAction;
+    KAction *rotateBarAction;
+    bool standalone_;
+
     bool passive_;
     bool enableThumbs_;
+    bool embedThumbs_;
     double thumbnailSize_;
     bool enableNotifier_;
     bool showModified_;
@@ -277,7 +267,13 @@ private:
     bool showAllWindows_;
     int thumbUpdateDelay_;
     bool groupWindows_;
-    KConfig *conf;
+    bool groupInactiveDesktops_;
+    bool showAttention_;
+    bool onlyShowMinimized_;
+    bool showClock_;
+    KasItem *clockItem;
+    bool showLoad_;
+    KasItem *loadItem;
 };
 
 #endif // KASTASKER_H
