@@ -27,25 +27,48 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <sys/types.h>
 
-#include <qpoint.h>
 #include <qobject.h>
-#include <qvaluelist.h>
+#include <qpixmap.h>
+#include <qpoint.h>
 #include <qptrlist.h>
 #include <qpixmap.h>
+#include <qdragobject.h>
+#include <qrect.h>
+#include <qvaluelist.h>
+#include <qvaluevector.h>
 
-#include <dcopobject.h>
-#include <kwin.h>
+#include <ksharedptr.h>
 #include <kstartupinfo.h>
+#include <kwin.h>
 
+#include <config.h>
+
+#if defined(HAVE_XCOMPOSITE) && \
+    defined(HAVE_XRENDER) && \
+    defined(HAVE_XFIXES)
+#include <X11/Xlib.h>
+#include <X11/extensions/Xcomposite.h>
+#include <X11/extensions/Xfixes.h>
+#include <X11/extensions/Xrender.h>
+#if XCOMPOSITE_VERSION >= 00200 && \
+    XFIXES_VERSION >= 20000 && \
+    (RENDER_MAJOR > 0 || RENDER_MINOR >= 6)
+#define THUMBNAILING_POSSIBLE
+#endif
+#endif
+
+class KWinModule;
 class TaskManager;
 
+typedef QValueList<WId> WindowList;
+
 /**
  * A dynamic interface to a task (main window).
  *
  * @see TaskManager
  * @see KWinModule
  */
-class Task: public QObject
+class KDE_EXPORT Task: public QObject, public KShared
 {
     Q_OBJECT
     Q_PROPERTY( QString visibleIconicName READ visibleIconicName )
@@ -72,12 +95,16 @@ class Task: public QObject
     Q_PROPERTY( QPixmap thumbnail READ thumbnail )
 
 public:
-    Task( WId win, TaskManager * parent, const char *name = 0 );
+    typedef KSharedPtr<Task> Ptr;
+    typedef QValueVector<Task::Ptr> List;
+    typedef QMap<WId, Task::Ptr> Dict;
+
+    Task(WId win, QObject *parent, const char *name = 0);
     virtual ~Task();
 
-    TaskManager* taskManager() const { return (TaskManager*) parent(); }
-
     WId window() const { return _win; }
+    KWin::WindowInfo info() const { return _info; }
+
 #if 0 // this would use (_NET_)WM_ICON_NAME, which is shorter, but can be different from window name
     QString visibleIconicName() const { return _info.visibleIconName(); }
     QString visibleIconicNameWithState() const { return _info.visibleIconNameWithState(); }
@@ -97,7 +124,7 @@ public:
      * A list of the window ids of all transient windows (dialogs) associated
      * with this task.
      */
-    QValueList<WId> transients() const { return _transients; }
+    WindowList transients() const { return _transients; }
 
     /**
      * Returns a 16x16 (KIcon::Small) icon for the task. This method will
@@ -189,6 +216,17 @@ public:
      */
     bool isAlwaysOnTop() const;
 
+    /**
+     * Returns true if the task's window will remain at the bottom of the
+     * stacking order.
+     */
+    bool isKeptBelowOthers() const;
+
+    /**
+     * Returns true if the task's window is in full screen mode
+     */
+    bool isFullScreen() const;
+
     /**
      * Returns true if the document the task is editing has been modified.
      * This is currently handled heuristically by looking for the string
@@ -201,22 +239,50 @@ public:
      * Returns the desktop on which this task's window resides.
      */
     int desktop() const { return _info.desktop(); }
-    
+
     /**
      * Returns true if the task is not active but demands user's attention.
      */
     bool demandsAttention() const;
 
+
+    /**
+    * Returns true if the window is on the specified screen of a multihead configuration
+    */
+    bool isOnScreen( int screen ) const;
+
+    /**
+     * Returns true if the task should be shown in taskbar-like apps
+     */
+    bool showInTaskbar() const { return _info.state() ^ NET::SkipTaskbar; }
+
+    /**
+     * Returns true if the task should be shown in pager-like apps
+     */
+    bool showInPager() const { return _info.state() ^ NET::SkipPager; }
+
+    /**
+     * Returns the geometry for this window
+     */
+    QRect geometry() const { return _info.geometry(); }
+
+    /**
+     * Returns the geometry for the from of this window
+     */
+    QRect frameGeometry() const { return _info.frameGeometry(); }
+
     // internal
 
     //* @internal
-    void refresh(bool icon = false);
+    void refresh(unsigned int dirty);
+    //* @internal
+    void refreshIcon();
     //* @internal
     void addTransient( WId w, const NETWinInfo& info );
     //* @internal
     void removeTransient( WId w );
     //* @internal
-    bool hasTransient( WId w ) const { return _transients.contains( w ); }
+    bool hasTransient(WId w) const { return _transients.find(w) != _transients.end(); }
     //* @internal
     void updateDemandsAttentionState( WId w );
     //* @internal
@@ -248,23 +314,39 @@ public:
      */
     const QPixmap &thumbnail() const { return _thumb; }
 
+    QPixmap thumbnail(int maxDimension);
+
+    void updateWindowPixmap();
+
 public slots:
     // actions
 
     /**
      * Maximise the main window of this task.
      */
-    void maximize();
+    void setMaximized(bool);
+    void toggleMaximized();
 
     /**
      * Restore the main window of the task (if it was iconified).
      */
     void restore();
 
+    /**
+     * Move the window of this task.
+     */
+    void move();
+
+    /**
+     * Resize the window of this task.
+     */
+    void resize();
+
     /**
      * Iconify the task.
      */
-    void iconify();
+    void setIconified(bool);
+    void toggleIconified();
 
     /**
      * Close the task's window.
@@ -299,6 +381,18 @@ public slots:
     void setAlwaysOnTop(bool);
     void toggleAlwaysOnTop();
 
+    /**
+     * If true, the task's window will remain at the bottom of the stacking order.
+     */
+    void setKeptBelowOthers(bool);
+    void toggleKeptBelowOthers();
+
+    /**
+     * If true, the task's window will enter full screen mode.
+     */
+    void setFullScreen(bool);
+    void toggleFullScreen();
+
     /**
      * If true then the task's window will be shaded. Most window managers
      * represent this state by displaying on the window's title bar.
@@ -359,13 +453,17 @@ protected slots:
     //* @internal
     void generateThumbnail();
 
+protected:
+    void findWindowFrameId();
+
 private:
     bool                _active;
     WId                 _win;
+    WId                 m_frameId;
     QPixmap             _pixmap;
     KWin::WindowInfo    _info;
-    QValueList<WId>     _transients;
-    QValueList<WId>     _transients_demanding_attention;
+    WindowList          _transients;
+    WindowList          _transients_demanding_attention;
 
     int                 _lastWidth;
     int                 _lastHeight;
@@ -375,16 +473,45 @@ private:
     double _thumbSize;
     QPixmap _thumb;
     QPixmap _grab;
-
-    class TaskPrivate *d;
+    QRect m_iconGeometry;
+#ifdef THUMBNAILING_POSSIBLE
+    Pixmap              m_windowPixmap;
+#endif // THUMBNAILING_POSSIBLE
 };
 
+
+/**
+ * Provids a drag object for tasks across desktops.
+ */
+class KDE_EXPORT TaskDrag : public QStoredDrag
+{
+public:
+    /**
+     * Constructs a task drag object for a task list.
+     */
+    TaskDrag(const Task::List& tasks, QWidget* source = 0,
+             const char* name = 0);
+    ~TaskDrag();
+
+    /**
+     * Returns true if the mime source can be decoded to a TaskDrag.
+     */
+    static bool canDecode( const QMimeSource* e );
+
+    /**
+     * Decodes the tasks from the mime source and returns them if successful.
+     * Otherwise an empty task list is returned.
+     */
+    static Task::List decode( const QMimeSource* e );
+};
+
+
 /**
  * Represents a task which is in the process of starting.
  *
  * @see TaskManager
  */
-class Startup: public QObject
+class KDE_EXPORT Startup: public QObject, public KShared
 {
     Q_OBJECT
     Q_PROPERTY( QString text READ text )
@@ -392,6 +519,9 @@ class Startup: public QObject
     Q_PROPERTY( QString icon READ icon )
 
 public:
+    typedef KSharedPtr<Startup> Ptr;
+    typedef QValueVector<Startup::Ptr> List;
+
     Startup( const KStartupInfoId& id, const KStartupInfoData& data, QObject * parent,
         const char *name = 0);
     virtual ~Startup();
@@ -425,9 +555,6 @@ private:
     class StartupPrivate *d;
 };
 
-typedef QPtrList<Task> TaskList;
-typedef QPtrList<Startup> StartupList;
-
 
 /**
  * A generic API for task managers. This class provides an easy way to
@@ -438,27 +565,35 @@ typedef QPtrList<Startup> StartupList;
  * @see Startup
  * @see KWinModule
  */
-class TaskManager : public QObject
+class KDE_EXPORT TaskManager : public QObject
 {
     Q_OBJECT
     Q_PROPERTY( int currentDesktop READ currentDesktop )
     Q_PROPERTY( int numberOfDesktops READ numberOfDesktops )
 
 public:
-    TaskManager( QObject *parent = 0, const char *name = 0 );
-    virtual ~TaskManager();
+    static TaskManager* the();
+    ~TaskManager();
 
     /**
-     * Returns a list of all current tasks. Return type changed to
-     * QPtrList in KDE 3.
+     * Returns the task for a given WId, or 0 if there is no such task.
      */
-    TaskList tasks() const { return _tasks; }
+    Task::Ptr findTask(WId w);
 
     /**
-     * Returns a list of all current startups. Return type changed to
-     * QPtrList in KDE 3.
+     * Returns the task for a given location, or 0 if there is no such task.
      */
-    StartupList startups() const { return _startups; }
+    Task::Ptr findTask(int desktop, const QPoint& p);
+
+    /**
+     * Returns a list of all current tasks.
+     */
+    Task::Dict tasks() const { return m_tasksByWId; }
+
+    /**
+     * Returns a list of all current startups.
+     */
+    Startup::List startups() const { return _startups; }
 
     /**
      * Returns the name of the nth desktop.
@@ -478,29 +613,47 @@ public:
     /**
      * Returns true if the specified task is on top.
      */
-    bool isOnTop( const Task*);
+    bool isOnTop(const Task*);
+
+    /**
+     * Tells the task manager whether or not we care about geometry
+     * updates. This generates a lot of activity so should only be used
+     * when necessary.
+     */
+    void trackGeometry() { m_trackGeometry = true; }
+
+    /**
+    * Returns whether the Window with WId wid is on the screen screen
+    */
+    static bool isOnScreen( int screen, const WId wid );
+
+    KWinModule* winModule() const { return m_winModule; }
+
+    void setXCompositeEnabled(bool state);
+    static bool xCompositeEnabled() { return m_xCompositeEnabled != 0; }
+
 signals:
     /**
      * Emitted when a new task has started.
      */
-    void taskAdded(Task*);
+    void taskAdded(Task::Ptr);
 
     /**
      * Emitted when a task has terminated.
      */
-    void taskRemoved(Task*);
+    void taskRemoved(Task::Ptr);
 
     /**
      * Emitted when a new task is expected.
      */
-    void startupAdded(Startup*);
+    void startupAdded(Startup::Ptr);
 
     /**
      * Emitted when a startup item should be removed. This could be because
      * the task has started, because it is known to have died, or simply
      * as a result of a timeout.
      */
-    void startupRemoved(Startup*);
+    void startupRemoved(Startup::Ptr);
 
     /**
      * Emitted when the current desktop changes.
@@ -510,7 +663,8 @@ signals:
     /**
      * Emitted when a window changes desktop.
      */
-    void windowChanged(WId);
+    void windowChanged(Task::Ptr);
+    void windowChangedGeometry(Task::Ptr);
 
 protected slots:
     //* @internal
@@ -527,28 +681,30 @@ protected slots:
     //* @internal
     void killStartup( const KStartupInfoId& );
     //* @internal
-    void killStartup(Startup*);
+    void killStartup(Startup::Ptr);
 
     //* @internal
     void gotNewStartup( const KStartupInfoId&, const KStartupInfoData& );
     //* @internal
     void gotStartupChange( const KStartupInfoId&, const KStartupInfoData& );
-    //* @internal
-    void gotRemoveStartup( const KStartupInfoId& );
 
 protected:
-    /**
-     * Returns the task for a given WId, or 0 if there is no such task.
-     */
-    Task* findTask(WId w);
     void configure_startup();
+    void updateWindowPixmap(WId);
 
 private:
-    Task*               _active;
-    TaskList           _tasks;
-    QValueList< WId >   _skiptaskbar_windows;
-    StartupList         _startups;
-    KStartupInfo*       _startup_info;
+    TaskManager();
+
+    Task::Ptr               _active;
+    Task::Dict m_tasksByWId;
+    WindowList _skiptaskbar_windows;
+    Startup::List _startups;
+    KStartupInfo* _startup_info;
+    KWinModule* m_winModule;
+    bool m_trackGeometry;
+
+    static TaskManager* m_self;
+    static uint m_xCompositeEnabled;
 
     class TaskManagerPrivate *d;
 };
