@@ -239,7 +239,7 @@ void SMBSlave::reportError(const SMBUrl &url)
     {
     case ENOENT:
         if (url.getType() == SMBURLTYPE_ENTIRE_NETWORK)
-            error( ERR_SLAVE_DEFINED, i18n("Unable to find any workgroups in your local network."));
+            error( ERR_SLAVE_DEFINED, i18n("Unable to find any workgroups in your local network. This might be caused by an enabled firewall."));
         else
             error( ERR_DOES_NOT_EXIST, url.prettyURL());
         break;
@@ -283,6 +283,9 @@ void SMBSlave::reportError(const SMBUrl &url)
     case EBADF:
         error( ERR_INTERNAL, i18n("BAD File descriptor"));
         break;
+    case ETIMEDOUT:
+        error( ERR_SERVER_TIMEOUT, url.host() );
+        break;
 #ifdef ENOTUNIQ
     case ENOTUNIQ:
         error( ERR_SLAVE_DEFINED, i18n( "The given name could not be resolved to a unique server. "
@@ -341,7 +344,9 @@ void SMBSlave::listDir( const KURL& kurl )
            // Set name
            atom.m_uds = KIO::UDS_NAME;
            QString dirpName = QString::fromUtf8( dirp->name );
-           QString comment = QString::fromUtf8( dirp->comment, dirp->commentlen - 1);
+           // We cannot trust dirp->commentlen has it might be with or without the NUL character
+           // See KDE bug #111430 and Samba bug #3030
+           QString comment = QString::fromUtf8( dirp->comment );
            if ( dirp->smbc_type == SMBC_SERVER || dirp->smbc_type == SMBC_WORKGROUP ) {
                atom.m_str = dirpName.lower();
                atom.m_str.at( 0 ) = dirpName.at( 0 ).upper();
