@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <errno.h>
@@ -57,9 +57,11 @@ static const char version[] = "2.0";
 
 QCString clientApp::startup_id_str;
 bool clientApp::m_ok;
+bool s_interactive = true;
 
 static const KCmdLineOptions options[] =
 {
+   { "noninteractive", I18N_NOOP("Non interactive use: no message boxes"), 0},
    { "commands", I18N_NOOP("Show available commands"), 0},
    { "+command", I18N_NOOP("Command (see --commands)"), 0},
    { "+[URL(s)]", I18N_NOOP("Arguments for command"), 0},
@@ -446,6 +448,9 @@ bool clientApp::doIt()
   int argc = args->count();
   checkArgumentCount(argc, 1, 0);
 
+  if ( !args->isSet( "ninteractive" ) ) {
+      s_interactive = false;
+  }
   QCString command = args->arg(0);
 
   // read ASN env. variable for non-KApp cases
@@ -536,6 +541,8 @@ bool clientApp::doIt()
       srcLst.append( args->url(i) );
 
     KIO::Job * job = KIO::move( srcLst, args->url(argc - 1) );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -564,11 +571,13 @@ bool clientApp::doIt()
        return m_ok;
     QString dst =
        KFileDialog::getSaveFileName( (argc<2) ? (QString::null) : (args->url(1).filename()) );
-    if (dst == QString::null)
+    if (dst.isEmpty()) // cancelled
        return m_ok; // AK - really okay?
     KURL dsturl;
     dsturl.setPath( dst );
     KIO::Job * job = KIO::copy( srcLst, dsturl );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -581,6 +590,8 @@ bool clientApp::doIt()
       srcLst.append( args->url(i) );
 
     KIO::Job * job = KIO::copy( srcLst, args->url(argc - 1) );
+    if ( !s_interactive )
+        job->setInteractive( false );
     connect( job, SIGNAL( result( KIO::Job * ) ), &app, SLOT( slotResult( KIO::Job * ) ) );
     app.exec();
     return m_ok;
@@ -617,7 +628,7 @@ bool clientApp::doIt()
 
 void clientApp::slotResult( KIO::Job * job )
 {
-  if (job->error())
+  if (job->error() && s_interactive)
     job->showErrorDialog();
   m_ok = !job->error();
   quit();
