@@ -1,5 +1,4 @@
-// -*- mode:cperl; cperl-indent-level:4; cperl-continued-statement-offset:4; indent-tabs-mode:nil -*-
-// vim: set ts=4 sts=4 sw=4 et:
+// kate: space-indent on; indent-width 3; replace-tabs on;
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
    Copyright (C) 2002-2003 Alexander Kellett <lypanov@kde.org>
@@ -15,8 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef __commands_h
@@ -25,36 +24,72 @@
 #include <kcommand.h>
 #include <kbookmark.h>
 
-class CreateCommand : public KCommand
+// Interface adds the affectedBookmarks method
+// Any class should on call add those bookmarks which are 
+// affected by executing or unexecuting the command
+// Or a common parent of the affected bookmarks
+// see KBookmarkManager::notifyChange(KBookmarkGroup)
+class IKEBCommand
+{
+public:
+   IKEBCommand() {};
+   virtual ~IKEBCommand() {};
+   virtual QString affectedBookmarks() const = 0;
+   virtual QString currentAddress() const { return QString::null; }
+};
+
+class KEBMacroCommand : public KMacroCommand, public IKEBCommand
+{
+public:
+   KEBMacroCommand(const QString &name)
+      : KMacroCommand(name) {};
+   virtual ~KEBMacroCommand() {};
+   virtual QString affectedBookmarks() const;
+};
+
+class DeleteManyCommand : public KEBMacroCommand
+{
+public:
+   DeleteManyCommand(const QString &name, const QValueList<QString>  & addresses);
+   virtual ~DeleteManyCommand() {};
+   virtual QString currentAddress() const;
+private:
+   QString prevOrParentAddress(QString addr);
+   QString preOrderNextAddress(QString addr);
+   bool isConsecutive(const QValueList<QString> & addresses);
+   QString m_currentAddress;
+};
+
+class CreateCommand : public KCommand, public IKEBCommand
 {
 public:
    // separator
-   CreateCommand(const QString &address, bool konqi = false)
+   CreateCommand(const QString &address)
       : KCommand(), m_to(address),
-        m_group(false), m_separator(true), m_konqi(konqi), m_originalBookmark(QDomElement())
+        m_group(false), m_separator(true), m_originalBookmark(QDomElement())
    { ; }
 
    // bookmark
    CreateCommand(const QString &address,
                  const QString &text, const QString &iconPath, 
-                 const KURL &url, bool konqi = false)
+                 const KURL &url)
       : KCommand(), m_to(address), m_text(text), m_iconPath(iconPath), m_url(url),
-        m_group(false), m_separator(false), m_konqi(konqi), m_originalBookmark(QDomElement())
+        m_group(false), m_separator(false), m_originalBookmark(QDomElement())
    { ; }
 
    // folder
    CreateCommand(const QString &address,
                  const QString &text, const QString &iconPath, 
-                 bool open, bool konqi = false)
+                 bool open)
       : KCommand(), m_to(address), m_text(text), m_iconPath(iconPath),
-        m_group(true), m_separator(false), m_open(open), m_konqi(konqi), m_originalBookmark(QDomElement())
+        m_group(true), m_separator(false), m_open(open), m_originalBookmark(QDomElement())
    { ; }
 
    // clone existing bookmark
    CreateCommand(const QString &address,
                  const KBookmark &original, const QString &name = QString::null)
       : KCommand(), m_to(address), m_group(false), m_separator(false),
-        m_open(false), m_konqi(false), m_originalBookmark(original), m_mytext(name)
+        m_open(false), m_originalBookmark(original), m_mytext(name)
    { ; }
 
    QString finalAddress() const;
@@ -63,6 +98,8 @@ public:
    virtual void execute();
    virtual void unexecute();
    virtual QString name() const;
+   virtual QString affectedBookmarks() const;
+   virtual QString currentAddress() const;
 private:
    QString m_to;
    QString m_text;
@@ -71,12 +108,11 @@ private:
    bool m_group:1;
    bool m_separator:1;
    bool m_open:1;
-   bool m_konqi:1;
    KBookmark m_originalBookmark;
    QString m_mytext;
 };
 
-class EditCommand : public KCommand
+class EditCommand : public KCommand, public IKEBCommand
 {
 public:
 
@@ -100,10 +136,14 @@ public:
                const QString &name = QString::null)
       : KCommand(), m_address(address), m_editions(editions), m_mytext(name)
    { ; }
+
+   void modify(const QString & a, const QString & v);
+
    virtual ~EditCommand() { ; }
    virtual void execute();
    virtual void unexecute();
    virtual QString name() const;
+   virtual QString affectedBookmarks() const;
 private:
    QString m_address;
    QValueList<Edition> m_editions;
@@ -111,7 +151,7 @@ private:
    QString m_mytext;
 };
 
-class NodeEditCommand : public KCommand
+class NodeEditCommand : public KCommand, public IKEBCommand
 {
 public:
    NodeEditCommand(const QString &address, 
@@ -119,9 +159,13 @@ public:
                    const QString &nodeName)
       : KCommand(), m_address(address), m_newText(newText), m_nodename(nodeName)
    { ; }
+
+   void modify(const QString & newText);
+
    virtual ~NodeEditCommand() { ; }
    virtual void execute();
    virtual void unexecute();
+   virtual QString affectedBookmarks() const;
    virtual QString name() const;
    static QString getNodeText(KBookmark bk, const QStringList &nodehier);
    static QString setNodeText(KBookmark bk, const QStringList &nodehier, 
@@ -133,28 +177,29 @@ private:
    QString m_nodename;
 };
 
-class DeleteCommand : public KCommand
+class DeleteCommand : public KCommand, public IKEBCommand
 {
 public:
    DeleteCommand(const QString &from, bool contentOnly = false)
       : KCommand(), m_from(from), m_cmd(0L), m_subCmd(0L), m_contentOnly(contentOnly)
    { ; }
-   virtual ~DeleteCommand() { delete m_cmd; }
+   virtual ~DeleteCommand() { delete m_cmd; delete m_subCmd;}
    virtual void execute();
    virtual void unexecute();
    virtual QString name() const { 
       // NOTE - DeleteCommand needs no name, its always embedded in a macrocommand
       return ""; 
    };
-   static KMacroCommand* deleteAll(const KBookmarkGroup &parentGroup);
+   virtual QString affectedBookmarks() const;
+   static KEBMacroCommand* deleteAll(const KBookmarkGroup &parentGroup);
 private:
    QString m_from;
    KCommand *m_cmd;
-   KMacroCommand *m_subCmd;
+   KEBMacroCommand *m_subCmd;
    bool m_contentOnly;
 };
 
-class MoveCommand : public KCommand
+class MoveCommand : public KCommand, public IKEBCommand
 {
 public:
    // "Create it with itemsAlreadyMoved=true since 
@@ -167,6 +212,7 @@ public:
    virtual void execute();
    virtual void unexecute();
    virtual QString name() const;
+   virtual QString affectedBookmarks() const;
 private:
    QString m_from;
    QString m_to;
@@ -175,16 +221,17 @@ private:
 
 class SortItem;
 
-class SortCommand : public KMacroCommand
+class SortCommand : public KEBMacroCommand
 {
 public:
    SortCommand(const QString &name, const QString &groupAddress)
-      : KMacroCommand(name), m_groupAddress(groupAddress) 
+      : KEBMacroCommand(name), m_groupAddress(groupAddress) 
    { ; }
    virtual ~SortCommand() 
    { ; }
    virtual void execute();
    virtual void unexecute();
+   virtual QString affectedBookmarks() const;
    // internal
    void moveAfter(const SortItem &moveMe, const SortItem &afterMe);
 private:
@@ -195,16 +242,14 @@ class KEBListViewItem;
 
 class CmdGen {
 public:
-   static CmdGen* self() { if (!s_self) s_self = new CmdGen(); return s_self; }
-   KMacroCommand* setAsToolbar(const KBookmark &bk);
-   KMacroCommand* setShownInToolbar(const KBookmark &bk, bool show);
-   bool shownInToolbar(const KBookmark &bk);
-   KMacroCommand* deleteItems(const QString &commandName, QPtrList<KEBListViewItem> *items);
-   KMacroCommand* insertMimeSource(const QString &cmdName, QMimeSource *data, const QString &addr);
-   KMacroCommand* itemsMoved(QPtrList<KEBListViewItem> *items, const QString &newAddress, bool copy);
+   static KEBMacroCommand* setAsToolbar(const KBookmark &bk);
+   static KEBMacroCommand* setShownInToolbar(const KBookmark &bk, bool show);
+   static bool shownInToolbar(const KBookmark &bk);
+   static KEBMacroCommand* deleteItems(const QString &commandName, const QMap<KEBListViewItem *, bool> & items);
+   static KEBMacroCommand* insertMimeSource(const QString &cmdName, QMimeSource *data, const QString &addr);
+   static KEBMacroCommand* itemsMoved(const QMap<KEBListViewItem *, bool> & items, const QString &newAddress, bool copy);
 private:
    CmdGen() { ; }
-   static CmdGen *s_self;
 };
 
 #endif
