@@ -1,5 +1,7 @@
-/* This file is part of the KDE project
+/* -*- c-basic-offset:2 -*-
+   This file is part of the KDE project
    Copyright (C) 1998, 1999 Simon Hausmann <hausmann@kde.org>
+   Copyright (C) 2000-2004 David Faure <faure@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -13,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef __konq_mainwindow_h__
@@ -45,6 +47,7 @@ class KAction;
 class KActionCollection;
 class KActionMenu;
 class KBookmarkMenu;
+class KCMultiDialog;
 class KHistoryCombo;
 class KNewMenu;
 class KProgress;
@@ -93,16 +96,24 @@ class KonqMainWindow : public KParts::MainWindow, public KonqFrameContainerBase
   Q_PROPERTY( QString currentTitle READ currentTitle )
   Q_PROPERTY( QString currentURL READ currentURL )
   Q_PROPERTY( bool isHTMLAllowed READ isHTMLAllowed )
+  Q_PROPERTY( QString currentProfile READ currentProfile )
 public:
   enum ComboAction { ComboClear, ComboAdd, ComboRemove };
+  enum PageSecurity { NotCrypted, Encrypted, Mixed };
 
   KonqMainWindow( const KURL &initialURL = KURL(), bool openInitialURL = true, const char *name = 0, const QString& xmluiFile="konqueror.rc");
   ~KonqMainWindow();
 
+
   /**
    * Filters the URL and calls the main openURL method.
    */
-  void openFilteredURL( const QString &_url, bool inNewTab = false );
+  void openFilteredURL( const QString & _url, KonqOpenURLRequest& _req);
+
+  /**
+   * Filters the URL and calls the main openURL method.
+   */
+  void openFilteredURL( const QString &_url, bool inNewTab = false, bool tempFile = false );
 
   /**
    * The main openURL method.
@@ -188,12 +199,21 @@ public:
    * Change URL displayed in the location bar
    */
   void setLocationBarURL( const QString &url );
+  /**
+   * Overload for convenience
+   */
+  void setLocationBarURL( const KURL &url );
   /**
    * Return URL displayed in the location bar - for KonqViewManager
    */
   QString locationBarURL() const;
   void focusLocationBar();
 
+  /**
+   * Set page security related to current view
+   */
+  void setPageSecurity( PageSecurity );
+
   void enableAllActions( bool enable );
 
   void disableActionsNoView();
@@ -217,6 +237,7 @@ public:
   KToggleAction * linkViewAction()const { return m_paLinkView; }
 
   void enableAction( const char * name, bool enabled );
+  void setActionText( const char * name, const QString& text );
 
   /**
    * The default settings "allow HTML" - the one used when creating a new view
@@ -224,8 +245,6 @@ public:
    */
   bool isHTMLAllowed() const { return m_bHTMLAllowed; }
 
-  bool isBackRightClickEnabled() const { return m_bBackRightClick; }
-
   bool saveViewPropertiesLocally() const { return m_bSaveViewPropertiesLocally; }
 
   static QPtrList<KonqMainWindow> *mainWindowList() { return s_lstViews; }
@@ -297,6 +316,7 @@ public:
 
   QString currentTitle() const;
   QString currentURL() const;
+  QString currentProfile() const;
 
   QStringList configModules() const;
 
@@ -307,7 +327,9 @@ signals:
 public slots:
   void slotCtrlTabPressed();
 
+  // for KBookmarkMenu and KBookmarkBar
   void slotFillContextMenu( const KBookmark &, QPopupMenu * );
+  void slotOpenBookmarkURL( const QString & url, Qt::ButtonState state );
 
   void slotPopupMenu( const QPoint &_global, const KURL &_url, const QString &_mimeType, mode_t mode );
   void slotPopupMenu( KXMLGUIClient *client, const QPoint &_global, const KURL &_url, const QString &_mimeType, mode_t mode );
@@ -335,7 +357,6 @@ public slots:
   void slotDuplicateWindow();
   void slotSendURL();
   void slotSendFile();
-  void slotRun();
   void slotCopyFiles();
   void slotMoveFiles();
   void slotNewDir();
@@ -353,21 +374,28 @@ public slots:
 
   // Go menu
   void slotUp();
+  void slotUp(KAction::ActivationReason, Qt::ButtonState state);
   void slotUpDelayed();
   void slotBack();
+  void slotBack(KAction::ActivationReason, Qt::ButtonState state);
   void slotForward();
+  void slotForward(KAction::ActivationReason, Qt::ButtonState state);
   void slotHome();
+  void slotHome(KAction::ActivationReason, Qt::ButtonState state);
+  void slotGoSystem();
   void slotGoApplications();
-  void slotGoDevices();
+  void slotGoMedia();
+  void slotGoNetworkFolders();
   void slotGoSettings();
   void slotGoDirTree();
   void slotGoTrash();
-  void slotGoTemplates();
   void slotGoAutostart();
+  void slotGoHistory();
 
   void slotConfigure();
   void slotConfigureToolbars();
-    void slotConfigureSpellChecking();
+  void slotConfigureExtensions();
+  void slotConfigureSpellChecking();
   void slotNewToolbarConfig();
 
   void slotUndoAvailable( bool avail );
@@ -375,7 +403,7 @@ public slots:
   void slotPartChanged( KonqView *childView, KParts::ReadOnlyPart *oldPart, KParts::ReadOnlyPart *newPart );
 
   void slotRunFinished();
-  void slotClearLocationBar();
+  void slotClearLocationBar( KAction::ActivationReason reason, Qt::ButtonState state );
 
   // reimplement from KParts::MainWindow
   virtual void slotSetStatusBarText( const QString &text );
@@ -385,6 +413,7 @@ public slots:
 
   virtual void setIcon( const QPixmap& );
   void slotGoHistoryActivated( int steps );
+  void slotGoHistoryActivated( int steps, Qt::ButtonState state );
 
   void slotAddTab();
 
@@ -406,7 +435,9 @@ protected slots:
   void slotBreakOffTabPopupDelayed();
 
   void slotPopupNewWindow();
+  void slotPopupThisWindow();
   void slotPopupNewTab();
+  void slotPopupNewTabRight();
   void slotPopupPasteTo();
   void slotRemoveView();
 
@@ -485,8 +516,6 @@ protected slots:
   void slotUpdateFullScreen( bool set ); // do not call directly
 
 protected:
-  static QString detectNameFilter( QString & url );
-
   virtual bool eventFilter(QObject*obj,QEvent *ev);
 
   void fillHistoryPopup( QPopupMenu *menu, const QPtrList<HistoryEntry> &history );
@@ -519,6 +548,9 @@ private slots:
   void bookmarksIntoCompletion();
 
   void initBookmarkBar();
+  void slotTrashActivated( KAction::ActivationReason reason, Qt::ButtonState state );
+
+  void showPageSecurity();
 
 private:
   /**
@@ -560,20 +592,18 @@ private:
 
   void plugViewModeActions();
   void unplugViewModeActions();
+  static QString viewModeActionKey( KService::Ptr service );
 
   void connectActionCollection( KActionCollection *coll );
   void disconnectActionCollection( KActionCollection *coll );
 
   bool stayPreloaded();
   bool checkPreloadResourceUsage();
-  
+
   QObject* lastFrame( KonqView *view );
 
   KNewMenu * m_pMenuNew;
 
-  KAction *m_paFileType;
-  KAction *m_paProperties;
-
   KAction *m_paPrint;
 
   KActionMenu *m_pamBookmarks;
@@ -599,7 +629,6 @@ private:
   KAction *m_paRemoveOtherTabs;
   KAction *m_paActivateNextTab;
   KAction *m_paActivatePrevTab;
-  KAction *m_paDumpDebugInfo;
 
   KAction *m_paSaveRemoveViewProfile;
   KActionMenu *m_pamLoadViewProfile;
@@ -617,7 +646,6 @@ private:
 
   KAction *m_paTrash;
   KAction *m_paDelete;
-  QPtrList<KAction> m_deleteActions;
 
   KAction *m_paCopyFiles;
   KAction *m_paMoveFiles;
@@ -626,6 +654,7 @@ private:
   KAction *m_paMoveTabLeft;
   KAction *m_paMoveTabRight;
 
+  KAction *m_paConfigureExtensions;
   KAction *m_paConfigureSpellChecking;
 
   KonqLogoAction *m_paAnimatedLogo;
@@ -650,6 +679,7 @@ private:
   uint m_bViewModeToggled:1;
 
   int m_goBuffer;
+  Qt::ButtonState m_goState;
 
   MapViews m_mapViews;
 
@@ -672,6 +702,18 @@ private:
 
   QString m_title;
 
+  /**
+   * @since 3.4
+   */
+  KCMultiDialog* m_configureDialog;
+
+  /**
+   * A list of the modules to be shown in
+   * the configure dialog.
+   * @since 3.4
+   */
+  QStringList m_configureModules;
+
   QLabel* m_locationLabel;
   QGuardedPtr<KonqCombo> m_combo;
   static KConfig *s_comboConfig;
@@ -688,8 +730,6 @@ private:
 
   QString m_initialFrameName;
 
-  QString m_sViewModeForDirectory; // is actually the name of the service
-
   QPtrList<KAction> m_openWithActions;
   KActionMenu *m_viewModeMenu;
   QPtrList<KAction> m_toolBarViewModeActions; // basically holds two KonqViewActions, one of
@@ -709,8 +749,6 @@ private:
 
   bool m_urlCompletionStarted;
 
-  bool m_bBackRightClick;
-
   static bool s_preloaded;
   static KonqMainWindow* s_preloadedWindow;
   static int s_initialMemoryUsage;
