@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-    MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+    MA  02110-1301, USA.
 */
 
 #include "konq_tabs.h"
@@ -38,6 +38,7 @@
 #include "konq_view.h"
 #include "konq_viewmgr.h"
 #include "konq_misc.h"
+#include "konq_settingsxt.h"
 
 #include <kaccelmanager.h>
 #include <konq_pixmapprovider.h>
@@ -61,8 +62,8 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
 {
   KAcceleratorManager::setNoAccel(this);
 
-  QWhatsThis::add( tabBar(), i18n( "This bar contains list of currently open tabs. Click on a tab to make it "
-			  "active. The option to show a close button instead of website icon in the left "
+  QWhatsThis::add( tabBar(), i18n( "This bar contains the list of currently open tabs. Click on a tab to make it "
+			  "active. The option to show a close button instead of the website icon in the left "
 			  "corner of the tab is configurable. You can also use keyboard shortcuts to "
 			  "navigate through tabs. The text on the tab is the title of the website "
 			  "currently open in it, put your mouse over the tab too see the full title in "
@@ -119,26 +120,23 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
   connect( this, SIGNAL( contextMenu( const QPoint & ) ),
            SLOT(slotContextMenu( const QPoint & ) ) );
 
-  KConfig *config = KGlobal::config();
-  KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
+  m_MouseMiddleClickClosesTab = KonqSettings::mouseMiddleClickClosesTab();
 
-  m_MouseMiddleClickClosesTab = config->readBoolEntry( "MouseMiddleClickClosesTab", false );
-
-  m_permanentCloseButtons = config->readBoolEntry( "PermanentCloseButton", false );
+  m_permanentCloseButtons = KonqSettings::permanentCloseButton();
   if (m_permanentCloseButtons) {
     setHoverCloseButton( true );
     setHoverCloseButtonDelayed( false );
   }
   else
-    setHoverCloseButton( config->readBoolEntry( "HoverCloseButton", false ) );
-  setTabCloseActivatePrevious( config->readBoolEntry("TabCloseActivatePrevious", false) );
-  if (config->readEntry("TabPosition")=="Bottom")
+    setHoverCloseButton( KonqSettings::hoverCloseButton() );
+  setTabCloseActivatePrevious( KonqSettings::tabCloseActivatePrevious() );
+  if (KonqSettings::tabPosition()=="Bottom")
     setTabPosition(QTabWidget::Bottom);
   connect( this, SIGNAL( closeRequest( QWidget * )), SLOT(slotCloseRequest( QWidget * )));
   connect( this, SIGNAL( removeTabPopup() ),
            m_pViewManager->mainWindow(), SLOT( slotRemoveTabPopup() ) );
 
-  if ( config->readBoolEntry( "AddTabButton", true ) ) {
+  if ( KonqSettings::addTabButton() ) {
     m_leftWidget = new QToolButton( this );
     connect( m_leftWidget, SIGNAL( clicked() ),
              m_pViewManager->mainWindow(), SLOT( slotAddTab() ) );
@@ -147,7 +145,7 @@ KonqFrameTabs::KonqFrameTabs(QWidget* parent, KonqFrameContainerBase* parentCont
     QToolTip::add(m_leftWidget, i18n("Open a new tab"));
     setCornerWidget( m_leftWidget, TopLeft );
   }
-  if ( config->readBoolEntry( "CloseTabButton", true ) ) {
+  if ( KonqSettings::closeTabButton() ) {
     m_rightWidget = new QToolButton( this );
     connect( m_rightWidget, SIGNAL( clicked() ),
              m_pViewManager->mainWindow(), SLOT( slotRemoveTab() ) );
