@@ -1,5 +1,6 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 David Faure <faure@kde.org>
+/* -*- c-basic-offset: 2 -*-
+   This file is part of the KDE project
+   Copyright (C) 1998-2005 David Faure <faure@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -13,14 +14,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 
 #include "konq_view.h"
 #include "kapplication.h"
 #include "KonqViewIface.h"
+#include "konq_settingsxt.h"
 #include "konq_frame.h"
 #include "konq_run.h"
 #include "konq_events.h"
@@ -92,10 +94,11 @@ KonqView::KonqView( KonqViewFactory &viewFactory,
   m_bAborted = false;
   m_bToggleView = false;
   m_bHierarchicalView = false;
+  m_bDisableScrolling = false;
   m_bGotIconURL = false;
   m_bPopupMenuEnabled = true;
   m_browserIface = new KonqBrowserInterface( this, "browseriface" );
-  m_bBackRightClick = m_pMainWindow->isBackRightClickEnabled();
+  m_bBackRightClick = KonqSettings::backRightClick();
   m_bFollowActive = false;
   m_bBuiltinView = false;
   m_bURLDropHandling = false;
@@ -188,6 +191,9 @@ void KonqView::openURL( const KURL &url, const QString & locationBarURL,
     m_bLockHistory = false;
 
   callExtensionStringMethod( "setNameFilter(const QString&)", nameFilter );
+  if ( m_bDisableScrolling )
+    callExtensionMethod( "disableScrolling()" );
+
   setLocationBarURL( locationBarURL );
   setPageSecurity(KonqMainWindow::NotCrypted);
 
@@ -202,7 +208,6 @@ void KonqView::openURL( const KURL &url, const QString & locationBarURL,
     m_pageReferrer = args.metaData()["referrer"];
   }
 
-  m_bAborted = false;
   if ( tempFile ) {
       // Store the path to the tempfile. Yes, we could store a bool only,
       // but this would be more dangerous. If anything goes wrong in the code,
@@ -213,9 +218,9 @@ void KonqView::openURL( const KURL &url, const QString & locationBarURL,
           kdWarning(1202) << "Tempfile option is set, but URL is remote: " << url << endl;
   }
 
-  m_pPart->openURL( url );
+  aboutToOpenURL( url, args );
 
-  sendOpenURLEvent( url, args );
+  m_pPart->openURL( url );
 
   updateHistoryEntry(false /* don't save location bar URL yet */);
   // add pending history entry
@@ -426,6 +431,9 @@ void KonqView::connectPart(  )
       connect( ext, SIGNAL( enableAction( const char *, bool ) ),
                this, SLOT( slotEnableAction( const char *, bool ) ) );
 
+      connect( ext, SIGNAL( setActionText( const char *, const QString& ) ),
+               this, SLOT( slotSetActionText( const char *, const QString& ) ) );
+
       connect( ext, SIGNAL( moveTopLevelWidget( int, int ) ),
                this, SLOT( slotMoveTopLevelWidget( int, int ) ) );
 
@@ -485,6 +493,14 @@ void KonqView::slotEnableAction( const char * name, bool enabled )
     // stored inside the browser-extension.
 }
 
+void KonqView::slotSetActionText( const char* name, const QString& text )
+{
+    if ( m_pMainWindow->currentView() == this )
+        m_pMainWindow->setActionText( name, text );
+    // Otherwise, we don't have to do anything, the state of the action is
+    // stored inside the browser-extension.
+}
+
 void KonqView::slotMoveTopLevelWidget( int x, int y )
 {
   KonqFrameContainerBase* container = frame()->parentContainer();
@@ -580,13 +596,10 @@ void KonqView::slotCompleted( bool hasPending )
 
   if (!m_bGotIconURL && !m_bAborted)
   {
-    KConfig *config = KGlobal::config();
-    config->setGroup( "HTML Settings" );
-
-    if ( config->readBoolEntry( "EnableFavicon", true ) == true )
+    if ( KonqSettings::enableFavicon() == true )
     {
       // Try to get /favicon.ico
-      if ( m_serviceType == "text/html" && url().protocol().startsWith( "http" ) )
+      if ( supportsServiceType( "text/html" ) && url().protocol().startsWith( "http" ) )
           KonqPixmapProvider::downloadHostIcon( url() );
     }
   }
@@ -742,6 +755,7 @@ void KonqView::updateHistoryEntry( bool saveLocationBarURL )
     kdDebug(1202) << "Saving location bar URL : " << m_sLocationBarURL << " in history position " << m_lstHistory.at() << endl;
 #endif
     current->locationBarURL = m_sLocationBarURL;
+    current->pageSecurity = m_pageSecurity;
   }
 #ifdef DEBUG_HISTORY
   kdDebug(1202) << "Saving title : " << m_caption << " in history position " << m_lstHistory.at() << endl;
@@ -813,6 +827,7 @@ void KonqView::restoreHistory()
   kdDebug(1202) << "Restoring servicetype/name, and location bar URL from history : " << h.locationBarURL << endl;
 #endif
   setLocationBarURL( h.locationBarURL );
+  setPageSecurity( h.pageSecurity );
   m_sTypedURL = QString::null;
   if ( ! changeViewMode( h.strServiceType, h.strServiceName ) )
   {
@@ -821,10 +836,10 @@ void KonqView::restoreHistory()
     return /*false*/;
   }
 
-  m_bAborted = false;
-
   setServiceTypeInExtension();
 
+  aboutToOpenURL( h.url );
+
   if ( browserExtension() )
   {
     //kdDebug(1202) << "Restoring view from stream" << endl;
@@ -840,8 +855,6 @@ void KonqView::restoreHistory()
   else
     m_pPart->openURL( h.url );
 
-  sendOpenURLEvent( h.url );
-
   if ( m_pMainWindow->currentView() == this )
     m_pMainWindow->updateToolBarActions();
 
@@ -854,7 +867,7 @@ const HistoryEntry * KonqView::historyAt(const int pos)
 {
     if(pos<0 || pos>=(int)m_lstHistory.count())
 	return 0L;
-    int oldpos = m_lstHistory.at();     
+    int oldpos = m_lstHistory.at();
     const HistoryEntry* h = m_lstHistory.at(pos);
     m_lstHistory.at( oldpos );
     return h;
@@ -882,7 +895,7 @@ KURL KonqView::upURL() const
     if ( m_pRun )
 	currentURL = m_pRun->url();
     else
-	currentURL = m_sLocationBarURL;
+	currentURL = KURL::fromPathOrURL( m_sLocationBarURL );
     return currentURL.upURL();
 }
 
@@ -926,8 +939,10 @@ void KonqView::stop()
     // Revert to working URL - unless the URL was typed manually
     // This is duplicated with KonqMainWindow::slotRunFinished, but we can't call it
     //   since it relies on sender()...
-    if ( history().current() && m_pRun->typedURL().isEmpty() ) // not typed
+    if ( history().current() && m_pRun->typedURL().isEmpty() ) { // not typed
       setLocationBarURL( history().current()->locationBarURL );
+      setPageSecurity( history().current()->pageSecurity );
+    }
 
     setRun( 0L );
     m_pKonqFrame->statusbar()->slotLoadingProgress( -1 );
@@ -983,14 +998,13 @@ void KonqView::setLockedLocation( bool b )
   m_bLockedLocation = b;
 }
 
-void KonqView::sendOpenURLEvent( const KURL &url, const KParts::URLArgs &args )
+void KonqView::aboutToOpenURL( const KURL &url, const KParts::URLArgs &args )
 {
   KParts::OpenURLEvent ev( m_pPart, url, args );
   QApplication::sendEvent( m_pMainWindow, &ev );
 
-  // We also do here what we want to do after opening an URL, whether a new one
-  // or one from the history (common stuff).
   m_bGotIconURL = false;
+  m_bAborted = false;
 }
 
 void KonqView::setServiceTypeInExtension()
@@ -1196,7 +1210,7 @@ void KonqView::enableBackRightClick( bool b )
 void KonqView::reparseConfiguration()
 {
     callExtensionMethod( "reparseConfiguration()" );
-    bool b = m_pMainWindow->isBackRightClickEnabled();
+    bool b = KonqSettings::backRightClick();
     if ( m_bBackRightClick != b )
     {
         if (m_bBackRightClick && m_pPart->widget()->inherits("QScrollView") )
@@ -1207,6 +1221,12 @@ void KonqView::reparseConfiguration()
     }
 }
 
+void KonqView::disableScrolling()
+{
+  m_bDisableScrolling = true;
+  callExtensionMethod( "disableScrolling()" );
+}
+
 KonqViewIface * KonqView::dcopObject()
 {
   if ( !m_dcopObject )
