@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -24,6 +24,7 @@
 #include "konq_tabs.h"
 #include "konq_profiledlg.h"
 #include "konq_events.h"
+#include "konq_settingsxt.h"
 
 #include <qfileinfo.h>
 #include <qptrlist.h>
@@ -35,6 +36,8 @@
 #include <kglobalsettings.h>
 #include <ktempfile.h>
 #include <dcopclient.h>
+#include <klocale.h>
+#include <kmessagebox.h>
 
 #include <assert.h>
 #include <kpopupmenu.h>
@@ -73,10 +76,9 @@ KonqView* KonqViewManager::Initialize( const QString &serviceType, const QString
   setActivePart( childView->part() );
   m_pDocContainer = childView->frame();
 
-  KConfig *config = KGlobal::config();
-  KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
-  if ( config->readBoolEntry( "AlwaysTabbedMode", false ) )
     convertDocContainer();
+  static_cast<KonqFrameTabs*>( m_pDocContainer )->setAlwaysTabbedMode(
+      KonqSettings::alwaysTabbedMode() );
 
   m_pDocContainer->widget()->show();
   return childView;
@@ -303,60 +305,6 @@ void KonqViewManager::convertDocContainer()
   m_pDocContainer = newContainer;
 }
 
-void KonqViewManager::revertDocContainer()
-{
-  // If the tab container is left with only one tab after the removal,
-  // destroy it and put its lone child frame in its place
-
-  KonqFrameTabs* tabContainer = static_cast<KonqFrameTabs*>(m_pDocContainer);
-
-  KonqFrameContainerBase* parentContainer = tabContainer->parentContainer();
-  kdDebug(1202) << "parentContainer=" << parentContainer << endl;
-  if (parentContainer == 0L) return;
-
-  bool moveNewContainer = false;
-  QValueList<int> splitterSizes;
-  if (parentContainer->frameType()=="Container") {
-    moveNewContainer = (static_cast<KonqFrameContainer*>(parentContainer)->idAfter( tabContainer ) != 0);
-    splitterSizes = static_cast<KonqFrameContainer*>(parentContainer)->sizes();
-  }
-
-  KonqFrameBase* otherFrame = tabContainer->childFrameList()->first();
-  kdDebug(1202) << "otherFrame=" << otherFrame << endl;
-  if (otherFrame == 0L ) return;
-
-  parentContainer->widget()->setUpdatesEnabled( false );
-
-  QPoint pos = otherFrame->widget()->pos();
-  otherFrame->reparentFrame( m_pMainWindow, pos );
-
-  tabContainer->removeChildFrame( otherFrame );
-  parentContainer->removeChildFrame( tabContainer );
-
-  delete tabContainer;
-
-  otherFrame->reparentFrame( parentContainer->widget(), pos );
-  parentContainer->insertChildFrame( otherFrame );
-
-  if ( moveNewContainer ) {
-    static_cast<KonqFrameContainer*>(parentContainer)->moveToFirst( otherFrame->widget() );
-    static_cast<KonqFrameContainer*>(parentContainer)->swapChildren();
-  }
-  if (parentContainer->frameType()=="Container")
-    static_cast<KonqFrameContainer*>(parentContainer)->setSizes( splitterSizes );
-
-  otherFrame->widget()->show();
-
-  parentContainer->widget()->setUpdatesEnabled( true );
-
-  parentContainer->setActiveChild( otherFrame );
-
-  parentContainer->activateChild();
-
-  m_pDocContainer = otherFrame;
-}
-
-
 KonqView* KonqViewManager::addTab(const QString &serviceType, const QString &serviceName, bool passiveMode, bool openAfterCurrentPage  )
 {
 #ifdef DEBUG_VIEWMGR
@@ -398,7 +346,43 @@ KonqView* KonqViewManager::addTab(const QString &serviceType, const QString &ser
   return childView;
 }
 
-void KonqViewManager::duplicateTab( KonqFrameBase* tab )
+KonqView* KonqViewManager::addTabFromHistory( int steps, bool openAfterCurrentPage )
+{
+  if (m_pDocContainer == 0L)
+  {
+      if (m_pMainWindow &&
+	  m_pMainWindow->currentView() &&
+	  m_pMainWindow->currentView()->frame()) {
+	  m_pDocContainer = m_pMainWindow->currentView()->frame();
+      } else {
+	  kdDebug(1202) << "This view profile does not support tabs." << endl;
+	  return 0L;
+      }
+  }
+  if (m_pDocContainer->frameType() != "Tabs") convertDocContainer();
+
+  int oldPos = m_pMainWindow->currentView()->historyPos();
+  int newPos = oldPos + steps;
+
+  const HistoryEntry * he = m_pMainWindow->currentView()->historyAt(newPos);  
+  if(!he)
+      return 0L;
+
+  KonqView* newView = 0L;
+  newView  = addTab( he->strServiceType, he->strServiceName, false, openAfterCurrentPage );
+    
+  if(!newView)
+      return 0L;
+
+  newView->copyHistory(m_pMainWindow->currentView());
+  newView->setHistoryPos(newPos);
+  newView->restoreHistory();
+
+  return newView;
+}
+
+
+void KonqViewManager::duplicateTab( KonqFrameBase* tab, bool openAfterCurrentPage )
 {
 #ifdef DEBUG_VIEWMGR
   kdDebug(1202) << "---------------- KonqViewManager::duplicateTab( " << tab << " ) --------------" << endl;
@@ -450,7 +434,7 @@ void KonqViewManager::duplicateTab( KonqFrameBase* tab )
   // from profile loading (e.g. in switchView)
   m_bLoadingProfile = true;
 
-  loadItem( config, tabContainer, rootItem, KURL(""), true );
+  loadItem( config, tabContainer, rootItem, KURL(""), true, openAfterCurrentPage );
 
   m_bLoadingProfile = false;
 
@@ -460,6 +444,9 @@ void KonqViewManager::duplicateTab( KonqFrameBase* tab )
   // so we do it once at the end :
   m_pMainWindow->viewCountChanged();
 
+  if (openAfterCurrentPage)
+    tabContainer->setCurrentPage( tabContainer->currentPageIndex () + 1 );
+  else
   tabContainer->setCurrentPage( tabContainer->count() - 1 );
 
   KonqFrameBase* duplicatedFrame = dynamic_cast<KonqFrameBase*>(tabContainer->currentPage());
@@ -507,6 +494,7 @@ void KonqViewManager::breakOffTab( KonqFrameBase* tab )
   QString prefix = QString::fromLatin1( currentFrame->frameType() ) + QString::number(0);
   config.writeEntry( "RootItem", prefix );
   prefix.append( '_' );
+  config.writeEntry( QString::fromLatin1( "docContainer" ).prepend( prefix ), true );
   currentFrame->saveConfig( &config, prefix, true, 0L, 0, 1);
 
   KonqMainWindow *mainWindow = new KonqMainWindow( KURL(), false );
@@ -514,10 +502,14 @@ void KonqViewManager::breakOffTab( KonqFrameBase* tab )
 
   mainWindow->viewManager()->loadViewProfile( config, "" );
 
-  mainWindow->viewManager()->setDocContainer( mainWindow->childFrame() );
-
-  if (mainWindow->currentView())
-    mainWindow->copyHistory( currentFrame );
+  KonqFrameBase * newDocContainer = mainWindow->viewManager()->docContainer();
+  if( newDocContainer && newDocContainer->frameType() == "Tabs")
+  {
+    KonqFrameTabs *kft = static_cast<KonqFrameTabs *>(newDocContainer);
+    KonqFrameBase *newFrame = dynamic_cast<KonqFrameBase*>(kft->currentPage());
+    if(newFrame)
+      newFrame->copyHistory( currentFrame );
+  }
 
   removeTab( currentFrame );
 
@@ -590,13 +582,6 @@ void KonqViewManager::removeTab( KonqFrameBase* tab )
 
   tabContainer->slotCurrentChanged(tabContainer->currentPage());
 
-  if (tabContainer->count() == 1) {
-    KConfig *config = KGlobal::config();
-    KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
-    if ( !( config->readBoolEntry( "AlwaysTabbedMode", false ) ) )
-      revertDocContainer();
-  }
-
 #ifdef DEBUG_VIEWMGR
   m_pMainWindow->dumpViewList();
   printFullHierarchy( m_pMainWindow );
@@ -648,7 +633,7 @@ void KonqViewManager::removeOtherTabs( KonqFrameBase* tab )
 
   for ( it.toFirst(); it != 0L; ++it )
   {
-    if( it.current() != tab)
+    if( it.current() != currentFrame )
       removeTab(it.current());
   }
 
@@ -713,10 +698,25 @@ void KonqViewManager::activatePrevTab()
   tabContainer->setCurrentPage( iTab );
 }
 
+void KonqViewManager::activateTab(int position)
+{
+  if (m_pDocContainer == 0L) return;
+  if (m_pDocContainer->frameType() != "Tabs") return;
+
+  KonqFrameTabs* tabContainer = static_cast<KonqFrameTabs*>(m_pDocContainer);
+  if (position<0 || tabContainer->count() == 1 || position>=tabContainer->count()) return;
+
+  tabContainer->setCurrentPage( position );
+}
+
 void KonqViewManager::showTab( KonqView *view )
 {
-  static_cast<KonqFrameTabs*>( docContainer() )->showPage( view->frame() );
-  emitActivePartChanged();
+  KonqFrameTabs *tabContainer = static_cast<KonqFrameTabs*>( docContainer() );
+  if (tabContainer->currentPage() != view->frame())
+  {
+    tabContainer->showPage( view->frame() );
+    emitActivePartChanged();
+  }
 }
 
 void KonqViewManager::updatePixmaps()
@@ -776,8 +776,8 @@ void KonqViewManager::removeView( KonqView *view )
 
     if( otherFrame == 0L )
     {
-    kdWarning(1202) << "KonqViewManager::removeView: This shouldn't happen!" << endl;
-    return;
+        kdWarning(1202) << "KonqViewManager::removeView: This shouldn't happen!" << endl;
+        return;
     }
 
     if (m_pDocContainer == parentContainer) m_pDocContainer = otherFrame;
@@ -795,10 +795,10 @@ void KonqViewManager::removeView( KonqView *view )
     //kdDebug(1202) << "--- Removing parentContainer from grandParentContainer" << endl;
     grandParentContainer->removeChildFrame( parentContainer );
 
-    //kdDebug(1202) << "--- Removing view from view list, and deleting view " << view << endl;
+    //kdDebug(1202) << "--- Removing view from view list" << endl;
     m_pMainWindow->removeChildView(view);
-    view->partDeleted();
-    delete view;
+    //kdDebug(1202) << "--- Deleting view " << view << endl;
+    delete view; // This deletes the view, which deletes the part, which deletes its widget
 
     //kdDebug(1202) << "--- Deleting parentContainer " << parentContainer
     //              << ". Its parent is " << parentContainer->widget()->parent() << endl;
@@ -954,7 +954,7 @@ KonqView *KonqViewManager::chooseNextView( KonqView *view )
        kdWarning() << "View " << view << " is not in list !" << endl;
      it = mapViews.begin();
      if ( it == end )
-       return 0L; // We have no view at all - this happens with RootItem=empty
+       return 0L; // We have no view at all - this used to happen with totally-empty-profiles
    }
 
   KonqMainWindow::MapViews::Iterator startIt = it;
@@ -984,7 +984,7 @@ KonqViewFactory KonqViewManager::createView( const QString &serviceType,
                                           KService::Ptr &service,
                                           KTrader::OfferList &partServiceOffers,
                                           KTrader::OfferList &appServiceOffers,
-					  bool forceAutoEmbed )
+                                          bool forceAutoEmbed )
 {
   kdDebug(1202) << "KonqViewManager::createView" << endl;
   KonqViewFactory viewFactory;
@@ -1122,24 +1122,73 @@ void KonqViewManager::saveViewProfile( KConfig & cfg, bool saveURLs, bool saveWi
 
 void KonqViewManager::loadViewProfile( const QString & path, const QString & filename,
                                        const KURL & forcedURL, const KonqOpenURLRequest &req,
-                                       bool resetWindow )
+                                       bool resetWindow, bool openURL )
 {
   KConfig cfg( path, true );
   cfg.setDollarExpansion( true );
   cfg.setGroup( "Profile" );
-  loadViewProfile( cfg, filename, forcedURL, req, resetWindow );
+  loadViewProfile( cfg, filename, forcedURL, req, resetWindow, openURL );
 }
 
 void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
                                        const KURL & forcedURL, const KonqOpenURLRequest &req,
-                                       bool resetWindow )
+                                       bool resetWindow, bool openURL )
 {
-  KConfig *config = KGlobal::config();
-  KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
-  bool alwaysTabbedMode = config->readBoolEntry( "AlwaysTabbedMode", false );
+  if ( docContainer() && docContainer()->frameType()=="Tabs" )
+  {
+      KonqFrameTabs* tabContainer = static_cast<KonqFrameTabs*>(docContainer());
+      if ( tabContainer->count() > 1 )
+      {
+          if ( KMessageBox::warningContinueCancel( 0,
+                  i18n("You have multiple tabs open in this window.\n"
+                        "Loading a view profile will close them."),
+                  i18n("Confirmation"),
+                  i18n("Load View Profile"),
+                  "LoadProfileTabsConfirm" ) == KMessageBox::Cancel )
+              return;
+      }
+
+      KonqView *originalView = m_pMainWindow->currentView();
+      QPtrList<KonqFrameBase> frameList = *tabContainer->childFrameList();
+      QPtrListIterator<KonqFrameBase> it( frameList );
+      for ( it.toFirst(); it != 0L; ++it )
+      {
+          KonqView *view = it.current()->activeChildView();
+          if (view && view->part() && (view->part()->metaObject()->findProperty("modified") != -1)) {
+            QVariant prop = view->part()->property("modified");
+            if (prop.isValid() && prop.toBool()) {
+                showTab( view );
+                if ( KMessageBox::warningContinueCancel( 0,
+                   i18n("This tab contains changes that have not been submitted.\nLoading a profile will discard these changes."),
+                   i18n("Discard Changes?"), i18n("&Discard Changes"), "discardchangesloadprofile") != KMessageBox::Continue )
+                {
+                    showTab( originalView );
+                    return;
+                }
+            }
+         }
+      }
+      showTab( originalView );
+  }
+  else
+  {
+      KonqView *view = m_pMainWindow->currentView();
+      if (view && view->part() && (view->part()->metaObject()->findProperty("modified") != -1)) {
+        QVariant prop = view->part()->property("modified");
+        if (prop.isValid() && prop.toBool())
+            if ( KMessageBox::warningContinueCancel( 0,
+               i18n("This page contains changes that have not been submitted.\nLoading a profile will discard these changes."),
+	       i18n("Discard Changes?"), i18n("&Discard Changes"), "discardchangesloadprofile") != KMessageBox::Continue )
+            return;
+      }
+  }
+
+  bool alwaysTabbedMode = KonqSettings::alwaysTabbedMode();
 
   m_currentProfile = filename;
   m_currentProfileText = cfg.readPathEntry("Name",filename);
+  m_profileHomeURL = cfg.readEntry("HomeURL", QString::null);
+
   m_pMainWindow->currentProfileChanged();
   KURL defaultURL;
   if ( m_pMainWindow->currentView() )
@@ -1149,28 +1198,15 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
 
   QString rootItem = cfg.readEntry( "RootItem", "empty" );
 
-  //kdDebug(1202) << rootItem << endl;
-
-  if( rootItem.isNull() ) {
-    // Config file doesn't contain anything about view profiles, fallback to defaults
-    rootItem = "InitialView";
-  }
   //kdDebug(1202) << "KonqViewManager::loadViewProfile : loading RootItem " << rootItem << endl;
 
-  if ( alwaysTabbedMode && rootItem == "empty" )
-  {
-    cfg.writeEntry( "View0_ServiceType", "text/html" );
-    cfg.writeEntry( "View0_ServiceName", "html" );
-    rootItem = "View0";
-  }
-
-  if ( rootItem != "empty" && forcedURL.url() != "about:blank" )
+  if ( forcedURL.url() != "about:blank" )
   {
     // This flag is used by KonqView, to distinguish manual view creation
     // from profile loading (e.g. in switchView)
     m_bLoadingProfile = true;
 
-    loadItem( cfg, m_pMainWindow, rootItem, defaultURL, forcedURL.isEmpty() );
+    loadItem( cfg, m_pMainWindow, rootItem, defaultURL, openURL && forcedURL.isEmpty() );
 
     m_bLoadingProfile = false;
 
@@ -1186,9 +1222,24 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
     m_pMainWindow->action( "clear_location" )->activate();
   }
 
-  if ( alwaysTabbedMode && m_pDocContainer->frameType() != "Tabs")
+  if (m_pDocContainer == 0L)
+  {
+    if (m_pMainWindow &&
+        m_pMainWindow->currentView() &&
+        m_pMainWindow->currentView()->frame()) {
+       m_pDocContainer = m_pMainWindow->currentView()->frame();
+    } else {
+       kdDebug(1202) << "This view profile does not support tabs." << endl;
+       return;
+    }
+  }
+
+
+    if ( m_pDocContainer->frameType() != "Tabs")
     convertDocContainer();
 
+  static_cast<KonqFrameTabs*>( m_pDocContainer )->setAlwaysTabbedMode( alwaysTabbedMode );
+
   // Set an active part first so that we open the URL in the current view
   // (to set the location bar correctly and asap)
   KonqView *nextChildView = 0L;
@@ -1196,12 +1247,15 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
   if (nextChildView == 0L) nextChildView = chooseNextView( 0L );
   setActivePart( nextChildView ? nextChildView->part() : 0L, true /* immediate */);
 
-  if ( !forcedURL.isEmpty())
+  // #71164
+  if ( !req.args.frameName.isEmpty() && nextChildView ) {
+      nextChildView->setViewName( req.args.frameName );
+  }
+
+  if ( openURL && !forcedURL.isEmpty())
   {
       KonqOpenURLRequest _req(req);
-      KConfig *config = KGlobal::config();
-      KConfigGroupSaver cs( config, QString::fromLatin1("FMSettings") );
-      _req.openAfterCurrentPage = config->readBoolEntry( "OpenAfterCurrentPage", false );
+      _req.openAfterCurrentPage = KonqSettings::openAfterCurrentPage();
       _req.forceAutoEmbed = true; // it's a new window, let's use it
 
       m_pMainWindow->openURL( nextChildView /* can be 0 for an empty profile */,
@@ -1239,8 +1293,8 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
   }
 
   if( resetWindow )
-  {
-     m_pMainWindow->applyMainWindowSettings( KGlobal::config(), "KonqMainWindow" );
+  { // force default settings for the GUI
+     m_pMainWindow->applyMainWindowSettings( KGlobal::config(), "KonqMainWindow", true );
   }
 
   // Apply menu/toolbar settings saved in profile. Read from a separate group
@@ -1301,14 +1355,16 @@ void KonqViewManager::setActivePart( KParts::Part *part, bool immediate )
     if (part && part->widget())
         part->widget()->setFocus();
 
-    if (!immediate)
-        // We use a 0s single shot timer so that when clicking on a part,
+    if (!immediate && reason() != ReasonRightClick)
+        // We use a 0s single shot timer so that when left-clicking on a part,
         // we process the mouse event before rebuilding the GUI.
         // Otherwise, when e.g. dragging icons, the mouse pointer can already
         // be very far from where it was...
         // TODO: use a QTimer member var, so that if two conflicting calls to
         // setActivePart(part,immediate=false) happen, the 1st one gets cancelled.
         QTimer::singleShot( 0, this, SLOT( emitActivePartChanged() ) );
+        // This is not done with right-clicking so that the part is activated before the
+        // popup appears (#75201)
     else
         emitActivePartChanged();
 }
@@ -1385,7 +1441,7 @@ QSize KonqViewManager::readConfigSize( KConfig &cfg, QWidget *widget )
 }
 
 void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
-                                const QString &name, const KURL & defaultURL, bool openURL )
+                                const QString &name, const KURL & defaultURL, bool openURL, bool openAfterCurrentPage )
 {
   QString prefix;
   if( name != "InitialView" )
@@ -1393,13 +1449,20 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
 
   //kdDebug(1202) << "begin loadItem: " << name << endl;
 
-  if( name.startsWith("View") ) {
-    //kdDebug(1202) << "Item is View" << endl;
+  if( name.startsWith("View") || name == "empty" ) {
     //load view config
-    QString serviceType = cfg.readEntry( QString::fromLatin1( "ServiceType" ).prepend( prefix ), "inode/directory");
-    //kdDebug(1202) << "ServiceType: " << serviceType << endl;
-
-    QString serviceName = cfg.readEntry( QString::fromLatin1( "ServiceName" ).prepend( prefix ) );
+    QString serviceType;
+    QString serviceName;
+    if ( name == "empty" ) {
+        // An empty profile is an empty KHTML part. Makes all KHTML actions available, avoids crashes,
+        // makes it easy to DND a URL onto it, and makes it fast to load a website from there.
+        serviceType = "text/html";
+        serviceName = "html";
+    } else {
+        serviceType = cfg.readEntry( QString::fromLatin1( "ServiceType" ).prepend( prefix ), "inode/directory");
+        serviceName = cfg.readEntry( QString::fromLatin1( "ServiceName" ).prepend( prefix ) );
+    }
+    //kdDebug(1202) << "ServiceType: " << serviceType << " " << serviceName << endl;
 
     KService::Ptr service;
     KTrader::OfferList partServiceOffers, appServiceOffers;
@@ -1414,7 +1477,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
     bool passiveMode = cfg.readBoolEntry( QString::fromLatin1( "PassiveMode" ).prepend( prefix ), false );
 
     //kdDebug(1202) << "Creating View Stuff" << endl;
-    KonqView *childView = setupView( parent, viewFactory, service, partServiceOffers, appServiceOffers, serviceType, passiveMode );
+    KonqView *childView = setupView( parent, viewFactory, service, partServiceOffers, appServiceOffers, serviceType, passiveMode, openAfterCurrentPage );
 
     if (!childView->isFollowActive()) childView->setLinkedView( cfg.readBoolEntry( QString::fromLatin1( "LinkedView" ).prepend( prefix ), false ) );
     childView->setToggleView( cfg.readBoolEntry( QString::fromLatin1( "ToggleView" ).prepend( prefix ), false ) );
@@ -1528,7 +1591,12 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
     {
       KonqFrameContainer *newContainer = new KonqFrameContainer( o, parent->widget(), parent );
       connect(newContainer,SIGNAL(ctrlTabPressed()),m_pMainWindow,SLOT(slotCtrlTabPressed()));
-      parent->insertChildFrame( newContainer );
+      
+      int tabindex = -1;
+      if(openAfterCurrentPage && parent->frameType() == "Tabs") // Need to honor it, if possible      
+	tabindex = static_cast<KonqFrameTabs*>(parent)->currentPageIndex() + 1;
+      parent->insertChildFrame( newContainer, tabindex );
+
 
       if (cfg.readBoolEntry( QString::fromLatin1( "docContainer" ).prepend( prefix ), false ))
         m_pDocContainer = newContainer;
@@ -1642,7 +1710,6 @@ void KonqViewManager::slotProfileActivated( int id )
       break;
     }
   }
-
 }
 
 void KonqViewManager::slotProfileListAboutToShow()
@@ -1678,7 +1745,9 @@ void KonqViewManager::setLoading( KonqView *view, bool loading )
     QColor color;
     KonqFrameTabs* konqframetabs = static_cast<KonqFrameTabs*>( parentContainer );
     if ( loading )
-      color = KGlobalSettings::inactiveTextColor();
+      color = QColor( (KGlobalSettings::linkColor().red()  + KGlobalSettings::inactiveTextColor().red())/2,
+                      (KGlobalSettings::linkColor().green()+ KGlobalSettings::inactiveTextColor().green())/2,
+                      (KGlobalSettings::linkColor().blue() + KGlobalSettings::inactiveTextColor().blue())/2 );
     else
     {
       if ( konqframetabs->currentPage() != view->frame() )
