@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "konq_sidebartreemodule.h"
@@ -189,6 +189,8 @@ KonqSidebarTree::KonqSidebarTree( KonqSidebar_Tree *parent, QWidget *parentWidge
       firstChild()->setOpen(true);
       m_bOpeningFirstChild = false;
     }
+
+    setFrameStyle( QFrame::ToolBarPanel | QFrame::Raised );
 }
 
 KonqSidebarTree::~KonqSidebarTree()
@@ -482,11 +484,6 @@ void KonqSidebarTree::slotMouseButtonPressed( int _button, QListViewItem* _item,
     KonqSidebarTreeItem * item = static_cast<KonqSidebarTreeItem*>( _item );
     if (_button == RightButton)
     {
-        if (!item)
-        {
-            showToplevelContextMenu();
-            return;
-        }
         if ( item && col < 2)
         {
             item->setSelected( true );
@@ -657,7 +654,7 @@ void KonqSidebarTree::scanDir( KonqSidebarTreeItem *parent, const QString &path,
         	            if ( *eIt != "." && *eIt != ".."
                 	         && !entries.contains( *eIt ) && !dirEntries.contains( *eIt ) )
 	                    { // we don't have that one yet -> copy it.
-                	        QString cp("cp -R ");
+                	        QString cp("cp -R -- ");
         	                cp += KProcess::quote(dirtree_dir + *eIt);
 	                        cp += " ";
         	                cp += KProcess::quote(path);
@@ -881,27 +878,30 @@ void KonqSidebarTree::enableActions( bool copy, bool cut, bool paste,
     enableAction( "rename", rename );
 }
 
-void KonqSidebarTree::showToplevelContextMenu()
+bool KonqSidebarTree::tabSupport()
 {
-    KonqSidebarTreeTopLevelItem *item = 0;
-    KonqSidebarTreeItem *treeItem = currentItem();
-    if (treeItem && treeItem->isTopLevelItem())
-        item = static_cast<KonqSidebarTreeTopLevelItem *>(treeItem);
-
     // see if the newTab() dcop function is available (i.e. the sidebar is embedded into konqueror)
-    bool tabSupport = false;
-    DCOPRef ref(kapp->dcopClient()->appId(), topLevelWidget()->name());
+   DCOPRef ref(kapp->dcopClient()->appId(), topLevelWidget()->name());
     DCOPReply reply = ref.call("functions()");
     if (reply.isValid()) {
         QCStringList funcs;
         reply.get(funcs, "QCStringList");
         for (QCStringList::ConstIterator it = funcs.begin(); it != funcs.end(); ++it) {
             if ((*it) == "void newTab(QString url)") {
-                tabSupport = true;
+                return true;
                 break;
             }
         }
     }
+    return false;
+}
+
+void KonqSidebarTree::showToplevelContextMenu()
+{
+    KonqSidebarTreeTopLevelItem *item = 0;
+    KonqSidebarTreeItem *treeItem = currentItem();
+    if (treeItem && treeItem->isTopLevelItem())
+        item = static_cast<KonqSidebarTreeTopLevelItem *>(treeItem);
 
     if (!m_collection)
     {
@@ -933,7 +933,7 @@ void KonqSidebarTree::showToplevelContextMenu()
             menu->insertSeparator();
             m_collection->action("create_folder")->plug(menu);
         } else {
-            if (tabSupport)
+            if (tabSupport())
                 m_collection->action("open_tab")->plug(menu);
             m_collection->action("open_window")->plug(menu);
             m_collection->action("copy_location")->plug(menu);
@@ -1022,8 +1022,8 @@ void KonqSidebarTree::slotOpenNewWindow()
 void KonqSidebarTree::slotOpenTab()
 {
     if (!m_currentTopLevelItem) return;
-    DCOPRef ref(kapp->dcopClient()->appId(), topLevelWidget()->name());
-    ref.call( "newTab(QString)", m_currentTopLevelItem->externalURL() );
+    DCOPRef ref(kapp->dcopClient()->appId(), topLevelWidget()->name());   
+    ref.call( "newTab(QString)", m_currentTopLevelItem->externalURL().url() );
 }
 
 void KonqSidebarTree::slotCopyLocation()
