@@ -1,6 +1,6 @@
 /* ------------------------------------------------------------------------ */
 /*                                                                          */
-/* [TEWidget.cpp]        Terminal Emulation Widget                          */
+/*                       Terminal Emulation Widget                          */
 /*                                                                          */
 /* ------------------------------------------------------------------------ */
 /*                                                                          */
@@ -73,16 +73,10 @@
 #include <kglobalsettings.h>
 #include <kshortcut.h>
 #include <kurldrag.h>
+#include <kio/netaccess.h>
 #include <qlabel.h>
 #include <qtimer.h>
 
-#ifndef HERE
-#define HERE printf("%s(%d): %s\n",__FILE__,__LINE__,__FUNCTION__)
-#endif
-#ifndef HCNT
-#define HCNT(Name) // { static int cnt = 1; printf("%s(%d): %s %d\n",__FILE__,__LINE__,Name,cnt++); }
-#endif
-
 #ifndef loc
 #define loc(X,Y) ((Y)*columns+(X))
 #endif
@@ -196,6 +190,8 @@ void TEWidget::setColorTable(const ColorEntry table[])
    QCodec.
 */
 
+static inline bool isLineChar(Q_UINT16 c) { return ((c & 0xFF80) == 0x2500);}
+
 // assert for i in [0..31] : vt100extended(vt100_graphics[i]) == i.
 
 unsigned short vt100_graphics[32] =
@@ -269,10 +265,9 @@ void TEWidget::fontChange(const QFont &)
     if (fw != fm.width(REPCHAR[i])){
       fixed_font = false;
       break;
-    } else
-      fw = fm.width(REPCHAR[i]);
   }
-
+  }
+  
   if (font_w>200) // don't trust unrealistic value, fallback to QFontMetrics::maxWidth()
     font_w=fm.maxWidth();
   if (font_w<1)
@@ -288,9 +283,11 @@ void TEWidget::fontChange(const QFont &)
 //printf("font_a: %d\n",font_a);
 //printf("rawname: %s\n",font().rawName().ascii());
 
+/*
 #if defined(Q_CC_GNU)
 #warning TODO: Review/fix vt100 extended font-mapping
 #endif
+*/
 
 //  fontMap = identicalMap;
   emit changedFontMetricSignal( font_h, font_w );
@@ -403,8 +400,8 @@ TEWidget::TEWidget(QWidget *parent, const char *name)
   setFocusPolicy( WheelFocus );
 
   // im
-  setInputMethodEnabled(true);
-
+  setInputMethodEnabled(true);  
+  
   if (!argb_visual)
   {
     // Looks better at startup with KRootPixmap based pseudo-transparancy
@@ -426,6 +423,122 @@ TEWidget::~TEWidget()
 /*                                                                           */
 /* ------------------------------------------------------------------------- */
 
+/**
+ A table for emulating the simple (single width) unicode drawing chars.
+ It represents the 250x - 257x glyphs. If it's zero, we can't use it.
+ if it's not, it's encoded as follows: imagine a 5x5 grid where the points are numbered
+ 0 to 24 left to top, top to bottom. Each point is represented by the corresponding bit.
+  
+ Then, the pixels basically have the following interpretation:
+ _|||_
+ -...-
+ -...-
+ -...-
+ _|||_
+
+where _ = none
+      | = vertical line.
+      - = horizontal line.
+ */
+ 
+
+enum LineEncode
+{
+    TopL  = (1<<1),
+    TopC  = (1<<2),
+    TopR  = (1<<3),
+    
+    LeftT = (1<<5),
+    Int11 = (1<<6),
+    Int12 = (1<<7),
+    Int13 = (1<<8),
+    RightT = (1<<9),
+    
+    LeftC = (1<<10),
+    Int21 = (1<<11),
+    Int22 = (1<<12),
+    Int23 = (1<<13),
+    RightC = (1<<14),
+    
+    LeftB = (1<<15),
+    Int31 = (1<<16),
+    Int32 = (1<<17),
+    Int33 = (1<<18),
+    RightB = (1<<19),
+    
+    BotL  = (1<<21),
+    BotC  = (1<<22),
+    BotR  = (1<<23)
+};
+
+#include "linefont.h"
+
+static void drawLineChar(QPainter& paint, int x, int y, int w, int h, uchar code)
+{
+    //Calculate cell midpoints, end points.
+    int cx = x + w/2;
+    int cy = y + h/2;
+    int ex = x + w - 1;
+    int ey = y + h - 1;
+
+    Q_UINT32 toDraw = LineChars[code];
+    
+    //Top lines:
+    if (toDraw & TopL)
+        paint.drawLine(cx-1, y, cx-1, cy-2);
+    if (toDraw & TopC)
+        paint.drawLine(cx, y, cx, cy-2);
+    if (toDraw & TopR)
+        paint.drawLine(cx+1, y, cx+1, cy-2);
+
+    //Bot lines:
+    if (toDraw & BotL)
+        paint.drawLine(cx-1, cy+2, cx-1, ey);
+    if (toDraw & BotC)
+        paint.drawLine(cx, cy+2, cx, ey);
+    if (toDraw & BotR)
+        paint.drawLine(cx+1, cy+2, cx+1, ey);
+
+    //Left lines:
+    if (toDraw & LeftT)
+        paint.drawLine(x, cy-1, cx-2, cy-1);
+    if (toDraw & LeftC)
+        paint.drawLine(x, cy, cx-2, cy);
+    if (toDraw & LeftB)
+        paint.drawLine(x, cy+1, cx-2, cy+1);
+
+    //Right lines:
+    if (toDraw & RightT)
+        paint.drawLine(cx+2, cy-1, ex, cy-1);
+    if (toDraw & RightC)
+        paint.drawLine(cx+2, cy, ex, cy);
+    if (toDraw & RightB)
+        paint.drawLine(cx+2, cy+1, ex, cy+1);
+    
+    //Intersection points.
+    if (toDraw & Int11)
+        paint.drawPoint(cx-1, cy-1);
+    if (toDraw & Int12)
+        paint.drawPoint(cx, cy-1);
+    if (toDraw & Int13)
+        paint.drawPoint(cx+1, cy-1);
+    
+    if (toDraw & Int21)
+        paint.drawPoint(cx-1, cy);
+    if (toDraw & Int22)
+        paint.drawPoint(cx, cy);
+    if (toDraw & Int23)
+        paint.drawPoint(cx+1, cy);
+
+    if (toDraw & Int31)
+        paint.drawPoint(cx-1, cy+1);
+    if (toDraw & Int32)
+        paint.drawPoint(cx, cy+1);
+    if (toDraw & Int33)
+        paint.drawPoint(cx+1, cy+1);
+
+}
+
 void TEWidget::drawTextFixed(QPainter &paint, int x, int y,
                              QString& str, const ca *attr)
 {
@@ -446,6 +559,19 @@ void TEWidget::drawTextFixed(QPainter &paint, int x, int y,
       w = font_w*2;
       nc+=2;
     }
+    
+    //Check for line-drawing char
+    if (isLineChar(drawstr[0].unicode()))
+    {
+        uchar code = drawstr[0].cell();
+        if (LineChars[code])
+        {
+            drawLineChar(paint, x, y, w, font_h, code);
+            x += w;
+            continue;
+        }
+    }
+    
     paint.drawText(x,y, w, font_h, Qt::AlignHCenter | Qt::DontClip, drawstr, -1);
     x += w;
   }
@@ -489,7 +615,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
           Q_UINT8 dalpha = 255 - salpha;
 
           int a, r, g, b;
-          a = QMIN( (qAlpha (col) * salpha) / 255 + (qAlpha (blend_color) * dalpha) / 255, 255 );
+          a = QMIN( (qAlpha (col) * salpha) / 255 + (qAlpha (blend_color) * dalpha) / 255, 255 ); 
           r = QMIN( (qRed   (col) * salpha) / 255 + (qRed   (blend_color) * dalpha) / 255, 255 );
           g = QMIN( (qGreen (col) * salpha) / 255 + (qGreen (blend_color) * dalpha) / 255, 255 );
           b = QMIN( (qBlue  (col) * salpha) / 255 + (qBlue  (blend_color) * dalpha) / 255, 255 );
@@ -519,7 +645,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
       int w = font_w * (m_imSelEnd - m_imSelStart);
       int h = font_h;
 
-      QRect tmpRect = QRect( x, y, w, h );
+      QRect tmpRect = QRect( x, y, w, h ); 
       if ( str != m_imPreeditText ) {  // ugly hack
         tmpRect.setLeft( tmpRect.left() + font_w );
         tmpRect.setWidth( tmpRect.width() + font_w );
@@ -585,7 +711,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
     {
       // The meaning of y differs between different versions of QPainter::drawText!!
       int y = rect.y()+a; // baseline
-
+ 
       if ( shadow ) {
         paint.setPen( Qt::black );
         paint.drawText(x+1,y+1, str, -1, bidiEnabled ? QPainter::Auto : QPainter::LTR );
@@ -664,7 +790,6 @@ void TEWidget::setImage(const ca* const newimg, int lines, int columns)
   QPainter paint;
   setUpdatesEnabled(false);
   paint.begin( this );
-HCNT("setImage");
 
   QPoint tL  = contentsRect().topLeft();
   int    tLx = tL.x();
@@ -717,6 +842,8 @@ HCNT("setImage");
             continue;
         int p = 0;
         disstrU[p++] = c; //fontMap(c);
+        bool lineDraw = isLineChar(c);
+        bool doubleWidth = (ext[x+1].c == 0);
         cr = ext[x].r;
         cb = ext[x].b;
         if (ext[x].f != cf) cf = ext[x].f;
@@ -725,13 +852,10 @@ HCNT("setImage");
         {
           c = ext[x+len].c;
           if (!c)
-          {
-            fixed_font = false;
             continue; // Skip trailing part of multi-col chars.
-          }
 
           if (ext[x+len].f != cf || ext[x+len].b != cb || ext[x+len].r != cr ||
-              !dirtyMask[x+len] )
+              !dirtyMask[x+len] || isLineChar(c) != lineDraw || (ext[x+len+1].c == 0) != doubleWidth)
             break;
 
           disstrU[p++] = c; //fontMap(c);
@@ -755,10 +879,16 @@ HCNT("setImage");
           if ( ( m_imSelStart < m_imSelEnd ) )
             m_isIMSel = true;
 	}
-
+        
+        bool save_fixed_font = fixed_font;
+        if (lineDraw)
+           fixed_font = false;
+        if (doubleWidth)
+           fixed_font = false;
         drawAttrStr(paint,
                     QRect(bX+tLx+font_w*x,bY+tLy+font_h*y,font_w*len,font_h),
                     unistr, &ext[x], pm != NULL, true);
+        fixed_font = save_fixed_font;
         x += len - 1;
       }
     }
@@ -840,7 +970,6 @@ void TEWidget::paintEvent( QPaintEvent* pe )
   setUpdatesEnabled(false);
   paint.begin( this );
   paint.setBackgroundMode( TransparentMode );
-HCNT("paintEvent");
 
   // Note that the actual widget size can be slightly larger
   // that the image (the size is truncated towards the smaller
@@ -918,33 +1047,39 @@ void TEWidget::paintContents(QPainter &paint, const QRect &rect, bool pm)
       c = image[loc(x,y)].c;
       if (c)
          disstrU[p++] = c; //fontMap(c);
+      bool lineDraw = isLineChar(c);
+      bool doubleWidth = (image[loc(x,y)+1].c == 0);
       int cf = image[loc(x,y)].f;
       int cb = image[loc(x,y)].b;
       int cr = image[loc(x,y)].r;
       while (x+len <= rlx &&
              image[loc(x+len,y)].f == cf &&
              image[loc(x+len,y)].b == cb &&
-             image[loc(x+len,y)].r == cr )
+             image[loc(x+len,y)].r == cr &&
+             (image[loc(x+len,y)+1].c == 0) == doubleWidth &&
+             isLineChar( c = image[loc(x+len,y)].c) == lineDraw) // Assignment!
       {
-        c = image[loc(x+len,y)].c;
         if (c)
           disstrU[p++] = c; //fontMap(c);
-        else
-          fixed_font = false;
+        if (doubleWidth) // assert((image[loc(x+len,y)+1].c == 0)), see above if condition
+          len++; // Skip trailing part of multi-column char
         len++;
       }
       if ((x+len < columns) && (!image[loc(x+len,y)].c))
-      {
-        fixed_font = false;
         len++; // Adjust for trailing part of multi-column char
-      }
 
       if (!isBlinkEvent || (cr & RE_BLINK))
       {
+         bool save_fixed_font = fixed_font;
+         if (lineDraw)
+            fixed_font = false;
+         if (doubleWidth)
+            fixed_font = false;
          QString unistr(disstrU,p);
          drawAttrStr(paint,
                 QRect(bX+tLx+font_w*x,bY+tLy+font_h*y,font_w*len,font_h),
                 unistr, &image[loc(x,y)], pm, !(isBlinkEvent || isPrinting));
+         fixed_font = save_fixed_font;
       }
       x += len - 1;
     }
@@ -1459,6 +1594,11 @@ void TEWidget::mouseDoubleClickEvent(QMouseEvent* ev)
      while( ((x<columns-1) || (endSel.y()<lines-1 && m_line_wrapped[endSel.y()])) && charClass(image[i+1].c) == selClass )
      { i++; if (x<columns-1) x++; else {x=0; endSel.ry()++; } }
      endSel.setX(x);
+
+     // In word selection mode don't select @ (64) if at end of word.
+     if ( ( QChar( image[i].c ) == '@' ) && ( ( endSel.x() - bgnSel.x() ) > 0 ) )
+       endSel.setX( x - 1 );
+
      actSel = 2; // within selection
      emit extendSelectionSignal( endSel.x(), endSel.y() );
      emit endSelectionSignal(preserve_line_breaks);
@@ -1774,7 +1914,7 @@ void TEWidget::imEndEvent( QIMEvent *e )
                              contentsRect().width(), contentsRect().height() );
   m_imStart = 0;
   m_imPreeditLength = 0;
-
+ 
   m_isIMEdit = m_isIMSel = false;
   repaint( repaintRect, true );
 }
@@ -1832,6 +1972,8 @@ void TEWidget::setBellMode(int mode)
 
 void TEWidget::Bell(bool visibleSession, QString message)
 {
+  if (m_bellMode==BELLNONE) return;
+
   if (m_bellMode==BELLSYSTEM) {
     KNotifyClient::beep();
   } else if (m_bellMode==BELLNOTIFY) {
@@ -1942,6 +2084,11 @@ void TEWidget::setFixedSize(int cols, int lins)
   isFixedSize = true;
   columns = cols;
   lines = lins;
+  if (image)
+  {
+     free(image);
+     makeImage();
+  }
   setSize(cols, lins);
   QFrame::setFixedSize(m_size);
 }
@@ -2007,7 +2154,7 @@ void TEWidget::dropEvent(QDropEvent* event)
           dropText += " ";
 	  m_drop->setItemEnabled(cd,false);
         }
-        KURL url = *it;
+        KURL url = KIO::NetAccess::mostLocalURL( *it, 0 );
         QString tmp;
         if (url.isLocalFile()) {
           tmp = url.path(); // local URL : remove protocol. This helps "ln" & "cd" and doesn't harm the others
