@@ -14,10 +14,6 @@
 #include <qregexp.h>
 #include <qtextedit.h>
 
-#ifndef HERE
-#define HERE fprintf(stderr,"%s(%d): here\n",__FILE__,__LINE__)
-#endif
-
 /*! \class TESession
 
     Sessions are combinations of TEPTy and Emulations.
@@ -28,8 +24,9 @@
     of the abilities of the framework - multible sessions.
 */
 
-TESession::TESession(TEWidget* _te, const QString &_pgm, const QStrList & _args, const QString &_term, ulong _winId, const QString &_sessionId, const QString &_initial_cwd)
+TESession::TESession(TEWidget* _te, const QString &_term, ulong _winId, const QString &_sessionId, const QString &_initial_cwd)
    : DCOPObject( _sessionId.latin1() )
+   , sh(0)
    , connected(true)
    , monitorActivity(false)
    , monitorSilence(false)
@@ -42,17 +39,17 @@ TESession::TESession(TEWidget* _te, const QString &_pgm, const QStrList & _args,
    , silence_seconds(10)
    , add_to_utmp(true)
    , xon_xoff(false)
-   , pgm(_pgm)
-   , args(_args)
+   , pgm(QString())
+   , args(QStrList())
    , sessionId(_sessionId)
    , cwd("")
    , initial_cwd(_initial_cwd)
    , zmodemBusy(false)
    , zmodemProc(0)
    , zmodemProgress(0)
+   , encoding_no(0)
 {
   //kdDebug(1211)<<"TESession ctor() new TEPty"<<endl;
-  sh = new TEPty();
   te = _te;
   //kdDebug(1211)<<"TESession ctor() new TEmuVt102"<<endl;
   em = new TEmuVt102(te);
@@ -65,19 +62,12 @@ TESession::TESession(TEWidget* _te, const QString &_pgm, const QStrList & _args,
 
   term = _term;
   winId = _winId;
-  iconName = "openterm";
+  iconName = "konsole";
 
-  //kdDebug(1211)<<"TESession ctor() sh->setSize()"<<endl;
-  sh->setSize(te->Lines(),te->Columns()); // not absolutely nessesary
-  //kdDebug(1211)<<"TESession ctor() connecting"<<endl;
-  connect( sh,SIGNAL(block_in(const char*,int)),this,SLOT(onRcvBlock(const char*,int)) );
-
-  connect( em,SIGNAL(sndBlock(const char*,int)),sh,SLOT(send_bytes(const char*,int)) );
-  connect( em,SIGNAL(lockPty(bool)),sh,SLOT(lockPty(bool)) );
+  setPty( new TEPty() );
 
   connect( em, SIGNAL( changeTitle( int, const QString & ) ),
            this, SLOT( setUserTitle( int, const QString & ) ) );
-
   connect( em, SIGNAL( notifySessionState(int) ),
            this, SLOT( notifySessionState(int) ) );
   monitorTimer = new QTimer(this);
@@ -85,15 +75,46 @@ TESession::TESession(TEWidget* _te, const QString &_pgm, const QStrList & _args,
 
   connect( em, SIGNAL( zmodemDetected() ), this, SLOT(slotZModemDetected()));
 
-  connect( sh,SIGNAL(done(int)), this,SLOT(done(int)) );
+  connect( em, SIGNAL( changeTabTextColor( int ) ),
+           this, SLOT( changeTabTextColor( int ) ) );
+
   //kdDebug(1211)<<"TESession ctor() done"<<endl;
+}
+
+void TESession::setPty(TEPty *_sh)
+{
+  if ( sh ) {
+    delete sh;
+  }
+  sh = _sh;
+  connect( sh, SIGNAL( forkedChild() ),
+           this, SIGNAL( forkedChild() ));
+
+  //kdDebug(1211)<<"TESession ctor() sh->setSize()"<<endl;
+  sh->setSize(te->Lines(),te->Columns()); // not absolutely nessesary
+  sh->useUtf8(em->utf8());
+  //kdDebug(1211)<<"TESession ctor() connecting"<<endl;
+  connect( sh,SIGNAL(block_in(const char*,int)),this,SLOT(onRcvBlock(const char*,int)) );
+
+  connect( em,SIGNAL(sndBlock(const char*,int)),sh,SLOT(send_bytes(const char*,int)) );
+  connect( em,SIGNAL(lockPty(bool)),sh,SLOT(lockPty(bool)) );
+  connect( em,SIGNAL(useUtf8(bool)),sh,SLOT(useUtf8(bool)) );
+
+  connect( sh,SIGNAL(done(int)), this,SLOT(done(int)) );
+
   if (!sh->error().isEmpty())
      QTimer::singleShot(0, this, SLOT(ptyError()));
 }
 
 void TESession::ptyError()
 {
-  KMessageBox::error(te->topLevelWidget(), sh->error());
+  // FIXME:  sh->error() is always empty
+  if ( sh->error().isEmpty() )
+    KMessageBox::error( te->topLevelWidget(),
+       i18n("Konsole is unable to open a PTY (pseudo teletype).  It is likely that this is due to an incorrect configuration of the PTY devices.  Konsole needs to have read/write access to the PTY devices."), 
+       i18n("A Fatal Error Has Occurred") );
+  else
+    KMessageBox::error(te->topLevelWidget(), sh->error());
   emit done(this);
 }
 
@@ -117,9 +138,15 @@ void TESession::changeWidget(TEWidget* w)
                    this,SLOT(onFontMetricChange(int,int)));
 }
 
+void TESession::setProgram( const QString &_pgm, const QStrList &_args )
+{
+    pgm = _pgm;
+    args = _args;
+}
+
 void TESession::run()
 {
-  //kdDebug(1211) << "Running the session!" << pgm << "\n";
+//  kdDebug(1211) << "Running the session!" << pgm << "\n";
   //pgm = "pine";
   QString appId=kapp->dcopClient()->appId();
 
@@ -127,9 +154,17 @@ void TESession::run()
   if (!initial_cwd.isEmpty())
      QDir::setCurrent(initial_cwd);
   sh->setXonXoff(xon_xoff);
-  sh->run(QFile::encodeName(pgm), args, term.latin1(), winId, add_to_utmp,
+
+  int result = sh->run(QFile::encodeName(pgm), args, term.latin1(), 
+          winId, add_to_utmp,
           ("DCOPRef("+appId+",konsole)").latin1(),
           ("DCOPRef("+appId+","+sessionId+")").latin1());
+  if (result < 0) {     // Error in opening pseudo teletype
+    kdWarning()<<"Unable to open a pseudo teletype!"<<endl;
+    QTimer::singleShot(0, this, SLOT(ptyError()));
+  }
+  sh->setErase(em->getErase());
+
   if (!initial_cwd.isEmpty())
      QDir::setCurrent(cwd_save);
   else
@@ -138,6 +173,11 @@ void TESession::run()
   sh->setWriteable(false);  // We are reachable via kwrited.
 }
 
+void TESession::changeTabTextColor( int color )
+{
+    emit changeTabTextColor( this, color );
+}
+
 void TESession::setUserTitle( int what, const QString &caption )
 {
     // (btw: what=0 changes title and icon, what=1 only icon, what=2 only title
@@ -286,7 +326,7 @@ void TESession::setListenToKeyPress(bool l)
 }
 
 void TESession::done() {
-  emit processExited();
+  emit processExited(sh);
   emit done(this);
 }
 
@@ -312,7 +352,7 @@ void TESession::done(int exitStatus)
     else
       KNotifyClient::event(winId, "Finished", i18n("Session '%1' exited unexpectedly.").arg(title));
   }
-  emit processExited();
+  emit processExited(sh);
   emit done(this);
 }
 
@@ -333,6 +373,11 @@ int TESession::schemaNo()
   return schema_no;
 }
 
+int TESession::encodingNo()
+{
+  return encoding_no;
+}
+
 int TESession::keymapNo()
 {
   return em->keymapNo();
@@ -363,6 +408,11 @@ void TESession::setSchemaNo(int sn)
   schema_no = sn;
 }
 
+void TESession::setEncodingNo(int index)
+{
+  encoding_no = index;
+}
+  
 void TESession::setKeymapNo(int kn)
 {
   em->setKeymap(kn);
@@ -689,4 +739,50 @@ void TESession::print( QPainter &paint, bool friendly, bool exact )
     te->print(paint, friendly, exact);
 }
 
+QString TESession::schema()
+{
+  QString currentSchema;
+  emit getSessionSchema(this, currentSchema);
+  return currentSchema;
+}
+
+void TESession::setSchema(const QString &schema)
+{
+  emit setSessionSchema(this, schema);
+}
+
+QString TESession::encoding()
+{
+  return em->codec()->name();
+}
+
+void TESession::setEncoding(const QString &encoding)
+{
+  emit setSessionEncoding(this, encoding);
+}
+
+QString TESession::keytab()
+{
+   return keymap();
+}
+
+void TESession::setKeytab(const QString &keytab)
+{
+   setKeymap(keytab);
+   emit updateSessionConfig(this);
+}
+
+QSize TESession::size()
+{
+  return em->imageSize();
+}
+
+void TESession::setSize(QSize size)
+{
+  if ((size.width() <= 1) || (size.height() <= 1))
+     return;
+  
+  emit resizeSession(this, size);
+}
+
 #include "session.moc"
