@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>. Please do
     not commit any changes without consulting me first. Thanks!
@@ -64,7 +64,7 @@ class HostItem : public QListViewItem
 
 SensorBrowser::SensorBrowser( QWidget* parent, KSGRD::SensorManager* sm,
                               const char* name)
-  : QListView( parent, name ), mSensorManager( sm )
+  : KListView( parent, name ), mSensorManager( sm )
 {
   mHostInfoList.setAutoDelete(true);
 
@@ -76,6 +76,8 @@ SensorBrowser::SensorBrowser( QWidget* parent, KSGRD::SensorManager* sm,
 
   addColumn( i18n( "Sensor Browser" ) );
   addColumn( i18n( "Sensor Type" ) );
+  setFullWidth( true );
+
   QToolTip::add( this, i18n( "Drag sensors to empty cells of a worksheet "
                              "or the panel applet." ) );
   setRootIsDecorated( true );
