@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 	KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>.
 	Please do not commit any changes without consulting me first. Thanks!
@@ -30,7 +30,6 @@
 
 #include <kdebug.h>
 
-#include <ksgrd/ColorPicker.h>
 #include <ksgrd/SensorManager.h>
 #include <ksgrd/StyleEngine.h>
 
@@ -38,13 +37,12 @@
 #include "MultiMeterSettings.h"
 
 MultiMeter::MultiMeter(QWidget* parent, const char* name,
-				   const QString& title, double, double, bool nf)
-	: KSGRD::SensorDisplay(parent, name, title)
+				   const QString& title, double, double, bool nf, bool isApplet)
+	: KSGRD::SensorDisplay(parent, name, title, nf, isApplet)
 {
 	setShowUnit( true );
 	lowerLimit = upperLimit = 0;
 	lowerLimitActive = upperLimitActive = false;
-	setNoFrame( nf );
 
 	normalDigitColor = KSGRD::Style->firstForegroundColor();
 	alarmDigitColor = KSGRD::Style->alarmColor();
