@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>.
     Please do not commit any changes without consulting me first. Thanks!
@@ -114,7 +114,7 @@ bool WorkSheet::load( const QString &fileName )
   // Check for proper size.
   QDomElement element = doc.documentElement();
   updateInterval( element.attribute( "interval" ).toUInt() );
-  if ( updateInterval() < 2 || updateInterval() > 300 )
+  if ( updateInterval() < 1 || updateInterval() > 300 )
     updateInterval( 2 );
 
   bool rowsOk, columnsOk;
@@ -296,6 +296,11 @@ void WorkSheet::setTitle( const QString &title )
   mTitle = title;
 }
 
+QString WorkSheet::title() const
+{
+  return mTitle;
+}
+
 KSGRD::SensorDisplay *WorkSheet::addDisplay( const QString &hostName,
                                              const QString &sensorName,
                                              const QString &sensorType,
@@ -383,6 +388,9 @@ void WorkSheet::settings()
 
     resizeGrid( dlg.rows(), dlg.columns() );
 
+    mTitle = dlg.sheetTitle();
+    emit titleChanged( this );
+
     setModified( true );
   }
 }
@@ -454,11 +462,8 @@ void WorkSheet::customEvent( QCustomEvent *e )
 {
   if ( e->type() == QEvent::User ) {
     // SensorDisplays send out this event if they want to be removed.
-    if ( KMessageBox::warningContinueCancel( this, i18n( "Do you really want to delete the display?" ),
-      i18n("Delete Display"), KStdGuiItem::del() )
-         == KMessageBox::Continue ) {
-      removeDisplay( (KSGRD::SensorDisplay*)e->data() );
-    }
+
+    removeDisplay( (KSGRD::SensorDisplay*)e->data() );
   }
 }
 
@@ -594,7 +599,7 @@ void WorkSheet::resizeGrid( uint newRows, uint newColumns )
         delete mDisplayList[ r ][ c ];
     delete mDisplayList[ r ];
   }
-  delete mDisplayList;
+  delete [] mDisplayList;
 
   /* now we make the new display the regular one */
   mDisplayList = newDisplayList;
