@@ -70,6 +70,9 @@ class Client : public QObject, public KDecorationDefines
     // prefer isXXX() instead
         NET::WindowType windowType( bool direct = false, int supported_types = SUPPORTED_WINDOW_TYPES_MASK ) const;
         const WindowRules* rules() const;
+        void removeRule( Rules* r );
+        void setupWindowRules( bool ignore_temporary );
+        void applyWindowRules();
 
         QRect geometry() const;
         QSize size() const;
@@ -125,6 +128,7 @@ class Client : public QObject, public KDecorationDefines
         bool isMinimized() const;
         bool isMaximizable() const;
         QRect geometryRestore() const;
+        MaximizeMode maximizeModeRestore() const;
         MaximizeMode maximizeMode() const;
         bool isMinimizable() const;
         void setMaximize( bool vertically, bool horizontally );
@@ -172,7 +176,6 @@ class Client : public QObject, public KDecorationDefines
         bool isDialog() const;
         bool isSplash() const;
         bool isUtility() const;
-        bool isOverride() const; // not override redirect, but NET::Override
     // returns true for "special" windows and false for windows which are "normal"
     // (normal=window which has a border, can be moved by the user, can be closed, etc.)
     // true for Desktop, Dock, Splash, Override and TopMenu (and Toolbar??? - for now)
@@ -203,6 +206,9 @@ class Client : public QObject, public KDecorationDefines
         // plainResize() simply resizes
         void plainResize( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void plainResize( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
+        // resizeWithChecks() resizes according to gravity, and checks workarea position
+        void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
+        void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
         void keepInArea( QRect area, bool partial = false );
 
         void growHorizontal();
@@ -229,10 +235,10 @@ class Client : public QObject, public KDecorationDefines
 
         Colormap colormap() const;
 
+    // updates visibility depending on being shaded, virtual desktop, etc.
+        void updateVisibility();
     // hides a client - basically like minimize, but without effects, it's simply hidden
         void hideClient( bool hide );
-    // updates visibility depending on whether it's on the current desktop
-        void virtualDesktopChange();
 
         QString caption( bool full = true ) const;
         void updateCaption();
@@ -275,6 +281,7 @@ class Client : public QObject, public KDecorationDefines
         void maximize( MaximizeMode );
         void toggleShade();
         void showContextHelp();
+        void cancelShadeHover();
         void cancelAutoRaise();
         void destroyClient();
         void checkActiveModal();
@@ -341,6 +348,7 @@ class Client : public QObject, public KDecorationDefines
     private slots:
         void pingTimeout();
         void processKillerExited();
+        void demandAttentionKNotify();
 
     private:
     // ICCCM 4.1.3.1, 4.1.4 , NETWM 2.5.1
@@ -365,7 +373,6 @@ class Client : public QObject, public KDecorationDefines
         QString readName() const;
         void setCaption( const QString& s, bool force = false );
         bool hasTransientInternal( const Client* c, bool indirect, ConstClientList& set ) const;
-        void setupWindowRules( bool ignore_temporary );
         void updateWindowRules();
         void finishWindowRules();
         void setShortcutInternal( const KShortcut& cut );
@@ -374,9 +381,6 @@ class Client : public QObject, public KDecorationDefines
         void checkDirection( int new_diff, int old_diff, QRect& rect, const QRect& area );
         static int computeWorkareaDiff( int left, int right, int a_left, int a_right );
         void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool );
-        // resizeWithChecks() resizes according to gravity, and checks workarea position
-        void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
-        void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
         NETExtendedStrut strut() const;
         bool hasStrut() const;
         int checkShadeGeometry( int w, int h );
@@ -392,7 +396,8 @@ class Client : public QObject, public KDecorationDefines
         void ungrabButton( int mod );
         void resetMaximize();
         void resizeDecoration( const QSize& s );
-        void setXTitleHeightProperty(int titleHeight);
+        void setDecoHashProperty(uint topHeight, uint rightWidth, uint bottomHeight, uint leftWidth);
+        void unsetDecoHashProperty();
 
         void pingWindow();
         void killProcess( bool ask, Time timestamp = CurrentTime );
@@ -402,8 +407,9 @@ class Client : public QObject, public KDecorationDefines
 
         void embedClient( Window w, const XWindowAttributes &attr );    
         void detectNoBorder();
+        void detectShapable();
         void destroyDecoration();
-        void updateFrameStrut();
+        void updateFrameExtents();
 
         void rawShow(); // just shows it
         void rawHide(); // just hides it
@@ -457,6 +463,7 @@ class Client : public QObject, public KDecorationDefines
         ClientList transients_list; // SELI make this ordered in stacking order?
         ShadeMode shade_mode;
         uint active :1;
+        uint deleting : 1; // true when doing cleanup and destroying the client
         uint keep_above : 1; // NET::KeepAbove (was stays_on_top)
         uint is_shape :1;
         uint skip_taskbar :1;
@@ -481,6 +488,7 @@ class Client : public QObject, public KDecorationDefines
         uint not_obscured : 1;
         uint urgency : 1; // XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; // don't apply focus stealing prevention to this client
+        uint demands_attention : 1;
         WindowRules client_rules;
         void getWMHints();
         void readIcons();
@@ -495,6 +503,7 @@ class Client : public QObject, public KDecorationDefines
         MaximizeMode max_mode;
         QRect geom_restore;
         QRect geom_fs_restore;
+        MaximizeMode maxmode_restore;
         int workarea_diff_x, workarea_diff_y;
         WinInfo* info;
         QTimer* autoRaiseTimer;
@@ -536,7 +545,7 @@ class Client : public QObject, public KDecorationDefines
         uint rule_opacity_inactive; //dto.
         //int shadeOriginalHeight;
         bool isBMP_;
-        
+        QTimer* demandAttentionKNotifyTimer;
     };
 
 // helper for Client::postponeGeometryUpdates() being called in pairs (true/false)
@@ -716,6 +725,11 @@ inline QRect Client::geometryRestore() const
     return geom_restore;
     }
 
+inline Client::MaximizeMode Client::maximizeModeRestore() const
+    {
+    return maxmode_restore;
+    }
+
 inline Client::MaximizeMode Client::maximizeMode() const
     {
     return max_mode;
@@ -903,7 +917,12 @@ inline void Client::setBMP(bool b)
     {
     isBMP_ = b;
     }
-    
+
+inline void Client::removeRule( Rules* rule )
+    {
+    client_rules.remove( rule );
+    }
+
 #ifdef NDEBUG
 inline
 kndbgstream& operator<<( kndbgstream& stream, const Client* ) { return stream; }
