@@ -248,6 +248,7 @@ QRect Workspace::clientArea( clientAreaOption opt, const QPoint& p, int desktop
                 return desktopwidget->geometry();
         case WorkArea:
             return warea;
+        case FullXineramaArea:
         case FullArea:
             return desktopwidget->geometry();
         case ScreenArea:
@@ -595,7 +596,8 @@ void Workspace::cascadeDesktop()
 // TODO XINERAMA this probably is not right for xinerama
     Q_ASSERT( block_stacking_updates == 0 );
     ClientList::ConstIterator it(stackingOrder().begin());
-    bool re_init_cascade_at_first_client = true;
+    initPositioning->reinitCascading( currentDesktop());
+    QRect area = clientArea( PlacementArea, QPoint( 0, 0 ), currentDesktop());
     for (; it != stackingOrder().end(); ++it)
         {
         if((!(*it)->isOnDesktop(currentDesktop())) ||
@@ -603,10 +605,7 @@ void Workspace::cascadeDesktop()
            ((*it)->isOnAllDesktops())              ||
            (!(*it)->isMovable()) )
             continue;
-        initPositioning->placeCascaded(*it, QRect(), re_init_cascade_at_first_client);
-        //CT is an if faster than an attribution?
-        if (re_init_cascade_at_first_client)
-          re_init_cascade_at_first_client = false;
+        initPositioning->placeCascaded(*it, area);
         }
     }
 
@@ -882,8 +881,6 @@ void Client::checkWorkspacePosition()
         }
     if( isDock())
         return;
-    if( isOverride())
-        return; // I wish I knew what to do here :(
     if( isTopMenu())
         {
         if( workspace()->managingTopMenus())
@@ -1557,7 +1554,7 @@ bool Client::isMovable() const
     {
     if( !motif_may_move || isFullScreen())
         return false;
-    if( isSpecialWindow() && !isOverride() && !isSplash() && !isToolbar()) // allow moving of splashscreens :)
+    if( isSpecialWindow() && !isSplash() && !isToolbar()) // allow moving of splashscreens :)
         return false;
     if( maximizeMode() == MaximizeFull && !options->moveResizeMaximizedWindows() )
         return false;
@@ -1573,7 +1570,7 @@ bool Client::isResizable() const
     {
     if( !motif_may_resize || isFullScreen())
         return false;
-    if(( isSpecialWindow() || isSplash() || isToolbar()) && !isOverride())
+    if( isSpecialWindow() )
         return false;
     if( maximizeMode() == MaximizeFull && !options->moveResizeMaximizedWindows() )
         return false;
@@ -1657,8 +1654,9 @@ void Client::setGeometry( int x, int y, int w, int h, ForceGeometry_t force )
         pending_geometry_update = true;
         return;
         }
-    XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
     resizeDecoration( QSize( w, h ));
+    XMoveResizeWindow( qt_xdisplay(), frameId(), x, y, w, h );
+//     resizeDecoration( QSize( w, h ));
     if( !isShade())
         {
         QSize cs = clientSize();
@@ -1711,8 +1709,9 @@ void Client::plainResize( int w, int h, ForceGeometry_t force )
         pending_geometry_update = true;
         return;
         }
-    XResizeWindow( qt_xdisplay(), frameId(), w, h );
     resizeDecoration( QSize( w, h ));
+    XResizeWindow( qt_xdisplay(), frameId(), w, h );
+//     resizeDecoration( QSize( w, h ));
     if( !isShade())
         {
         QSize cs = clientSize();
@@ -1817,12 +1816,12 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     QRect clientArea = workspace()->clientArea( MaximizeArea, this );
 
     // save sizes for restoring, if maximalizing
-    if( !( y() == clientArea.top() && height() == clientArea.height()))
+    if( !adjust && !( y() == clientArea.top() && height() == clientArea.height()))
         {
         geom_restore.setTop( y());
         geom_restore.setHeight( height());
         }
-    if( !( x() == clientArea.left() && width() == clientArea.width()))
+    if( !adjust && !( x() == clientArea.left() && width() == clientArea.width()))
         {
         geom_restore.setLeft( x());
         geom_restore.setWidth( width());
@@ -1840,6 +1839,21 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
     if( decoration != NULL ) // decorations may turn off some borders when maximized
         decoration->borders( border_left, border_right, border_top, border_bottom );
 
+    // restore partial maximizations
+    if ( old_mode==MaximizeFull && max_mode==MaximizeRestore )
+        {
+        if ( maximizeModeRestore()==MaximizeVertical )
+	    {
+	    max_mode = MaximizeVertical;
+	    maxmode_restore = MaximizeRestore;
+	    }
+	if ( maximizeModeRestore()==MaximizeHorizontal )
+	    {
+	    max_mode = MaximizeHorizontal;
+	    maxmode_restore = MaximizeRestore;
+	    }	
+	}
+    
     switch (max_mode)
         {
 
@@ -1854,11 +1868,11 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                     }
                 else
                     setGeometry( QRect(QPoint( geom_restore.x(), clientArea.top()),
-                              adjustedSize(QSize( geom_restore.width(), clientArea.height()), SizemodeFixedH )));
+                              adjustedSize(QSize( geom_restore.width(), clientArea.height()), SizemodeFixedH )), ForceGeometrySet);
                 }
             else
                 setGeometry( QRect(QPoint(x(), clientArea.top()),
-                              adjustedSize(QSize(width(), clientArea.height()), SizemodeFixedH )));
+                              adjustedSize(QSize(width(), clientArea.height()), SizemodeFixedH )), ForceGeometrySet);
             info->setState( NET::MaxVert, NET::Max );
             break;
             }
@@ -1874,11 +1888,11 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                     }
                 else
                     setGeometry( QRect( QPoint(clientArea.left(), geom_restore.y()),
-                              adjustedSize(QSize(clientArea.width(), geom_restore.height()), SizemodeFixedW )));
+                              adjustedSize(QSize(clientArea.width(), geom_restore.height()), SizemodeFixedW )), ForceGeometrySet);
                 }
             else
                 setGeometry( QRect( QPoint(clientArea.left(), y()),
-                              adjustedSize(QSize(clientArea.width(), height()), SizemodeFixedW )));
+                              adjustedSize(QSize(clientArea.width(), height()), SizemodeFixedW )), ForceGeometrySet);
             info->setState( NET::MaxHoriz, NET::Max );
             break;
             }
@@ -1912,16 +1926,23 @@ void Client::changeMaximize( bool vertical, bool horizontal, bool adjust )
                 if( geom_restore.height() > 0 )
                     restore.moveTop( geom_restore.y());
                 }
-            setGeometry( restore );
+            setGeometry( restore, ForceGeometrySet );
             info->setState( 0, NET::Max );
             break;
             }
 
         case MaximizeFull:
             {
+            if( !adjust )
+                {
+                if( old_mode & MaximizeVertical )
+                    maxmode_restore = MaximizeVertical;
+                if( old_mode & MaximizeHorizontal )
+    	            maxmode_restore = MaximizeHorizontal;
+                }
             QSize adjSize = adjustedSize(clientArea.size(), SizemodeMax );
             QRect r = QRect(clientArea.topLeft(), adjSize);
-            setGeometry( r );
+            setGeometry( r, ForceGeometrySet );
             info->setState( NET::Max, NET::Max );
             break;
             }
@@ -1998,7 +2019,7 @@ bool Client::isFullScreenable( bool fullscreen_hack ) const
     if( !rules()->checkFullScreen( true ))
         return false;
     if( fullscreen_hack )
-        return isNormalWindow() || isOverride();
+        return isNormalWindow();
     if( rules()->checkStrictGeometry( false ))
         {
         // the app wouldn't fit exactly fullscreen geometry due its strict geometry requirements
@@ -2042,7 +2063,12 @@ void Client::setFullScreen( bool set, bool user )
     workspace()->updateClientLayer( this ); // active fullscreens get different layer
     info->setState( isFullScreen() ? NET::FullScreen : 0, NET::FullScreen );
     updateDecoration( false, false );
-    if( isFullScreen())
+    // XINERAMA
+    if( isFullScreen() && xSizeHint.width > workspace()->clientArea(FullScreenArea, this).width() )
+        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
+    else if( isFullScreen() && xSizeHint.height > workspace()->clientArea(FullScreenArea, this).height() )
+        setGeometry( workspace()->clientArea( FullXineramaArea, this ));
+    else if( isFullScreen())
         setGeometry( workspace()->clientArea( FullScreenArea, this ));
     else
         {
