@@ -148,6 +148,7 @@ Group* Workspace::findGroup( Window leader ) const
 // group with windows with the same client leader.
 Group* Workspace::findClientLeaderGroup( const Client* c ) const
     {
+    Group* ret = NULL;
     for( ClientList::ConstIterator it = clients.begin();
          it != clients.end();
          ++it )
@@ -155,9 +156,28 @@ Group* Workspace::findClientLeaderGroup( const Client* c ) const
         if( *it == c )
             continue;
         if( (*it)->wmClientLeader() == c->wmClientLeader())
-            return (*it)->group();
+            {
+            if( ret == NULL || ret == (*it)->group())
+                ret = (*it)->group();
+            else
+                {
+                // There are already two groups with the same client leader.
+                // This most probably means the app uses group transients without
+                // setting group for its windows. Merging the two groups is a bad
+                // hack, but there's no really good solution for this case.
+                Group* old_group = (*it)->group();
+                // old_group autodeletes when being empty
+                for( int cnt = old_group->members().count();
+                     cnt > 0;
+                     --cnt )
+                    {
+                    Client* tmp = old_group->members().first();
+                    tmp->checkGroup( ret ); // change group
+                    }
+                }
+            }
         }
-    return NULL;
+    return ret;
     }
 
 void Workspace::updateMinimizedOfTransients( Client* c )
@@ -170,10 +190,9 @@ void Workspace::updateMinimizedOfTransients( Client* c )
              ++it )
             {
             if( !(*it)->isMinimized()
-                 && !(*it)->isShade()
                  && !(*it)->isTopMenu() ) // topmenus are not minimized, they're hidden
                 {
-                (*it)->minimize();
+                (*it)->minimize( true ); // avoid animation
                 updateMinimizedOfTransients( (*it) );
                 }
             }
@@ -187,7 +206,7 @@ void Workspace::updateMinimizedOfTransients( Client* c )
             if( (*it)->isMinimized()
                 && !(*it)->isTopMenu())
                 {
-                (*it)->unminimize();
+                (*it)->unminimize( true ); // avoid animation
                 updateMinimizedOfTransients( (*it) );
                 }
             }
@@ -247,7 +266,7 @@ bool Client::belongToSameApplication( const Client* c1, const Client* c2, bool a
     else if( c2->isTransient() && c1->hasTransient( c2, true ))
         same_app = true; // c2 has c1 as mainwindow
     else if( c1->pid() != c2->pid()
-        || c1->wmClientMachine() != c2->wmClientMachine())
+        || c1->wmClientMachine( false ) != c2->wmClientMachine( false ))
         ; // different processes
     else if( c1->wmClientLeader() != c2->wmClientLeader()
         && c1->wmClientLeader() != c1->window() // if WM_CLIENT_LEADER is not set, it returns window(),
@@ -398,20 +417,10 @@ void Client::setTransient( Window new_transient_for_id )
             transient_for = workspace()->findClient( WindowMatchPredicate( transient_for_id ));
             assert( transient_for != NULL ); // verifyTransient() had to check this
             transient_for->addTransient( this );
-            }
-        checkGroup(); // first check group
-        if( groupTransient())
-            { // and make transient for all in the group
-            for( ClientList::ConstIterator it = group()->members().begin();
-                 it != group()->members().end();
-                 ++it )
-                {
-                if( *it == this )
-                    break; // this means the window is only transient for windows mapped before it
-                (*it)->addTransient( this );
-                }
-            }
-        checkGroupTransients();
+            } // checkGroup() will check 'check_active_modal'
+        checkGroup( NULL, true ); // force, because transiency has changed
+        if( isTopMenu())
+            workspace()->updateCurrentTopMenu();
         workspace()->updateClientLayer( this );
         }
     }
@@ -532,6 +541,25 @@ void Client::checkGroupTransients()
             // TODO This could possibly be optimized, it also requires hasTransient() to check for loops.
             if( (*it2)->groupTransient() && (*it1)->hasTransient( *it2, true ) && (*it2)->hasTransient( *it1, true ))
                 (*it2)->transients_list.remove( *it1 );
+            // if there are already windows W1 and W2, W2 being transient for W1, and group transient W3
+            // is added, make it transient only for W2, not for W1, because it's already indirectly
+            // transient for it - the indirect transiency actually shouldn't break anything,
+            // but it can lead to exponentially expensive operations (#95231)
+            // TODO this is pretty slow as well
+            for( ClientList::ConstIterator it3 = group()->members().begin();
+                 it3 != group()->members().end();
+                 ++it3 )
+                {
+                if( *it1 == *it2 || *it2 == *it3 || *it1 == *it3 )
+                    continue;
+                if( (*it2)->hasTransient( *it1, false ) && (*it3)->hasTransient( *it1, false ))
+                    {
+                    if( (*it2)->hasTransient( *it3, true ))
+                        (*it3)->transients_list.remove( *it1 );
+                    if( (*it3)->hasTransient( *it2, true ))
+                        (*it2)->transients_list.remove( *it1 );
+                    }
+                }
             }
         }
     }
@@ -618,8 +646,8 @@ void Client::addTransient( Client* cl )
 //    assert( !cl->hasTransient( this, true )); will be fixed in checkGroupTransients()
     assert( cl != this );
     transients_list.append( cl );
-    if( workspace()->mostRecentlyActivatedClient() == this && cl->isModal())
-        workspace()->activateClient( findModal());
+    if( workspace()->mostRecentlyActivatedClient() == this && cl->isModal())        
+        check_active_modal = true;
 //    kdDebug() << "ADDTRANS:" << this << ":" << cl << endl;
 //    kdDebug() << kdBacktrace() << endl;
 //    for( ClientList::ConstIterator it = transients_list.begin();
@@ -664,15 +692,15 @@ bool Client::hasTransient( const Client* cl, bool indirect ) const
 
 bool Client::hasTransientInternal( const Client* cl, bool indirect, ConstClientList& set ) const
     {
-    if( set.contains( this ))
-        return false;
-    set.append( this );
     if( cl->transientFor() != NULL )
         {
         if( cl->transientFor() == this )
             return true;
         if( !indirect )
             return false;
+        if( set.contains( cl ))
+            return false;
+        set.append( cl );
         return hasTransientInternal( cl->transientFor(), indirect, set );
         }
     if( !cl->isTransient())
@@ -684,6 +712,9 @@ bool Client::hasTransientInternal( const Client* cl, bool indirect, ConstClientL
         return true;
     if( !indirect )
         return false;
+    if( set.contains( this ))
+        return false;
+    set.append( this );
     for( ClientList::ConstIterator it = transients().begin();
          it != transients().end();
          ++it )
@@ -721,10 +752,21 @@ Client* Client::findModal()
 
 // Client::window_group only holds the contents of the hint,
 // but it should be used only to find the group, not for anything else
-void Client::checkGroup()
+// Argument is only when some specific group needs to be set.
+void Client::checkGroup( Group* set_group, bool force )
     {
     Group* old_group = in_group;
-    if( window_group != None )
+    if( set_group != NULL )
+        {
+        if( set_group != in_group )
+            {
+            if( in_group != NULL )
+                in_group->removeMember( this );
+            in_group = set_group;
+            in_group->addMember( this );
+            }
+        }
+    else if( window_group != None )
         {
         Group* new_group = workspace()->findGroup( window_group );
         if( transientFor() != NULL && transientFor()->group() != new_group )
@@ -784,7 +826,7 @@ void Client::checkGroup()
                 }
             }
         }
-    if( in_group != old_group )
+    if( in_group != old_group || force )
         {
         for( ClientList::Iterator it = transients_list.begin();
              it != transients_list.end();
@@ -795,6 +837,17 @@ void Client::checkGroup()
             else
                 ++it;
             }
+        if( groupTransient())
+            { // and make transient for all in the group
+            for( ClientList::ConstIterator it = group()->members().begin();
+                 it != group()->members().end();
+                 ++it )
+                {
+                if( *it == this )
+                    break; // this means the window is only transient for windows mapped before it
+                (*it)->addTransient( this );
+                }
+            }
 #if 0 // TODO
         if( groupTransient())
             {
@@ -814,21 +867,45 @@ void Client::checkGroup()
                 (*it)->addTransient( this );
             }
 #endif
-#if 0 // this would make group transients transient for window that were mapped later
+        // group transient splashscreens should be transient even for windows
+        // in group mapped later
         for( ClientList::ConstIterator it = group()->members().begin();
              it != group()->members().end();
              ++it )
             {
-            if( !(*it)->groupTransient())  // and group transients in the new group are transient for it
+            if( !(*it)->isSplash())
                 continue;
-            if( *it == this )
+            if( !(*it)->groupTransient())
+                continue;
+            if( *it == this || hasTransient( *it, true )) // TODO indirect?
                 continue;
             addTransient( *it );
 	    }
-        checkGroupTransients();
-#endif
+        }
+    checkGroupTransients();
+    checkActiveModal();
+    workspace()->updateClientLayer( this );
+    }
+
+bool Client::check_active_modal = false;
+
+void Client::checkActiveModal()
+    {
+    // if the active window got new modal transient, activate it.
+    // cannot be done in AddTransient(), because there may temporarily
+    // exist loops, breaking findModal
+    Client* check_modal = workspace()->mostRecentlyActivatedClient();
+    if( check_modal != NULL && check_modal->check_active_modal )
+        {
+        Client* new_modal = check_modal->findModal();
+        if( new_modal != NULL && new_modal != check_modal )
+            {
+            if( !new_modal->isManaged())
+                return; // postpone check until end of manage()
+            workspace()->activateClient( new_modal );
+            }
+        check_modal->check_active_modal = false;
         }
     }
 
-
 } // namespace
