@@ -13,7 +13,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Cambridge, MA 02110-1301, USA.
  */
 
 #include "ruleswidget.h"
@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <kmessagebox.h>
 #include <qtabwidget.h>
+#include <qtimer.h>
 
 #include "../../rules.h"
 
@@ -61,13 +62,21 @@ RulesWidget::RulesWidget( QWidget* parent, const char* name )
               " after the window is created. No further changes will be affected.</li>"
               "<li><em>Remember:</em> The value of the window property will be remembered and every time"
               " time the window is created, the last remembered value will be applied.</li>"
-              "<li><em>Force:</em> The window property will be always forced to the given value.</li></ul>" );
+              "<li><em>Force:</em> The window property will be always forced to the given value.</li>"
+              "<li><em>Apply Now:</em> The window property will be set to the given value immediately"
+              " and will not be affected later (this action will be deleted afterwards).</li>"
+              "<li><em>Force temporarily:</em> The window property will be forced to the given value"
+              " until it is hidden (this action will be deleted after the window is hidden).</li>"
+              "</ul>" );
     QString forceRuleDesc =
         i18n( "Specify how the window property should be affected:<ul>"
               "<li><em>Do Not Affect:</em> The window property will not be affected and therefore"
               " the default handling for it will be used. Specifying this will block more generic"
               " window settings from taking effect.</li>"
-              "<li><em>Force:</em> The window property will be always forced to the given value.</li></ul>" );
+              "<li><em>Force:</em> The window property will be always forced to the given value.</li>"
+              "<li><em>Force temporarily:</em> The window property will be forced to the given value"
+              " until it is hidden (this action will be deleted after the window is hidden).</li>"
+              "</ul>" );
     // window tabs have enable signals done in designer
     // geometry tab
     SETUP( position, set );
@@ -87,6 +96,9 @@ RulesWidget::RulesWidget( QWidget* parent, const char* name )
     SETUP( skippager, set );
     SETUP( acceptfocus, force );
     SETUP( closeable, force );
+    SETUP( opacityactive, force );
+    SETUP( opacityinactive, force );
+    SETUP( shortcut, force );
     // workarounds tab
     SETUP( fsplevel, force );
     SETUP( moveresizemode, force );
@@ -94,16 +106,14 @@ RulesWidget::RulesWidget( QWidget* parent, const char* name )
     SETUP( ignoreposition, force );
     SETUP( minsize, force );
     SETUP( maxsize, force );
+    SETUP( strictgeometry, force );
+    SETUP( disableglobalshortcuts, force );
     KWinModule module;
     int i;
     for( i = 1;
          i <= module.numberOfDesktops();
          ++i )
         desktop->insertItem( QString::number( i ).rightJustify( 2 ) + ":" + module.desktopName( i ));
-    for(;
-         i <= 16;
-         ++i )
-        desktop->insertItem( QString::number( i ).rightJustify( 2 ));
     desktop->insertItem( i18n( "All Desktops" ));
     }
 
@@ -134,6 +144,13 @@ UPDATE_ENABLE_SLOT( skiptaskbar )
 UPDATE_ENABLE_SLOT( skippager )
 UPDATE_ENABLE_SLOT( acceptfocus )
 UPDATE_ENABLE_SLOT( closeable )
+UPDATE_ENABLE_SLOT( opacityactive )
+UPDATE_ENABLE_SLOT( opacityinactive )
+void RulesWidget::updateEnableshortcut()
+    {
+    shortcut->setEnabled( enable_shortcut->isChecked() && rule_shortcut->currentItem() != 0 );
+    shortcut_edit->setEnabled( enable_shortcut->isChecked() && rule_shortcut->currentItem() != 0 );
+    }
 // workarounds tab
 UPDATE_ENABLE_SLOT( fsplevel )
 UPDATE_ENABLE_SLOT( moveresizemode )
@@ -141,6 +158,8 @@ UPDATE_ENABLE_SLOT( type )
 UPDATE_ENABLE_SLOT( ignoreposition )
 UPDATE_ENABLE_SLOT( minsize )
 UPDATE_ENABLE_SLOT( maxsize )
+UPDATE_ENABLE_SLOT( strictgeometry )
+UPDATE_ENABLE_SLOT( disableglobalshortcuts )
 
 #undef UPDATE_ENABLE_SLOT
 
@@ -151,6 +170,8 @@ static const int set_rule_to_combo[] =
     3, // Force
     1, // Apply
     2, // Remember
+    4, // ApplyNow
+    5  // ForceTemporarily
     };
 
 static const Rules::SetRule combo_to_set_rule[] =
@@ -158,20 +179,27 @@ static const Rules::SetRule combo_to_set_rule[] =
     ( Rules::SetRule )Rules::DontAffect,
     ( Rules::SetRule )Rules::Apply,
     ( Rules::SetRule )Rules::Remember,
-    ( Rules::SetRule )Rules::Force
+    ( Rules::SetRule )Rules::Force,
+    ( Rules::SetRule )Rules::ApplyNow,
+    ( Rules::SetRule )Rules::ForceTemporarily
     };
 
 static const int force_rule_to_combo[] =
     {
     0, // Unused
     0, // Don't Affect
-    1 // Force
+    1, // Force
+    0, // Apply
+    0, // Remember
+    0, // ApplyNow
+    2  // ForceTemporarily
     };
 
 static const Rules::ForceRule combo_to_force_rule[] =
     {
     ( Rules::ForceRule )Rules::DontAffect,
-    ( Rules::ForceRule )Rules::Force
+    ( Rules::ForceRule )Rules::Force,
+    ( Rules::ForceRule )Rules::ForceTemporarily
     };
 
 static QString positionToStr( const QPoint& p )
@@ -204,17 +232,32 @@ static QSize strToSize( const QString& str )
     return QSize( reg.cap( 1 ).toInt(), reg.cap( 2 ).toInt());
     }
 
-
-static int desktopToCombo( int desktop )
+//used for opacity settings
+static QString intToStr( const int& s )
     {
-    if( desktop >= 1 && desktop <= 16 )
-        return desktop - 1;
-    return 16; // on all desktops
+    if( s < 1 || s > 100 )
+        return QString::null;
+    return QString::number(s);
+    }
+ 
+static int strToInt( const QString& str )
+    {
+    int tmp = str.toInt();
+    if( tmp < 1 || tmp > 100 )
+        return 100;
+    return tmp;
+    }    
+    
+int RulesWidget::desktopToCombo( int d ) const
+    {
+    if( d >= 1 && d < desktop->count())
+        return d - 1;
+    return desktop->count() - 1; // on all desktops
     }
 
-static int comboToDesktop( int val )
+int RulesWidget::comboToDesktop( int val ) const
     {
-    if( val == 16 )
+    if( val == desktop->count() - 1 )
         return NET::OnAllDesktops;
     return val + 1;
     }
@@ -225,13 +268,15 @@ static int placementToCombo( Placement::Policy placement )
         {
         1, // NoPlacement
         0, // Default
-        5, // Random
+        0, // Unknown
+        6, // Random
         2, // Smart
-        3, // Cascade
-        4, // Centered
-        6, // ZeroCornered
-        7, // UnderMouse
-        8 // OnMainWindow
+        4, // Cascade
+        5, // Centered
+        7, // ZeroCornered
+        8, // UnderMouse
+        9, // OnMainWindow
+        3  // Maximizing
         };
     return conv[ placement ];
     }
@@ -243,12 +288,14 @@ static Placement::Policy comboToPlacement( int val )
         Placement::Default,
         Placement::NoPlacement,
         Placement::Smart,
+        Placement::Maximizing,
         Placement::Cascade,
         Placement::Centered,
         Placement::Random,
         Placement::ZeroCornered,
         Placement::UnderMouse,
         Placement::OnMainWindow
+        // no Placement::Unknown
         };
     return conv[ val ];
     }
@@ -372,12 +419,17 @@ void RulesWidget::setRules( Rules* rules )
     CHECKBOX_SET_RULE( skippager, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    LINEEDIT_FORCE_RULE( opacityactive, intToStr );
+    LINEEDIT_FORCE_RULE( opacityinactive, intToStr );
+    LINEEDIT_SET_RULE( shortcut, );
     COMBOBOX_FORCE_RULE( fsplevel, );
     COMBOBOX_FORCE_RULE( moveresizemode, moveresizeToCombo );
     COMBOBOX_FORCE_RULE( type, typeToCombo );
     CHECKBOX_FORCE_RULE( ignoreposition, );
     LINEEDIT_FORCE_RULE( minsize, sizeToStr );
     LINEEDIT_FORCE_RULE( maxsize, sizeToStr );
+    CHECKBOX_FORCE_RULE( strictgeometry, );
+    CHECKBOX_FORCE_RULE( disableglobalshortcuts, );
     }
 
 #undef GENERIC_RULE
@@ -457,12 +509,17 @@ Rules* RulesWidget::rules() const
     CHECKBOX_SET_RULE( skippager, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    LINEEDIT_FORCE_RULE( opacityactive, strToInt );
+    LINEEDIT_FORCE_RULE( opacityinactive, strToInt );
+    LINEEDIT_SET_RULE( shortcut, );
     COMBOBOX_FORCE_RULE( fsplevel, );
     COMBOBOX_FORCE_RULE( moveresizemode, comboToMoveResize );
     COMBOBOX_FORCE_RULE( type, comboToType );
     CHECKBOX_FORCE_RULE( ignoreposition, );
     LINEEDIT_FORCE_RULE( minsize, strToSize );
     LINEEDIT_FORCE_RULE( maxsize, strToSize );
+    CHECKBOX_FORCE_RULE( strictgeometry, );
+    CHECKBOX_FORCE_RULE( disableglobalshortcuts, );
     return rules;
     }
 
@@ -569,12 +626,17 @@ void RulesWidget::prefillUnusedValues( const KWin::WindowInfo& info )
     CHECKBOX_PREFILL( skippager,, info.state() & NET::SkipPager );
     //CHECKBOX_PREFILL( acceptfocus, );
     //CHECKBOX_PREFILL( closeable, );
+    LINEEDIT_PREFILL( opacityactive, intToStr, 100 /*get the actual opacity somehow*/);
+    LINEEDIT_PREFILL( opacityinactive, intToStr, 100 /*get the actual opacity somehow*/);
+    //LINEEDIT_PREFILL( shortcut, );
     //COMBOBOX_PREFILL( fsplevel, );
     //COMBOBOX_PREFILL( moveresizemode, moveresizeToCombo );
     COMBOBOX_PREFILL( type, typeToCombo, info.windowType( SUPPORTED_WINDOW_TYPES_MASK ) );
     //CHECKBOX_PREFILL( ignoreposition, );
     LINEEDIT_PREFILL( minsize, sizeToStr, info.frameGeometry().size() );
     LINEEDIT_PREFILL( maxsize, sizeToStr, info.frameGeometry().size() );
+    //CHECKBOX_PREFILL( strictgeometry, );
+    //CHECKBOX_PREFILL( disableglobalshortcuts, );
     }
 
 #undef GENERIC_PREFILL
@@ -616,8 +678,16 @@ void RulesWidget::prepareWindowSpecific( WId window )
     prefillUnusedValues( info );
     }
 
+void RulesWidget::shortcutEditClicked()
+    {
+    EditShortcutDialog dlg( topLevelWidget());
+    dlg.setShortcut( shortcut->text());
+    if( dlg.exec() == QDialog::Accepted )
+        shortcut->setText( dlg.shortcut());
+    }
+
 RulesDialog::RulesDialog( QWidget* parent, const char* name )
-: KDialogBase( parent, name, true, "", Ok | Cancel )
+: KDialogBase( parent, name, true, i18n( "Edit Window-Specific Settings" ), Ok | Cancel )
     {
     widget = new RulesWidget( this );
     setMainWidget( widget );
@@ -625,16 +695,31 @@ RulesDialog::RulesDialog( QWidget* parent, const char* name )
 
 // window is set only for Alt+F3/Window-specific settings, because the dialog
 // is then related to one specific window
-Rules* RulesDialog::edit( Rules* r, WId window )
+Rules* RulesDialog::edit( Rules* r, WId window, bool show_hints )
     {
     rules = r;
     widget->setRules( rules );
     if( window != 0 )
         widget->prepareWindowSpecific( window );
+    if( show_hints )
+        QTimer::singleShot( 0, this, SLOT( displayHints()));
     exec();
     return rules;
     }
 
+void RulesDialog::displayHints()
+    {
+    QString str = "<qt><p>";
+    str += i18n( "This configuration dialog allows altering settings only for the selected window"
+                 " or application. Find the setting you want to affect, enable the setting using the checkbox,"
+                 " select in what way the setting should be affected and to which value." );
+#if 0 // maybe later
+    str += "</p><p>" + i18n( "Consult the documentation for more details." );
+#endif
+    str += "</p></qt>";
+    KMessageBox::information( this, str, QString::null, "displayhints" );
+    }
+
 void RulesDialog::accept()
     {
     if( !widget->finalCheck())
@@ -643,6 +728,76 @@ void RulesDialog::accept()
     KDialogBase::accept();
     }
 
+EditShortcut::EditShortcut( QWidget* parent, const char* name )
+: EditShortcutBase( parent, name )
+    {
+    }
+
+void EditShortcut::editShortcut()
+    {
+    ShortcutDialog dlg( KShortcut( shortcut->text()), topLevelWidget());
+    if( dlg.exec() == QDialog::Accepted )
+        shortcut->setText( dlg.shortcut().toString());
+    }
+
+void EditShortcut::clearShortcut()
+    {
+    shortcut->setText( "" );
+    }
+
+EditShortcutDialog::EditShortcutDialog( QWidget* parent, const char* name )
+: KDialogBase( parent, name, true, i18n( "Edit Shortcut" ), Ok | Cancel )
+    {
+    widget = new EditShortcut( this );
+    setMainWidget( widget );
+    }
+
+void EditShortcutDialog::setShortcut( const QString& cut )
+    {
+    widget->shortcut->setText( cut );
+    }
+
+QString EditShortcutDialog::shortcut() const
+    {
+    return widget->shortcut->text();
+    }
+
+ShortcutDialog::ShortcutDialog( const KShortcut& cut, QWidget* parent, const char* name )
+    : KShortcutDialog( cut, false /*TODO???*/, parent, name )
+    {
+    }
+
+void ShortcutDialog::accept()
+    {
+    for( int i = 0;
+         ;
+         ++i )
+        {
+        KKeySequence seq = shortcut().seq( i );
+        if( seq.isNull())
+            break;
+        if( seq.key( 0 ) == Key_Escape )
+            {
+            reject();
+            return;
+            }
+        if( seq.key( 0 ) == Key_Space )
+            { // clear
+            setShortcut( KShortcut());
+            KShortcutDialog::accept();
+            return;
+            }
+        if( seq.key( 0 ).modFlags() == 0 )
+            { // no shortcuts without modifiers
+            KShortcut cut = shortcut();
+            cut.setSeq( i, KKeySequence());
+            setShortcut( cut );
+            return;
+            }
+        }
+    KShortcutDialog::accept();
+    }
+
 } // namespace
 
 #include "ruleswidget.moc"
