@@ -60,6 +60,7 @@ Rules::Rules()
     , closeablerule( UnusedForceRule )
     , strictgeometryrule( UnusedForceRule )
     , shortcutrule( UnusedSetRule )
+    , disableglobalshortcutsrule( UnusedForceRule )
     {
     }
 
@@ -161,6 +162,7 @@ void Rules::readFromCfg( KConfig& cfg )
     READ_FORCE_RULE( closeable, Bool, );
     READ_FORCE_RULE( strictgeometry, Bool, );
     READ_SET_RULE( shortcut, , );
+    READ_FORCE_RULE( disableglobalshortcuts, Bool, );
     }
 
 #undef READ_MATCH_STRING
@@ -249,6 +251,7 @@ void Rules::write( KConfig& cfg ) const
     WRITE_FORCE_RULE( closeable, );
     WRITE_FORCE_RULE( strictgeometry, );
     WRITE_SET_RULE( shortcut, );
+    WRITE_FORCE_RULE( disableglobalshortcuts, );
     }
     
 #undef WRITE_MATCH_STRING
@@ -284,13 +287,14 @@ bool Rules::isEmpty() const
         && moveresizemoderule == UnusedForceRule
         && closeablerule == UnusedForceRule
         && strictgeometryrule == UnusedForceRule
-        && shortcutrule == UnusedSetRule );
+        && shortcutrule == UnusedSetRule
+        && disableglobalshortcutsrule == UnusedForceRule );
     }
 
 Rules::SetRule Rules::readSetRule( KConfig& cfg, const QString& key )
     {
     int v = cfg.readNumEntry( key );
-    if( v >= DontAffect && v <= Remember )
+    if( v >= DontAffect && v <= ForceTemporarily )
         return static_cast< SetRule >( v );
     return UnusedSetRule;
     }
@@ -298,7 +302,7 @@ Rules::SetRule Rules::readSetRule( KConfig& cfg, const QString& key )
 Rules::ForceRule Rules::readForceRule( KConfig& cfg, const QString& key )
     {
     int v = cfg.readNumEntry( key );
-    if( v == DontAffect || v == Force )
+    if( v == DontAffect || v == Force || v == ForceTemporarily )
         return static_cast< ForceRule >( v );
     return UnusedForceRule;
     }
@@ -560,6 +564,13 @@ APPLY_FORCE_RULE( maxsize, MaxSize, QSize )
 APPLY_FORCE_RULE( opacityactive, OpacityActive, int )
 APPLY_FORCE_RULE( opacityinactive, OpacityInactive, int )
 APPLY_FORCE_RULE( ignoreposition, IgnorePosition, bool )
+
+// the cfg. entry needs to stay named the say for backwards compatibility
+bool Rules::applyIgnoreGeometry( bool& ignore ) const
+    {
+    return applyIgnorePosition( ignore );
+    }
+
 APPLY_RULE( desktop, Desktop, int )
 APPLY_FORCE_RULE( type, Type, NET::WindowType )
 
@@ -603,6 +614,8 @@ APPLY_FORCE_RULE( moveresizemode, MoveResizeMode, Options::MoveResizeMode )
 APPLY_FORCE_RULE( closeable, Closeable, bool )
 APPLY_FORCE_RULE( strictgeometry, StrictGeometry, bool )
 APPLY_RULE( shortcut, Shortcut, QString )
+APPLY_FORCE_RULE( disableglobalshortcuts, DisableGlobalShortcuts, bool )
+
 
 #undef APPLY_RULE
 #undef APPLY_FORCE_RULE
@@ -623,6 +636,51 @@ bool Rules::discardTemporary( bool force )
         }
     return false;
     }
+    
+#define DISCARD_USED_SET_RULE( var ) \
+    do { \
+    if( var##rule == ( SetRule ) ApplyNow || ( withdrawn && var##rule == ( SetRule ) ForceTemporarily )) \
+        var##rule = UnusedSetRule; \
+    } while( false )
+#define DISCARD_USED_FORCE_RULE( var ) \
+    do { \
+    if( withdrawn && var##rule == ( ForceRule ) ForceTemporarily ) \
+        var##rule = UnusedForceRule; \
+    } while( false )
+
+void Rules::discardUsed( bool withdrawn )
+    {
+    DISCARD_USED_FORCE_RULE( placement );
+    DISCARD_USED_SET_RULE( position );
+    DISCARD_USED_SET_RULE( size );
+    DISCARD_USED_FORCE_RULE( minsize );
+    DISCARD_USED_FORCE_RULE( maxsize );
+    DISCARD_USED_FORCE_RULE( opacityactive );
+    DISCARD_USED_FORCE_RULE( opacityinactive );
+    DISCARD_USED_FORCE_RULE( ignoreposition );
+    DISCARD_USED_SET_RULE( desktop );
+    DISCARD_USED_FORCE_RULE( type );
+    DISCARD_USED_SET_RULE( maximizevert );
+    DISCARD_USED_SET_RULE( maximizehoriz );
+    DISCARD_USED_SET_RULE( minimize );
+    DISCARD_USED_SET_RULE( shade );
+    DISCARD_USED_SET_RULE( skiptaskbar );
+    DISCARD_USED_SET_RULE( skippager );
+    DISCARD_USED_SET_RULE( above );
+    DISCARD_USED_SET_RULE( below );
+    DISCARD_USED_SET_RULE( fullscreen );
+    DISCARD_USED_SET_RULE( noborder );
+    DISCARD_USED_FORCE_RULE( fsplevel );
+    DISCARD_USED_FORCE_RULE( acceptfocus );
+    DISCARD_USED_FORCE_RULE( moveresizemode );
+    DISCARD_USED_FORCE_RULE( closeable );
+    DISCARD_USED_FORCE_RULE( strictgeometry );
+    DISCARD_USED_SET_RULE( shortcut );
+    DISCARD_USED_FORCE_RULE( disableglobalshortcuts );
+    }
+#undef DISCARD_USED_SET_RULE
+#undef DISCARD_USED_FORCE_RULE
+
 #endif
 
 #ifndef NDEBUG
@@ -708,6 +766,12 @@ CHECK_FORCE_RULE( MaxSize, QSize )
 CHECK_FORCE_RULE( OpacityActive, int )
 CHECK_FORCE_RULE( OpacityInactive, int )
 CHECK_FORCE_RULE( IgnorePosition, bool )
+
+bool WindowRules::checkIgnoreGeometry( bool ignore ) const
+    {
+    return checkIgnorePosition( ignore );
+    }
+
 CHECK_RULE( Desktop, int )
 CHECK_FORCE_RULE( Type, NET::WindowType )
 CHECK_RULE( MaximizeVert, KDecorationDefines::MaximizeMode )
@@ -734,67 +798,67 @@ CHECK_FORCE_RULE( MoveResizeMode, Options::MoveResizeMode )
 CHECK_FORCE_RULE( Closeable, bool )
 CHECK_FORCE_RULE( StrictGeometry, bool )
 CHECK_RULE( Shortcut, QString )
+CHECK_FORCE_RULE( DisableGlobalShortcuts, bool )
 
 #undef CHECK_RULE
 #undef CHECK_FORCE_RULE
 
 // Client
 
-#define FORCE_RULE( rule, type, getf, setf ) \
-    { \
-    type val = client_rules.check##rule( getf()); \
-    if( val != getf()) \
-        setf( val ); \
-    }
-
 void Client::setupWindowRules( bool ignore_temporary )
     {
     client_rules = workspace()->findWindowRules( this, ignore_temporary );
     // check only after getting the rules, because there may be a rule forcing window type
     if( isTopMenu()) // TODO cannot have restrictions
         client_rules = WindowRules();
+    }
+
+// Applies Force, ForceTemporarily and ApplyNow rules
+// Used e.g. after the rules have been modified using the kcm.    
+void Client::applyWindowRules()
+    {
     checkAndSetInitialRuledOpacity();        
-    if( isManaged())
-        { // apply force rules
-        // Placement - does need explicit update, just like some others below
-        // Geometry : setGeometry() doesn't check rules
+    // apply force rules
+    // Placement - does need explicit update, just like some others below
+    // Geometry : setGeometry() doesn't check rules
     QRect orig_geom = QRect( pos(), sizeForClientSize( clientSize())); // handle shading
     QRect geom = client_rules.checkGeometry( orig_geom );
     if( geom != orig_geom )
-            setGeometry( geom );
-        // MinSize, MaxSize handled by Geometry
-        // IgnorePosition
-        setDesktop( desktop());
-        // Type
-        maximize( maximizeMode());
-        // Minimize : functions don't check, and there are two functions
-        if( client_rules.checkMinimize( isMinimized()))
-            minimize();
-        else
-            unminimize();
-        setShade( shadeMode());
-        setSkipTaskbar( skipTaskbar(), true );
-        setSkipPager( skipPager());
-        setKeepAbove( keepAbove());
-        setKeepBelow( keepBelow());
-        setFullScreen( isFullScreen(), true );
-        setUserNoBorder( isUserNoBorder());
-        // FSP
-        // AcceptFocus :
-        if( workspace()->mostRecentlyActivatedClient() == this
-            && !client_rules.checkAcceptFocus( true ))
-            workspace()->activateNextClient( this );
-        // MoveResizeMode
-        // Closeable
+        setGeometry( geom );
+    // MinSize, MaxSize handled by Geometry
+    // IgnorePosition
+    setDesktop( desktop());
+    // Type
+    maximize( maximizeMode());
+    // Minimize : functions don't check, and there are two functions
+    if( client_rules.checkMinimize( isMinimized()))
+        minimize();
+    else
+        unminimize();
+    setShade( shadeMode());
+    setSkipTaskbar( skipTaskbar(), true );
+    setSkipPager( skipPager());
+    setKeepAbove( keepAbove());
+    setKeepBelow( keepBelow());
+    setFullScreen( isFullScreen(), true );
+    setUserNoBorder( isUserNoBorder());
+    // FSP
+    // AcceptFocus :
+    if( workspace()->mostRecentlyActivatedClient() == this
+        && !client_rules.checkAcceptFocus( true ))
+        workspace()->activateNextClient( this );
+    // MoveResizeMode
+    // Closeable
     QSize s = adjustedSize();
-        if( s != size())
-            resizeWithChecks( s );
-        setShortcut( rules()->checkShortcut( shortcut().toString()));
-        }
+    if( s != size())
+        resizeWithChecks( s );
+    // StrictGeometry
+    setShortcut( rules()->checkShortcut( shortcut().toString()));
+    // see also Client::setActive()
+    if( isActive())
+        workspace()->disableGlobalShortcutsForClient( rules()->checkDisableGlobalShortcuts( false ));
     }
 
-#undef FORCE_RULE
-
 void Client::updateWindowRules()
     {
     if( !isManaged()) // not fully setup yet
@@ -875,10 +939,14 @@ WindowRules Workspace::findWindowRules( const Client* c, bool ignore_temporary )
     return WindowRules( ret );
     }
 
-void Workspace::editWindowRules( Client* c )
+void Workspace::editWindowRules( Client* c, bool whole_app )
     {
     writeWindowRules();
-    KApplication::kdeinitExec( "kwin_rules_dialog", QStringList() << "--wid" << QString::number( c->window()));
+    QStringList args;
+    args << "--wid" << QString::number( c->window());
+    if( whole_app )
+        args << "--whole-app";
+    KApplication::kdeinitExec( "kwin_rules_dialog", args );
     }
 
 void Workspace::loadWindowRules()
@@ -954,6 +1022,32 @@ void Workspace::cleanupTemporaryRules()
         QTimer::singleShot( 60000, this, SLOT( cleanupTemporaryRules()));
     }
 
+void Workspace::discardUsedWindowRules( Client* c, bool withdrawn )
+    {
+    bool updated = false;
+    for( QValueList< Rules* >::Iterator it = rules.begin();
+         it != rules.end();
+         )
+        {
+        if( c->rules()->contains( *it ))
+            {
+            updated = true;
+            (*it)->discardUsed( withdrawn );
+            if( (*it)->isEmpty())
+                {
+                c->removeRule( *it );
+                Rules* r = *it;
+                it = rules.remove( it );
+                delete r;
+                continue;
+                }
+            }
+        ++it;
+        }
+    if( updated )
+        rulesUpdated();
+    }
+
 void Workspace::rulesUpdated()
     {
     rulesUpdatedTimer.start( 1000, true );
