@@ -27,6 +27,8 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <kmenubar.h>
 #include <kprocess.h>
 #include <kglobalaccel.h>
+#include <dcopclient.h>
+#include <kipc.h>
 
 #include "plugins.h"
 #include "client.h"
@@ -52,6 +54,10 @@ extern int screen_number;
 
 Workspace *Workspace::_self = 0;
 
+KProcess* kompmgr = 0;
+
+bool allowKompmgrRestart = TRUE;
+
 // Rikkus: This class is too complex. It needs splitting further.
 // It's a nightmare to understand, especially with so few comments :(
 
@@ -64,7 +70,8 @@ Workspace::Workspace( bool restore )
     QObject           (0, "workspace"),
     current_desktop   (0),
     number_of_desktops(0),
-    popup_client      (0),
+    active_popup( NULL ),
+    active_popup_client( NULL ),
     desktop_widget    (0),
     temporaryRulesMessages( "_KDE_NET_WM_TEMPORARY_RULES", NULL, false ),
     active_client     (0),
@@ -73,6 +80,8 @@ Workspace::Workspace( bool restore )
     movingClient(0),
     pending_take_activity ( NULL ),
     delayfocus_client (0),
+    showing_desktop( false ),
+    block_showing_desktop( 0 ),
     was_user_interaction (false),
     session_saving    (false),
     control_grab      (false),
@@ -86,6 +95,12 @@ Workspace::Workspace( bool restore )
     desk_popup        (0),
     desk_popup_index  (0),
     keys              (0),
+    client_keys       ( NULL ),
+    client_keys_dialog ( NULL ),
+    client_keys_client ( NULL ),
+    disable_shortcuts_keys ( NULL ),
+    global_shortcuts_disabled( false ),
+    global_shortcuts_disabled_for_client( false ),
     root              (0),
     workspaceInit     (true),
     startup(0), electric_have_borders(false),
@@ -99,10 +114,10 @@ Workspace::Workspace( bool restore )
     layoutY(2),
     workarea(NULL),
     screenarea(NULL),
+    managing_topmenus( false ),
     topmenu_selection( NULL ),
     topmenu_watcher( NULL ),
     topmenu_height( 0 ),
-    managing_topmenus( false ),
     topmenu_space( NULL ),
     set_active_client_recursion( 0 ),
     block_stacking_updates( 0 ),
@@ -171,6 +186,7 @@ Workspace::Workspace( bool restore )
       1
     );
 
+    client_keys = new KGlobalAccel( this );
     initShortcuts();
     tab_box = new TabBox( this );
     popupinfo = new PopupInfo( );
@@ -180,6 +196,15 @@ Workspace::Workspace( bool restore )
 #if (QT_VERSION-0 >= 0x030200) // XRANDR support
     connect( kapp->desktop(), SIGNAL( resized( int )), SLOT( desktopResized()));
 #endif
+
+    // start kompmgr - i wanted to put this into main.cpp, but that would prevent dcop support, as long as Application was no dcop_object
+    if (options->useTranslucency)
+        {
+        kompmgr = new KProcess;
+        connect(kompmgr, SIGNAL(receivedStderr(KProcess*, char*, int)), SLOT(handleKompmgrOutput(KProcess*, char*, int)));
+        *kompmgr << "kompmgr";
+        startKompmgr();
+        }
     }
 
 
@@ -187,6 +212,10 @@ void Workspace::init()
     {
     checkElectricBorders();
 
+// not used yet
+//     topDock = 0L;
+//     maximizedWindowCounter = 0;
+    
     supportWindow = new QWidget;
     XLowerWindow( qt_xdisplay(), supportWindow->winId()); // see usage in layers.cpp
 
@@ -221,7 +250,7 @@ void Workspace::init()
         NET::WMPid |
         NET::WMMoveResize |
         NET::WMKDESystemTrayWinFor |
-        NET::WMKDEFrameStrut |
+        NET::WMFrameExtents |
         NET::WMPing
         ,
         NET::NormalMask |
@@ -258,6 +287,7 @@ void Workspace::init()
         NET::WM2MoveResizeWindow |
         NET::WM2ExtendedStrut |
         NET::WM2KDETemporaryRules |
+        NET::WM2ShowingDesktop |
         0
         ,
         NET::ActionMove |
@@ -302,6 +332,7 @@ void Workspace::init()
             SLOT(slotReconfigure()));
     connect(kapp, SIGNAL(settingsChanged(int)), this,
             SLOT(slotSettingsChanged(int)));
+    connect(kapp, SIGNAL( kipcMessage( int, int )), this, SLOT( kipcMessage( int, int )));
 
     active_client = NULL;
     rootInfo->setActiveWindow( None );
@@ -366,6 +397,7 @@ void Workspace::init()
         desktop_geometry.height = geom.height();
     // TODO update also after gaining XRANDR support
         rootInfo->setDesktopGeometry( -1, desktop_geometry );
+        setShowingDesktop( false );
 
         } // end updates blocker block
 
@@ -394,6 +426,8 @@ void Workspace::init()
 
 Workspace::~Workspace()
     {
+    if (kompmgr)
+        delete kompmgr;
     blockStackingUpdates( true );
 // TODO    grabXServer();
     // use stacking_order, so that kwin --replace keeps stacking order
@@ -425,6 +459,7 @@ Workspace::~Workspace()
     delete topmenu_watcher;
     delete topmenu_selection;
     delete topmenu_space;
+    delete client_keys_dialog;
     while( !rules.isEmpty())
         {
         delete rules.front();
@@ -450,6 +485,21 @@ Client* Workspace::createClient( Window w, bool is_mapped )
 
 void Workspace::addClient( Client* c, allowed_t )
     {
+    // waited with trans settings until window figured out if active or not ;)
+//     qWarning("%s", (const char*)(c->resourceClass()));
+    c->setBMP(c->resourceName() == "beep-media-player" || c->decorationId() == None);
+    // first check if the window has it's own opinion of it's translucency ;)
+    c->getWindowOpacity();
+    if (c->isDock())
+        {
+//         if (c->x() == 0 && c->y() == 0 && c->width() > c->height()) topDock = c;
+        if (!c->hasCustomOpacity()) // this xould be done slightly more efficient, but we want to support the topDock in future
+            {
+            c->setShadowSize(options->dockShadowSize);
+            c->setOpacity(options->translucentDocks, options->dockOpacity);
+            }
+        }
+//------------------------------------------------        
     Group* grp = findGroup( c->window());
     if( grp != NULL )
         grp->gotLeader( c );
@@ -481,6 +531,7 @@ void Workspace::addClient( Client* c, allowed_t )
         if( activeClient() == NULL && should_get_focus.count() == 0 )
             activateClient( findDesktop( true, currentDesktop()));
         }
+    c->checkActiveModal();
     checkTransients( c->window()); // SELI does this really belong here?
     updateStackingOrder( true ); // propagate new client
     if( c->isUtility() || c->isMenu() || c->isToolbar())
@@ -492,10 +543,13 @@ void Workspace::addClient( Client* c, allowed_t )
  */
 void Workspace::removeClient( Client* c, allowed_t )
     {
-    if (c == active_client && popup)
-        popup->close();
-    if( c == popup_client )
-        popup_client = 0;
+    if (c == active_popup_client)
+        closeActivePopup();
+
+    if( client_keys_client == c )
+        setupWindowShortcutDone( false );
+    if( !c->shortcut().isNull())
+        c->setShortcut( QString::null ); // remove from client_keys
 
     if( c->isDialog())
         Notify::raise( Notify::TransDelete );
@@ -636,6 +690,14 @@ void Workspace::updateCurrentTopMenu()
 void Workspace::updateToolWindows( bool also_hide )
     {
     // TODO what if Client's transiency/group changes? should this be called too? (I'm paranoid, am I not?)
+    if( !options->hideUtilityWindowsForInactive )
+        {
+        for( ClientList::ConstIterator it = clients.begin();
+             it != clients.end();
+             ++it )
+            (*it)->hideClient( false );
+        return;
+        }
     const Group* group = NULL;
     const Client* client = active_client;
 // Go up in transiency hiearchy, if the top is found, only tool transients for the top mainwindow
@@ -758,7 +820,6 @@ void Workspace::slotSettingsChanged(int category)
   Reread settings
  */
 KWIN_PROCEDURE( CheckBorderSizesProcedure, cl->checkBorderSizes() );
-KWIN_PROCEDURE( ResetupRulesProcedure, cl->setupWindowRules( true ) );
 
 void Workspace::slotReconfigure()
     {
@@ -769,8 +830,10 @@ void Workspace::slotReconfigure()
     unsigned long changed = options->updateSettings();
     tab_box->reconfigure();
     popupinfo->reconfigure();
+    initPositioning->reinitCascading( 0 );
     readShortcuts();
     forEachClient( CheckIgnoreFocusStealingProcedure());
+    updateToolWindows( true );
 
     if( mgr->reset( changed ))
         { // decorations need to be recreated
@@ -815,7 +878,22 @@ void Workspace::slotReconfigure()
         }
 
     loadWindowRules();
-    forEachClient( ResetupRulesProcedure());
+    for( ClientList::Iterator it = clients.begin();
+         it != clients.end();
+         ++it )
+        {
+        (*it)->setupWindowRules( true );
+        (*it)->applyWindowRules();
+        discardUsedWindowRules( *it, false );
+        }
+
+    if (options->resetKompmgr) // need restart
+        {
+        bool tmp = options->useTranslucency;
+        stopKompmgr();
+        if (tmp)
+            QTimer::singleShot( 200, this, SLOT(startKompmgr()) ); // wait some time to ensure system's ready for restart
+        }
     }
 
 void Workspace::loadDesktopSettings()
@@ -886,7 +964,7 @@ QStringList Workspace::configModules(bool controlCenter)
     if (controlCenter)
         args << "kde-kwinoptions.desktop";
     else if (kapp->authorizeControlModule("kde-kwinoptions.desktop"))
-        args  << "kwinactions" << "kwinfocus" <<  "kwinmoving" << "kwinadvanced" << "kwinrules";
+        args  << "kwinactions" << "kwinfocus" <<  "kwinmoving" << "kwinadvanced" << "kwinrules" << "kwintranslucency";
     return args;
     }
 
@@ -1012,14 +1090,15 @@ bool Workspace::setCurrentDesktop( int new_desktop )
     if (new_desktop < 1 || new_desktop > number_of_desktops )
         return false;
 
-    if( popup )
-        popup->close();
+    closeActivePopup();
     ++block_focus;
 // TODO    Q_ASSERT( block_stacking_updates == 0 ); // make sure stacking_order is up to date
     StackingUpdatesBlocker blocker( this );
 
+    int old_desktop = current_desktop;
     if (new_desktop != current_desktop) 
         {
+        ++block_showing_desktop;
         /*
           optimized Desktop switching: unmapping done from back to front
           mapping done from front to back => less exposure events
@@ -1028,15 +1107,14 @@ bool Workspace::setCurrentDesktop( int new_desktop )
 
         ObscuringWindows obs_wins;
 
-        int old_desktop = current_desktop;
-        current_desktop = new_desktop; // change the desktop (so that Client::virtualDesktopChange() works)
+        current_desktop = new_desktop; // change the desktop (so that Client::updateVisibility() works)
 
         for ( ClientList::ConstIterator it = stacking_order.begin(); it != stacking_order.end(); ++it)
             if ( !(*it)->isOnDesktop( new_desktop ) && (*it) != movingClient )
                 {
                 if( (*it)->isShown( true ) && (*it)->isOnDesktop( old_desktop ))
                     obs_wins.create( *it );
-                (*it)->virtualDesktopChange();
+                (*it)->updateVisibility();
                 }
 
         rootInfo->setCurrentDesktop( current_desktop ); // now propagate the change, after hiding, before showing
@@ -1046,7 +1124,11 @@ bool Workspace::setCurrentDesktop( int new_desktop )
 
         for ( ClientList::ConstIterator it = stacking_order.fromLast(); it != stacking_order.end(); --it)
             if ( (*it)->isOnDesktop( new_desktop ) )
-                (*it)->virtualDesktopChange();
+                (*it)->updateVisibility();
+
+        --block_showing_desktop;
+        if( showingDesktop()) // do this only after desktop change to avoid flicker
+            resetShowingDesktop( false );
         }
 
     // restore the focus on this desktop
@@ -1126,23 +1208,151 @@ bool Workspace::setCurrentDesktop( int new_desktop )
 //    for( uint i = 0; i < desktop_focus_chain.size(); i++ )
 //        s += QString::number(desktop_focus_chain[i]) + ", ";
 //    kdDebug(1212) << s << "}\n";
+
+    if( old_desktop != 0 )  // not for the very first time
+        popupinfo->showInfo( desktopName(currentDesktop()) );
     return true;
     }
 
+// called only from DCOP
 void Workspace::nextDesktop()
     {
     int desktop = currentDesktop() + 1;
     setCurrentDesktop(desktop > numberOfDesktops() ? 1 : desktop);
-    popupinfo->showInfo( desktopName(currentDesktop()) );
     }
 
+// called only from DCOP
 void Workspace::previousDesktop()
     {
     int desktop = currentDesktop() - 1;
     setCurrentDesktop(desktop > 0 ? desktop : numberOfDesktops());
-    popupinfo->showInfo( desktopName(currentDesktop()) );
     }
 
+int Workspace::desktopToRight( int desktop ) const
+    {
+    int x,y;
+    calcDesktopLayout(x,y);
+    int dt = desktop-1;
+    if (layoutOrientation == Qt::Vertical)
+        {
+        dt += y;
+        if ( dt >= numberOfDesktops() ) 
+            {
+            if ( options->rollOverDesktops )
+              dt -= numberOfDesktops();
+            else
+              return desktop;
+            }
+        }
+    else
+        {
+        int d = (dt % x) + 1;
+        if ( d >= x ) 
+            {
+            if ( options->rollOverDesktops )
+              d -= x;
+            else
+              return desktop;
+            }
+        dt = dt - (dt % x) + d;
+        }
+    return dt+1;
+    }
+
+int Workspace::desktopToLeft( int desktop ) const
+    {
+    int x,y;
+    calcDesktopLayout(x,y);
+    int dt = desktop-1;
+    if (layoutOrientation == Qt::Vertical)
+        {
+        dt -= y;
+        if ( dt < 0 ) 
+            {
+            if ( options->rollOverDesktops )
+              dt += numberOfDesktops();
+            else
+              return desktop;
+            }
+        }
+    else
+        {
+        int d = (dt % x) - 1;
+        if ( d < 0 ) 
+            {
+            if ( options->rollOverDesktops )
+              d += x;
+            else
+              return desktop;
+            }
+        dt = dt - (dt % x) + d;
+        }
+    return dt+1;
+    }
+
+int Workspace::desktopUp( int desktop ) const
+    {
+    int x,y;
+    calcDesktopLayout(x,y);
+    int dt = desktop-1;
+    if (layoutOrientation == Qt::Horizontal)
+        {
+        dt -= x;
+        if ( dt < 0 ) 
+            {
+            if ( options->rollOverDesktops )
+              dt += numberOfDesktops();
+            else
+              return desktop;
+            }
+        }
+    else
+        {
+        int d = (dt % y) - 1;
+        if ( d < 0 ) 
+            {
+            if ( options->rollOverDesktops )
+              d += y;
+            else
+              return desktop;
+            }
+        dt = dt - (dt % y) + d;
+        }
+    return dt+1;
+    }
+
+int Workspace::desktopDown( int desktop ) const
+    {
+    int x,y;
+    calcDesktopLayout(x,y);
+    int dt = desktop-1;
+    if (layoutOrientation == Qt::Horizontal)
+        {
+        dt += x;
+        if ( dt >= numberOfDesktops() ) 
+            {
+            if ( options->rollOverDesktops )
+              dt -= numberOfDesktops();
+            else
+              return desktop;
+            }
+        }
+    else
+        {
+        int d = (dt % y) + 1;
+        if ( d >= y ) 
+            {
+            if ( options->rollOverDesktops )
+              d -= y;
+            else
+              return desktop;
+            }
+        dt = dt - (dt % y) + d;
+        }
+    return dt+1;
+    }
+
+
 /*!
   Sets the number of virtual desktops to \a n
  */
@@ -1241,7 +1451,7 @@ void Workspace::setDesktopLayout(int o, int x, int y)
     layoutY = y;
     }
 
-void Workspace::calcDesktopLayout(int &x, int &y)
+void Workspace::calcDesktopLayout(int &x, int &y) const
     {
     x = layoutX;
     y = layoutY;
@@ -1782,6 +1992,16 @@ void Workspace::createBorderWindows()
                                   CopyFromParent,
                                   valuemask, &attributes);
     XMapWindow(qt_xdisplay(),  electric_right_border);
+    // Set XdndAware on the windows, so that DND enter events are received (#86998)
+    Atom version = 4; // XDND version
+    XChangeProperty( qt_xdisplay(), electric_top_border, atoms->xdnd_aware, XA_ATOM,
+        32, PropModeReplace, ( unsigned char* )&version, 1 );
+    XChangeProperty( qt_xdisplay(), electric_bottom_border, atoms->xdnd_aware, XA_ATOM,
+        32, PropModeReplace, ( unsigned char* )&version, 1 );
+    XChangeProperty( qt_xdisplay(), electric_left_border, atoms->xdnd_aware, XA_ATOM,
+        32, PropModeReplace, ( unsigned char* )&version, 1 );
+    XChangeProperty( qt_xdisplay(), electric_right_border, atoms->xdnd_aware, XA_ATOM,
+        32, PropModeReplace, ( unsigned char* )&version, 1 );
     }
 
 
@@ -1825,7 +2045,7 @@ void Workspace::clientMoved(const QPoint &pos, Time now)
 
     Time treshold_set = options->electricBorderDelay(); // set timeout
     Time treshold_reset = 250; // reset timeout
-    int distance_reset = 10; // Mouse should not move more than this many pixels
+    int distance_reset = 30; // Mouse should not move more than this many pixels
 
     int border = 0;
     if (pos.x() == electricLeft)
@@ -1914,12 +2134,36 @@ void Workspace::clientMoved(const QPoint &pos, Time now)
 
 // this function is called when the user entered an electric border
 // with the mouse. It may switch to another virtual desktop
-void Workspace::electricBorder(XEvent *e)
+bool Workspace::electricBorder(XEvent *e)
     {
-    Time now = e->xcrossing.time;
-    QPoint p(e->xcrossing.x_root, e->xcrossing.y_root);
-
-    clientMoved(p, now);
+    if( !electric_have_borders )
+        return false;
+    if( e->type == EnterNotify )
+        {
+        if( e->xcrossing.window == electric_top_border ||
+            e->xcrossing.window == electric_left_border ||
+            e->xcrossing.window == electric_bottom_border ||
+            e->xcrossing.window == electric_right_border)
+            // the user entered an electric border
+            {
+            clientMoved( QPoint( e->xcrossing.x_root, e->xcrossing.y_root ), e->xcrossing.time );
+            return true;
+            }
+        }
+    if( e->type == ClientMessage )
+        {
+        if( e->xclient.message_type == atoms->xdnd_position
+            && ( e->xclient.window == electric_top_border
+                 || e->xclient.window == electric_bottom_border
+                 || e->xclient.window == electric_left_border
+                 || e->xclient.window == electric_right_border ))
+            {
+            updateXTime();
+            clientMoved( QPoint( e->xclient.data.l[2]>>16, e->xclient.data.l[2]&0xffff), qt_x_time );
+            return true;
+            }
+        }
+    return false;
     }
 
 // electric borders (input only windows) have to be always on the
@@ -2100,6 +2344,287 @@ void Workspace::helperDialog( const QString& message, const Client* c )
     proc.start( KProcess::DontCare );
     }
 
+
+// kompmgr stuff
+    
+void Workspace::startKompmgr()
+{
+    if (!kompmgr || kompmgr->isRunning())
+        return;
+    if (!kompmgr->start(KProcess::OwnGroup, KProcess::Stderr))
+        {
+        options->useTranslucency = FALSE;
+        KProcess proc;
+        proc << "kdialog" << "--error"
+            << i18n("The Composite Manager could not be started.\\nMake sure you have \"kompmgr\" in a $PATH directory.")
+            << "--title" << "Composite Manager Failure";
+        proc.start(KProcess::DontCare);
+        }
+    else
+        {
+        connect(kompmgr, SIGNAL(processExited(KProcess*)), SLOT(restartKompmgr()));
+        options->useTranslucency = TRUE;
+        allowKompmgrRestart = FALSE;
+        QTimer::singleShot( 60000, this, SLOT(unblockKompmgrRestart()) );
+        QByteArray ba;
+        QDataStream arg(ba, IO_WriteOnly);
+        arg << "";
+        kapp->dcopClient()->emitDCOPSignal("default", "kompmgrStarted()", ba);
+        }
+        if (popup){ delete popup; popup = 0L; } // to add/remove opacity slider
+}
+
+void Workspace::stopKompmgr()
+{
+    if (!kompmgr  || !kompmgr->isRunning())
+        return;
+    kompmgr->disconnect(this, SLOT(restartKompmgr()));
+    options->useTranslucency = FALSE;
+    if (popup){ delete popup; popup = 0L; } // to add/remove opacity slider
+    kompmgr->kill();
+    QByteArray ba;
+    QDataStream arg(ba, IO_WriteOnly);
+    arg << "";
+    kapp->dcopClient()->emitDCOPSignal("default", "kompmgrStopped()", ba);
+}
+
+bool Workspace::kompmgrIsRunning()
+{
+   return kompmgr && kompmgr->isRunning();
+}
+
+void Workspace::unblockKompmgrRestart()
+{
+    allowKompmgrRestart = TRUE;
+}
+
+void Workspace::restartKompmgr()
+// this is for inernal purpose (crashhandling) only, usually you want to use workspace->stopKompmgr(); QTimer::singleShot(200, workspace, SLOT(startKompmgr()));
+{
+    if (!allowKompmgrRestart) // uh-ohh
+        {
+        options->useTranslucency = FALSE;
+        KProcess proc;
+        proc << "kdialog" << "--error"
+            << i18n( "The Composite Manager crashed twice within a minute and is therefore disabled for this session.")
+            << "--title" << i18n("Composite Manager Failure");
+        proc.start(KProcess::DontCare);
+        return;
+        }
+    if (!kompmgr)
+        return;
+// this should be useless, i keep it for maybe future need
+//     if (!kcompmgr)
+//         {
+//         kompmgr = new KProcess;
+//         kompmgr->clearArguments();
+//         *kompmgr << "kompmgr";
+//         }
+// -------------------
+    if (!kompmgr->start(KProcess::NotifyOnExit, KProcess::Stderr))
+        {
+        options->useTranslucency = FALSE;
+        KProcess proc;
+        proc << "kdialog" << "--error"
+            << i18n("The Composite Manager could not be started.\\nMake sure you have \"kompmgr\" in a $PATH directory.")
+            << "--title" << i18n("Composite Manager Failure");
+        proc.start(KProcess::DontCare);
+        }
+    else
+        {
+        allowKompmgrRestart = FALSE;
+        QTimer::singleShot( 60000, this, SLOT(unblockKompmgrRestart()) );
+        }
+}
+
+void Workspace::handleKompmgrOutput( KProcess* , char *buffer, int buflen)
+{
+    QString message;
+    QString output = QString::fromLocal8Bit( buffer, buflen );
+    if (output.contains("Started",false))
+        ; // don't do anything, just pass to the connection release
+    else if (output.contains("Can't open display",false))
+        message = i18n("<qt><b>kompmgr failed to open the display</b><br>There is probably an invalid display entry in your ~/.xcompmgrrc.</qt>");
+    else if (output.contains("No render extension",false))
+        message = i18n("<qt><b>kompmgr cannot find the Xrender extension</b><br>You are using either an outdated or a crippled version of XOrg.<br>Get XOrg &ge; 6.8 from www.freedesktop.org.<br></qt>");
+    else if (output.contains("No composite extension",false))
+        message = i18n("<qt><b>Composite extension not found</b><br>You <i>must</i> use XOrg &ge; 6.8 for translucency and shadows to work.<br>Additionally, you need to add a new section to your X config file:<br>"
+        "<i>Section \"Extensions\"<br>"
+        "Option \"Composite\" \"Enable\"<br>"
+        "EndSection</i></qt>");
+    else if (output.contains("No damage extension",false))
+        message = i18n("<qt><b>Damage extension not found</b><br>You <i>must</i> use XOrg &ge; 6.8 for translucency and shadows to work.</qt>");
+    else if (output.contains("No XFixes extension",false))
+        message = i18n("<qt><b>XFixes extension not found</b><br>You <i>must</i> use XOrg &ge; 6.8 for translucency and shadows to work.</qt>");
+    else return; //skip others
+    // kompmgr startup failed or succeeded, release connection
+    kompmgr->closeStderr();
+    disconnect(kompmgr, SIGNAL(receivedStderr(KProcess*, char*, int)), this, SLOT(handleKompmgrOutput(KProcess*, char*, int)));
+    if( !message.isEmpty())
+        {
+        KProcess proc;
+        proc << "kdialog" << "--error"
+            << message
+            << "--title" << i18n("Composite Manager Failure");
+        proc.start(KProcess::DontCare);
+        }
+}
+    
+        
+void Workspace::setOpacity(unsigned long winId, unsigned int opacityPercent)
+{
+    if (opacityPercent > 100) opacityPercent = 100;
+    for( ClientList::ConstIterator it = stackingOrder().begin(); it != stackingOrder().end(); it++ )
+        if (winId == (*it)->window())
+            {
+            (*it)->setOpacity(opacityPercent < 100, (unsigned int)((opacityPercent/100.0)*0xFFFFFFFF));
+            return;
+            }
+}
+
+void Workspace::setShadowSize(unsigned long winId, unsigned int shadowSizePercent)
+{
+    //this is open to the user by dcop - to avoid stupid trials, we limit the max shadow size to 400%
+    if (shadowSizePercent > 400) shadowSizePercent = 400;
+    for( ClientList::ConstIterator it = stackingOrder().begin(); it != stackingOrder().end(); it++ )
+        if (winId == (*it)->window())
+            {
+            (*it)->setShadowSize(shadowSizePercent);
+            return;
+            }
+}
+
+void Workspace::setUnshadowed(unsigned long winId)
+{
+    for( ClientList::ConstIterator it = stackingOrder().begin(); it != stackingOrder().end(); it++ )
+        if (winId == (*it)->window())
+            {
+            (*it)->setShadowSize(0);
+            return;
+            }
+}
+
+void Workspace::setShowingDesktop( bool showing )
+    {
+    rootInfo->setShowingDesktop( showing );
+    showing_desktop = showing;
+    ++block_showing_desktop;
+    if( showing_desktop )
+        {
+        showing_desktop_clients.clear();
+        ++block_focus;
+        ClientList cls = stackingOrder();
+        // find them first, then minimize, otherwise transients may get minimized with the window
+        // they're transient for
+        for( ClientList::ConstIterator it = cls.begin();
+             it != cls.end();
+             ++it )
+            {
+            if( (*it)->isOnCurrentDesktop() && (*it)->isShown( true ) && !(*it)->isSpecialWindow())
+                showing_desktop_clients.prepend( *it ); // topmost first to reduce flicker
+            }
+        for( ClientList::ConstIterator it = showing_desktop_clients.begin();
+             it != showing_desktop_clients.end();
+             ++it )
+            (*it)->minimize(true);
+        --block_focus;
+        if( Client* desk = findDesktop( true, currentDesktop()))
+            requestFocus( desk );
+        }
+    else
+        {
+        for( ClientList::ConstIterator it = showing_desktop_clients.begin();
+             it != showing_desktop_clients.end();
+             ++it )
+            (*it)->unminimize(true);
+        if( showing_desktop_clients.count() > 0 )
+            requestFocus( showing_desktop_clients.first());
+        showing_desktop_clients.clear();
+        }
+    --block_showing_desktop;
+    }
+
+// Following Kicker's behavior:
+// Changing a virtual desktop resets the state and shows the windows again.
+// Unminimizing a window resets the state but keeps the windows hidden (except
+// the one that was unminimized).
+// A new window resets the state and shows the windows again, with the new window
+// being active.
+void Workspace::resetShowingDesktop( bool keep_hidden )
+    {
+    if( block_showing_desktop > 0 )
+        return;
+    rootInfo->setShowingDesktop( false );
+    showing_desktop = false;
+    ++block_showing_desktop;
+    if( !keep_hidden )
+        {
+        for( ClientList::ConstIterator it = showing_desktop_clients.begin();
+             it != showing_desktop_clients.end();
+             ++it )
+            (*it)->unminimize(true);
+        }
+    showing_desktop_clients.clear();
+    --block_showing_desktop;
+    }
+
+// Activating/deactivating this feature works like this:
+// When nothing is active, and the shortcut is pressed, global shortcuts are disabled
+//   (using global_shortcuts_disabled)
+// When a window that has disabling forced is activated, global shortcuts are disabled.
+//   (using global_shortcuts_disabled_for_client)
+// When a shortcut is pressed and global shortcuts are disabled (either by a shortcut
+// or for a client), they are enabled again.
+void Workspace::slotDisableGlobalShortcuts()
+    {
+    if( global_shortcuts_disabled || global_shortcuts_disabled_for_client )
+        disableGlobalShortcuts( false );
+    else
+        disableGlobalShortcuts( true );
+    }
+
+static bool pending_dfc = false;
+
+void Workspace::disableGlobalShortcutsForClient( bool disable )
+    {
+    if( global_shortcuts_disabled_for_client == disable )
+        return;
+    if( !global_shortcuts_disabled )
+        {
+        if( disable )
+            pending_dfc = true;
+        KIPC::sendMessageAll( KIPC::BlockShortcuts, disable );
+        // kwin will get the kipc message too
+        }
+    }
+
+void Workspace::disableGlobalShortcuts( bool disable )
+    {
+    KIPC::sendMessageAll( KIPC::BlockShortcuts, disable );
+    // kwin will get the kipc message too
+    }
+
+void Workspace::kipcMessage( int id, int data )
+    {
+    if( id != KIPC::BlockShortcuts )
+        return;
+    if( pending_dfc && data )
+        {
+        global_shortcuts_disabled_for_client = true;
+        pending_dfc = false;
+        }
+    else
+        {
+        global_shortcuts_disabled = data;
+        global_shortcuts_disabled_for_client = false;
+        }
+    // update also Alt+LMB actions etc.
+    for( ClientList::ConstIterator it = clients.begin();
+         it != clients.end();
+         ++it )
+        (*it)->updateMouseGrab();
+    }
+
 } // namespace
 
 #include "workspace.moc"
