@@ -23,7 +23,9 @@
 #include <kstandarddirs.h>
 #include <kdebug.h>
 #include <kapplication.h>
+#include <kiconloader.h>
 
+#include "extension.h"
 #include "rules.h"
 #include "kcmlayout.h"
 #include "kcmlayout.moc"
@@ -78,7 +80,8 @@ OptionListItem * OptionListItem::findChildItem( const QString& optionName )
 }
 
 LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
-  : KCModule(parent, name), m_rules(0)
+  : KCModule(parent, name), 
+    m_rules(NULL)
 {
   QVBoxLayout *main = new QVBoxLayout(this, 0, KDialog::spacingHint());
 
@@ -87,11 +90,12 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
 
   connect( widget->chkEnable, SIGNAL( toggled( bool )), this, SLOT(changed()));
   connect( widget->chkShowSingle, SIGNAL( toggled( bool )), this, SLOT(changed()));
+  connect( widget->chkShowFlag, SIGNAL( toggled( bool )), this, SLOT(changed()));
   connect( widget->comboModel, SIGNAL(activated(int)), this, SLOT(changed()));
 
-  connect( widget->btnAdd, SIGNAL(clicked()), this, SLOT(changed()));
+  connect( widget->listLayoutsSrc, SIGNAL(doubleClicked(QListViewItem*,const QPoint&, int)),
+									this, SLOT(add()));
   connect( widget->btnAdd, SIGNAL(clicked()), this, SLOT(add()));
-  connect( widget->btnRemove, SIGNAL(clicked()), this, SLOT(changed()));
   connect( widget->btnRemove, SIGNAL(clicked()), this, SLOT(remove()));
 
   connect( widget->comboVariant, SIGNAL(activated(int)), this, SLOT(changed()));
@@ -102,8 +106,10 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
   connect( widget->chkLatin, SIGNAL(clicked()), this, SLOT(changed()));
   connect( widget->chkLatin, SIGNAL(clicked()), this, SLOT(latinChanged()));
 
+  widget->btnUp->setIconSet(SmallIconSet("1uparrow"));
   connect( widget->btnUp, SIGNAL(clicked()), this, SLOT(changed()));
   connect( widget->btnUp, SIGNAL(clicked()), this, SLOT(moveUp()));
+  widget->btnDown->setIconSet(SmallIconSet("1downarrow"));
   connect( widget->btnDown, SIGNAL(clicked()), this, SLOT(changed()));
   connect( widget->btnDown, SIGNAL(clicked()), this, SLOT(moveDown()));
 
@@ -121,7 +127,7 @@ LayoutConfig::LayoutConfig(QWidget *parent, const char *name)
 #endif
 
   //Read rules - we _must_ read _before_ creating xkb-options comboboxes
-  ruleChanged("xfree86");
+  ruleChanged();
 
   makeOptionsTab();
 
@@ -145,13 +151,17 @@ void LayoutConfig::add()
     QListViewItem* sel = widget->listLayoutsSrc->selectedItem();
     if( sel == 0 )
 	return;
-
+    
+    widget->listLayoutsSrc->takeItem(sel);
     widget->listLayoutsDst->insertItem(sel);
-    sel->moveItem(widget->listLayoutsDst->lastItem());
-    widget->listLayoutsSrc->setSelected(sel, true);
-    layoutSelChanged(sel);
+    if( widget->listLayoutsDst->childCount() > 1 )
+	sel->moveItem(widget->listLayoutsDst->lastItem());
+// disabling temporary: does not work reliable in Qt :(
+//    widget->listLayoutsSrc->setSelected(sel, true);
+//    layoutSelChanged(sel);
 
     updateStickyLimit();
+    changed();
 }
 
 void LayoutConfig::remove()
@@ -168,12 +178,14 @@ void LayoutConfig::remove()
     if( sel->itemAbove() )
 	newSel = sel->itemAbove();
 
+    widget->listLayoutsDst->takeItem(sel);
     widget->listLayoutsSrc->insertItem(sel);
     if( newSel )
 	widget->listLayoutsSrc->setSelected(newSel, true);
     layoutSelChanged(newSel);
 
     updateStickyLimit();
+    changed();
 }
 
 void LayoutConfig::moveUp()
@@ -231,7 +243,6 @@ void LayoutConfig::latinChanged()
 
 void LayoutConfig::layoutSelChanged(QListViewItem *sel)
 {
-
     widget->comboVariant->clear();
     widget->comboVariant->setEnabled( sel != 0 );
     widget->chkLatin->setChecked( false );
@@ -244,12 +255,14 @@ void LayoutConfig::layoutSelChanged(QListViewItem *sel)
 
     QString kbdLayout = lookupLocalized( m_rules->layouts(), sel->text(1) );
 
-    if (  ! m_rules->isSingleGroup(kbdLayout) || kbdLayout == "us" ) {
-        widget->chkLatin->setEnabled(  false );
+// need better algorithm here for determining if needs us group
+    if (  ! m_rules->isSingleGroup(kbdLayout) 
+	    || kbdLayout.startsWith("us") || kbdLayout.startsWith("en") ) {
+        widget->chkLatin->setEnabled( false );
     }
     else {
         char* inc = m_includes[ kbdLayout ];
-        if ( inc && strcmp(inc, "us")==0 ) {
+        if ( inc && (strncmp(inc, "us", 2)==0 || strncmp(inc, "en", 2)==0) ) {
             widget->chkLatin->setChecked(true);
         }
         else {
@@ -259,7 +272,6 @@ void LayoutConfig::layoutSelChanged(QListViewItem *sel)
 
     QStringList vars = m_rules->getVariants(kbdLayout);
 
-
     if( vars.count() == 0 ) {	// cowardly running away
         updateLayoutCommand();
         return;
@@ -291,6 +303,8 @@ QWidget* LayoutConfig::makeOptionsTab()
   connect(listView, SIGNAL(clicked(QListViewItem *)), SLOT(changed()));
   connect(listView, SIGNAL(clicked(QListViewItem *)), SLOT(updateOptionsCommand()));
 
+  connect(widget->chkEnableOptions, SIGNAL(toggled(bool)), SLOT(changed()));
+
   connect(widget->checkResetOld, SIGNAL(toggled(bool)), SLOT(changed()));
   connect(widget->checkResetOld, SIGNAL(toggled(bool)), SLOT(updateOptionsCommand()));
 
@@ -404,10 +418,10 @@ void LayoutConfig::load()
   bool use = config->readBoolEntry( "Use", false );
 
   // find out which rule applies
-  QString rule = "xfree86"; //config->readEntry("Rule", "xfree86");
+  //QString rule = "xfree86"; //config->readEntry("Rule", "xfree86");
 
   // update other files
-  ruleChanged(rule);
+  ruleChanged();
 
   // find out about the model
   QString model = config->readEntry("Model", "pc104");
@@ -431,6 +445,7 @@ void LayoutConfig::load()
     for ( ; src_it.current(); ++src_it ) {
 	QListViewItem* srcItem = src_it.current();
 	if ( *it == lookupLocalized(m_rules->layouts(), src_it.current()->text(1)) ) {
+    	    widget->listLayoutsSrc->takeItem(srcItem);
     	    widget->listLayoutsDst->insertItem(srcItem);
 	    srcItem->moveItem(widget->listLayoutsDst->lastItem());
 	    break;
@@ -449,6 +464,9 @@ void LayoutConfig::load()
   bool showSingle = config->readBoolEntry( "ShowSingle", false );
   widget->chkShowSingle->setChecked(showSingle);
 
+  bool showFlag = config->readBoolEntry( "ShowFlag", true );
+  widget->chkShowFlag->setChecked(showFlag);
+
   bool enableXkbOptions = config->readBoolEntry( "EnableXkbOptions", true );
   widget->chkEnableOptions->setChecked( enableXkbOptions );
   bool resetOld = config->readBoolEntry("ResetOldOptions", false);
@@ -498,12 +516,12 @@ void LayoutConfig::load()
   emit KCModule::changed( false );
 }
 
-void LayoutConfig::ruleChanged(const QString &rule)
+void LayoutConfig::ruleChanged()
 {
-  if( rule == m_rule )
-    return;
+//  if( rule == m_rule )
+//    return;
 
-  m_rule = rule;
+//  m_rule = rule;
 
   QString model; //, layout;
   if (m_rules)
@@ -513,7 +531,7 @@ void LayoutConfig::ruleChanged(const QString &rule)
     }
 
   delete m_rules;
-  m_rules = new KeyRules(rule);
+  m_rules = new KeyRules();
 
   QStringList tmp;
   widget->comboModel->clear();
@@ -529,12 +547,13 @@ void LayoutConfig::ruleChanged(const QString &rule)
 
   // fill in the additional layouts  -- moved from load() by A. Rysin
   widget->listLayoutsSrc->clear();
+  widget->listLayoutsDst->clear();
   QDictIterator<char> it2(m_rules->layouts());
   while (it2.current())
     {
       QCheckListItem *item = new QCheckListItem(widget->listLayoutsSrc, "");
       QString addLayout = it2.currentKey();
-      item->setPixmap(0, findPixmap(addLayout));
+      item->setPixmap(0, LayoutIcon::findPixmap(addLayout, true));
       item->setText(1, i18n(it2.current()) );
       item->setText(2, "(" + addLayout + ")" );
       ++it2;
@@ -649,6 +668,7 @@ void LayoutConfig::save()
 
   config->writeEntry("Use", widget->chkEnable->isChecked());
   config->writeEntry("ShowSingle", widget->chkShowSingle->isChecked());
+  config->writeEntry("ShowFlag", widget->chkShowFlag->isChecked());
 
   int modeId = widget->grpSwitching->id(widget->grpSwitching->selected());
   if( modeId < 1 || modeId > 2 )
@@ -671,7 +691,7 @@ void LayoutConfig::save()
 void LayoutConfig::defaults()
 {
   widget->chkEnable->setChecked(false);
-  ruleChanged("xfree86");
+  ruleChanged();
 
   widget->comboModel->setCurrentText("pc104");
   //layoutCombo->setCurrentText("us");
@@ -696,17 +716,17 @@ void LayoutConfig::defaults()
 
 extern "C"
 {
-  KCModule *create_keyboard_layout(QWidget *parent, const char *)
+  KDE_EXPORT KCModule *create_keyboard_layout(QWidget *parent, const char *)
   {
     return new LayoutConfig(parent, "kcmlayout");
   }
 
-  KCModule *create_keyboard(QWidget *parent, const char *)
+  KDE_EXPORT KCModule *create_keyboard(QWidget *parent, const char *)
   {
     return new KeyboardConfig(parent, "kcmlayout");
   }
 
-  void init_keyboard()
+  KDE_EXPORT void init_keyboard()
   {
     KConfig *config = new KConfig("kcminputrc", true); // Read-only, no globals
     config->setGroup("Keyboard");
@@ -725,7 +745,7 @@ extern "C"
 
     if( key ) {
         int delay_ = config->readNumEntry("RepeatDelay", 250);
-        int rate_ = config->readNumEntry("RepeatRate", 30);
+        double rate_ = config->readDoubleNumEntry("RepeatRate", 30);
         set_repeatrate(delay_, rate_);
     }
 
@@ -738,6 +758,18 @@ extern "C"
 
     config = new KConfig("kxkbrc", true, false);
     config->setGroup("Layout");
+
+// Even if the layouts have been disabled we still want to set Xkb options
+// user can always switch them off now in the "Options" tab
+    bool enableXkbOptions = config->readBoolEntry("EnableXkbOptions", true);
+    if( enableXkbOptions ) {
+	bool resetOldOptions = config->readBoolEntry("ResetOldOptions", false);
+	QString options = config->readEntry("Options", "");
+	if( !XKBExtension::setXkbOptions(options, resetOldOptions) ) {
+	    kdDebug() << "Setting XKB options failed!" << endl;
+	}
+    }
+
     if (config->readBoolEntry("Use", false) == true)
         kapp->startServiceByDesktopName("kxkb");
     delete config;
@@ -823,4 +855,9 @@ extern "C"
  I18N_NOOP("Special keys (Ctrl+Alt+<key>) handled in a server.");
  I18N_NOOP("Miscellaneous compatibility options");
  I18N_NOOP("Right Control key works as Right Alt");
+
+//these seem to be in x.org and Debian XFree86 4.3
+ I18N_NOOP("Right Alt key switches group while pressed");
+ I18N_NOOP("Left Alt key switches group while pressed");
+ I18N_NOOP("Press Right Alt-key to choose 3rd level");
 #endif
