@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "konq_dirpart.h"
@@ -22,6 +22,7 @@
 #include "konq_propsview.h"
 #include "konq_settings.h"
 
+#include <kio/paste.h>
 #include <kapplication.h>
 #include <kaction.h>
 #include <kdatastream.h>
@@ -135,8 +136,8 @@ KonqDirPart::KonqDirPart( QObject *parent, const char *name )
 
     actionCollection()->setHighlightingEnabled( true );
 
-    m_paIncIconSize = new KAction( i18n( "Increase Icon Size" ), "viewmag+", 0, this, SLOT( slotIncIconSize() ), actionCollection(), "incIconSize" );
-    m_paDecIconSize = new KAction( i18n( "Decrease Icon Size" ), "viewmag-", 0, this, SLOT( slotDecIconSize() ), actionCollection(), "decIconSize" );
+    m_paIncIconSize = new KAction( i18n( "Enlarge Icons" ), "viewmag+", 0, this, SLOT( slotIncIconSize() ), actionCollection(), "incIconSize" );
+    m_paDecIconSize = new KAction( i18n( "Shrink Icons" ), "viewmag-", 0, this, SLOT( slotDecIconSize() ), actionCollection(), "decIconSize" );
 
     m_paDefaultIcons = new KRadioAction( i18n( "&Default Size" ), 0, actionCollection(), "modedefault" );
     d->aEnormousIcons = new KRadioAction( i18n( "&Huge" ), 0,
@@ -220,7 +221,7 @@ KonqDirPart::KonqDirPart( QObject *parent, const char *name )
     KAction *a = new KAction( i18n( "Configure Background..." ), "background", 0, this, SLOT( slotBackgroundSettings() ),
                               actionCollection(), "bgsettings" );
 
-    a->setStatusText( i18n( "Allows choosing of background settings for this view" ) );
+    a->setToolTip( i18n( "Allows choosing of background settings for this view" ) );
 }
 
 KonqDirPart::~KonqDirPart()
@@ -420,12 +421,13 @@ void KonqDirPart::slotClipboardDataChanged()
     updatePasteAction();
 }
 
-void KonqDirPart::updatePasteAction()
+void KonqDirPart::updatePasteAction() // KDE4: merge into method above
 {
-    QMimeSource *data = QApplication::clipboard()->data();
-    bool paste = ( data->format() != 0 );
-
-    emit m_extension->enableAction( "paste", paste ); // TODO : if only one url, check that it's a dir
+    QString actionText = KIO::pasteActionText();
+    bool paste = !actionText.isEmpty();
+    if ( paste )
+      emit m_extension->setActionText( "paste", actionText );
+    emit m_extension->enableAction( "paste", paste );
 }
 
 void KonqDirPart::newItems( const KFileItemList & entries )
@@ -482,49 +484,48 @@ void KonqDirPart::emitTotalCount()
     emit setStatusBarText( bShowsResult ? i18n("Search result: %1").arg(summary) : summary );
 }
 
-void KonqDirPart::emitCounts( const KFileItemList & lst, bool selectionChanged )
+void KonqDirPart::emitCounts( const KFileItemList & lst )
 {
-    // Compare the new value with our cache
-    /*bool multiple = lst.count()>1;
-    if (multiple != m_bMultipleItemsSelected)
-    {
-        m_bMultipleItemsSelected = multiple;
-        updatePasteAction();
-    }*/
-
-    if ( lst.count()==1)
-    {
+    if ( lst.count() == 1 )
         emit setStatusBarText( ((KFileItemList)lst).first()->getStatusBarInfo() );
-    }
-    else if ( lst.count()>1)
+    else
     {
         long long fileSizeSum = 0;
         uint fileCount = 0;
         uint dirCount = 0;
 
-        for (KFileItemListIterator it( lst ); it.current(); ++it )
+        for ( KFileItemListIterator it( lst ); it.current(); ++it )
+        {
             if ( it.current()->isDir() )
                 dirCount++;
             else
             {
-                if (!it.current()->isLink()) // ignore symlinks
+                if ( !it.current()->isLink() ) // ignore symlinks
                     fileSizeSum += it.current()->size();
                 fileCount++;
             }
+        }
 
-        emit setStatusBarText( KIO::itemsSummaryString(fileCount + dirCount,
-                                                         fileCount,
-                                                         dirCount,
-                                                         fileSizeSum,
-                                                         true));
+        emit setStatusBarText( KIO::itemsSummaryString( fileCount + dirCount,
+                                                        fileCount, dirCount,
+                                                        fileSizeSum, true ) );
     }
-    else
+}
+
+void KonqDirPart::emitCounts( const KFileItemList & lst, bool selectionChanged )
+{
+    if ( lst.count() == 0 )
         emitTotalCount();
+    else
+        emitCounts( lst );
 
     // Yes, the caller could do that too :)
     // But this bool could also be used to cache the QString for the last
     // selection, as long as selectionChanged is false.
     // Not sure it's worth it though.
+    // MiB: no, I don't think it's worth it. Especially regarding the
+    //      loss of readability of the code. Thus, this will be removed in
+    //      KDE 4.0.
     if ( selectionChanged )
         emit m_extension->selectionInfo( lst );
 }
