@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qbitmap.h>
@@ -51,14 +51,14 @@ KonqPixmapProvider::~KonqPixmapProvider()
 // if not available, tries to find the pixmap for the mimetype of url
 // if that fails, gets the icon for the protocol
 // finally, inserts the url/icon pair into the cache
-QPixmap KonqPixmapProvider::pixmapFor( const QString& url, int size )
+QString KonqPixmapProvider::iconNameFor( const QString& url )
 {
     QMapIterator<QString,QString> it = iconMap.find( url );
     QString icon;
     if ( it != iconMap.end() ) {
         icon = it.data();
         if ( !icon.isEmpty() )
-	    return loadIcon( url, icon, size );
+	    return icon;
     }
 
     if ( url.isEmpty() ) {
@@ -81,15 +81,19 @@ QPixmap KonqPixmapProvider::pixmapFor( const QString& url, int size )
     // cache the icon found for url
     iconMap.insert( url, icon );
 
-    return loadIcon( url, icon, size );
+    return icon;
 }
 
+QPixmap KonqPixmapProvider::pixmapFor( const QString& url, int size )
+{
+    return loadIcon( url, iconNameFor( url ), size );
+}
 
 void KonqPixmapProvider::load( KConfig *kc, const QString& key )
 {
     iconMap.clear();
     QStringList list;
-    list = kc->readListEntry( key );
+    list = kc->readPathListEntry( key );
     QStringList::Iterator it = list.begin();
     QString url, icon;
     while ( it != list.end() ) {
@@ -120,7 +124,7 @@ void KonqPixmapProvider::save( KConfig *kc, const QString& key,
 
 	++it;
     }
-    kc->writeEntry( key, list );
+    kc->writePathEntry( key, list );
 }
 
 void KonqPixmapProvider::notifyChange( bool isHost, QString hostOrURL,
