@@ -5,7 +5,7 @@
 
   Copyright (c) 2000 Matthias Hoelzer-Kluepfel <hoelzer@kde.org>
                      Stefan Schimanski <1Stein@gmx.de>
-                2003 George Staikos <staikos@kde.org>
+                2003-2005 George Staikos <staikos@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -52,7 +52,7 @@
 #include <klocale.h>
 #include <kprocess.h>
 #include <kprotocolmanager.h>
-#include <kstddirs.h>
+#include <kstandarddirs.h>
 #include <ktempfile.h>
 #include <kurl.h>
 
@@ -245,8 +245,10 @@ NPError g_NPN_GetURL(NPP instance, const char *url, const char *target)
    kdDebug(1431) << "g_NPN_GetURL: url=" << url << " target=" << target << endl;
 
    NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
-   inst->requestURL( QString::fromLatin1(url), QString::null,
-                     QString::fromLatin1(target), 0 );
+   if (inst) {
+      inst->requestURL( QString::fromLatin1(url), QString::null,
+                        QString::fromLatin1(target), 0 );
+   }
 
    return NPERR_NO_ERROR;
 }
@@ -257,9 +259,11 @@ NPError g_NPN_GetURLNotify(NPP instance, const char *url, const char *target,
 {
     kdDebug(1431) << "g_NPN_GetURLNotify: url=" << url << " target=" << target << " inst=" << (void*)instance << endl;
    NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
-   kdDebug(1431) << "g_NPN_GetURLNotify: ndata=" << (void*)inst << endl;
-   inst->requestURL( QString::fromLatin1(url), QString::null,
-                     QString::fromLatin1(target), notifyData );
+   if (inst) {
+      kdDebug(1431) << "g_NPN_GetURLNotify: ndata=" << (void*)inst << endl;
+      inst->requestURL( QString::fromLatin1(url), QString::null,
+                        QString::fromLatin1(target), notifyData, true );
+   }
 
    return NPERR_NO_ERROR;
 }
@@ -352,7 +356,7 @@ NPError g_NPN_PostURLNotify(NPP instance, const char* url, const char* target,
    NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
    if (inst && !inst->normalizedURL(QString::fromLatin1(url)).isNull()) {
       inst->postURL( QString::fromLatin1(url), postdata, args.contentType(),
-                     QString::fromLatin1(target), notifyData, args );
+                     QString::fromLatin1(target), notifyData, args, true );
    } else {
       // Unsupported / insecure
       return NPERR_INVALID_URL;
@@ -367,8 +371,95 @@ NPError g_NPN_PostURL(NPP instance, const char* url, const char* target,
 {
 // http://devedge.netscape.com/library/manuals/2002/plugin/1.0/npn_api13.html
    kdDebug(1431) << "g_NPN_PostURL()" << endl;
+   kdDebug(1431) << "url=[" << url << "] target=[" << target << "]" << endl;
+   QByteArray postdata;
+   KParts::URLArgs args;
 
-   return g_NPN_PostURLNotify(instance, url, target, len, buf, file, 0L);
+   if (len == 0) {
+      return NPERR_NO_DATA;
+   }
+
+   if (file) { // buf is a filename
+      QFile f(buf);
+      if (!f.open(IO_ReadOnly)) {
+         return NPERR_FILE_NOT_FOUND;
+      }
+
+      // FIXME: this will not work because we need to strip the header out!
+      postdata = f.readAll();
+      f.close();
+   } else {    // buf is raw data
+      // First strip out the header
+      const char *previousStart = buf;
+      uint32 l;
+      bool previousCR = true;
+
+      for (l = 1;; l++) {
+         if (l == len) {
+            break;
+         }
+
+         if (buf[l-1] == '\n' || (previousCR && buf[l-1] == '\r')) {
+            if (previousCR) { // header is done!
+               if ((buf[l-1] == '\r' && buf[l] == '\n') ||
+                   (buf[l-1] == '\n' &&  buf[l] == '\r'))
+                  l++;
+               l++;
+               previousStart = &buf[l-1];
+               break;
+            }
+
+            QString thisLine = QString::fromLatin1(previousStart, &buf[l-1] - previousStart).stripWhiteSpace();
+
+            previousStart = &buf[l];
+            previousCR = true;
+
+            kdDebug(1431) << "Found header line: [" << thisLine << "]" << endl;
+            if (thisLine.startsWith("Content-Type: ")) {
+               args.setContentType(thisLine);
+            }
+         } else {
+            previousCR = false;
+         }
+      }
+
+      postdata.duplicate(previousStart, len - l + 1);
+   }
+
+   kdDebug(1431) << "Post data: " << postdata.size() << " bytes" << endl;
+#if 0
+   QFile f("/tmp/nspostdata");
+   f.open(IO_WriteOnly);
+   f.writeBlock(postdata);
+   f.close();
+#endif
+
+   if (!target || !*target) {
+      // Send the results of the post to the plugin
+      // (works by default)
+   } else if (!strcmp(target, "_current") || !strcmp(target, "_self") ||
+              !strcmp(target, "_top")) {
+      // Unload the plugin, put the results in the frame/window that the
+      // plugin was loaded in
+      // FIXME
+   } else if (!strcmp(target, "_new") || !strcmp(target, "_blank")){
+      // Open a new browser window and write the results there
+      // FIXME
+   } else {
+      // Write the results to the specified frame
+      // FIXME
+   }
+
+   NSPluginInstance *inst = static_cast<NSPluginInstance*>(instance->ndata);
+   if (inst && !inst->normalizedURL(QString::fromLatin1(url)).isNull()) {
+      inst->postURL( QString::fromLatin1(url), postdata, args.contentType(),
+                     QString::fromLatin1(target), 0L, args, false );
+   } else {
+      // Unsupported / insecure
+      return NPERR_INVALID_URL;
+   }
+
+   return NPERR_NO_ERROR;
 }
 
 
@@ -497,6 +588,7 @@ NSPluginInstance::NSPluginInstance(NPP privateData, NPPluginFuncs *pluginFuncs,
    : DCOPObject(), QObject( parent, name ) 
 {
     Q_UNUSED(embed);
+   _firstResize = true;
    _visible = false;
    _npp = privateData;
    _npp->ndata = this;
@@ -632,7 +724,9 @@ void NSPluginInstance::destroy()
         XtDestroyWidget(_toplevel);
 	_toplevel = 0;
 
-        ::free(_npp);   // matched with malloc() in newInstance
+        if (_npp) {
+            ::free(_npp);   // matched with malloc() in newInstance
+        }
 
         _destroyed = true;
     }
@@ -641,9 +735,11 @@ void NSPluginInstance::destroy()
 
 void NSPluginInstance::shutdown()
 {
-    NSPluginClass *cls =  static_cast<NSPluginClass*>(parent());
+    NSPluginClass *cls = dynamic_cast<NSPluginClass*>(parent());
     //destroy();
-    cls->destroyInstance( this );
+    if (cls) {
+        cls->destroyInstance( this );
+    }
 }
 
 
@@ -654,7 +750,7 @@ void NSPluginInstance::timer()
          return;
     }
 
-    _streams.clear();
+    //_streams.clear();
 
     // start queued requests
     kdDebug(1431) << "looking for waiting requests" << endl;
@@ -712,9 +808,8 @@ void NSPluginInstance::timer()
                 } else if (url.lower().startsWith("javascript:")){
                     if (_callback) {
                         static int _jsrequestid = 0;
-                        _callback->evalJavaScript( _jsrequestid, url.mid(11) );
-                        if ( req.notify )
-                            _jsrequests.insert(_jsrequestid++, new Request( req ));
+			_jsrequests.insert(_jsrequestid, new Request(req));
+                        _callback->evalJavaScript(_jsrequestid++, url.mid(11));
                     } else {
                         kdDebug() << "No callback for javascript: url!" << endl;
                     }
@@ -728,10 +823,10 @@ void NSPluginInstance::timer()
                     kdDebug() << "getting " << url << endl;
 
                     emitStatus( i18n("Requesting %1").arg(url) );
-                    s->get( url, req.mime, req.notify );
+                    s->get( url, req.mime, req.notify, req.reload );
                 }
 
-                break;
+                //break;
             }
         }
     }
@@ -758,7 +853,7 @@ QString NSPluginInstance::normalizedURL(const QString& url) const {
 
 
 void NSPluginInstance::requestURL( const QString &url, const QString &mime,
-                                   const QString &target, void *notify )
+                                   const QString &target, void *notify, bool forceNotify, bool reload )
 {
     // Generally this should already be done, but let's be safe for now.
     QString nurl = normalizedURL(url);
@@ -767,17 +862,15 @@ void NSPluginInstance::requestURL( const QString &url, const QString &mime,
     }
 
     kdDebug(1431) << "NSPluginInstance::requestURL url=" << nurl << " target=" << target << " notify=" << notify << endl;
-    _waitingRequests.enqueue( new Request( nurl, mime, target, notify ) );
-    if ( _streams.count() == 0 ) {
-        _timer->start( 0, true );
-    }
+    _waitingRequests.enqueue( new Request( nurl, mime, target, notify, forceNotify, reload ) );
+    _timer->start( 100, true );
 }
 
 
 void NSPluginInstance::postURL( const QString &url, const QByteArray& data,
                                 const QString &mime,
                                 const QString &target, void *notify,
-                                const KParts::URLArgs& args )
+                                const KParts::URLArgs& args, bool forceNotify )
 {
     // Generally this should already be done, but let's be safe for now.
     QString nurl = normalizedURL(url);
@@ -786,10 +879,8 @@ void NSPluginInstance::postURL( const QString &url, const QByteArray& data,
     }
 
     kdDebug(1431) << "NSPluginInstance::postURL url=" << nurl << " target=" << target << " notify=" << notify << endl;
-    _waitingRequests.enqueue( new Request( nurl, data, mime, target, notify, args) );
-    if ( _streams.count() == 0 ) {
-        _timer->start( 0, true );
-    }
+    _waitingRequests.enqueue( new Request( nurl, data, mime, target, notify, args, forceNotify) );
+    _timer->start( 100, true );
 }
 
 
@@ -800,11 +891,12 @@ void NSPluginInstance::emitStatus(const QString &message)
 }
 
 
-void NSPluginInstance::streamFinished( NSPluginStreamBase* /*strm*/ )
+void NSPluginInstance::streamFinished( NSPluginStreamBase* strm )
 {
    kdDebug(1431) << "-> NSPluginInstance::streamFinished" << endl;
    emitStatus( QString::null );
-   _timer->start( 0, true );
+   _streams.remove(strm); // deletes automatically
+   _timer->start( 100, true );
 }
 
 
@@ -849,18 +941,16 @@ int NSPluginInstance::setWindow(int remove)
 }
 
 
-static void resizeWidgetTree(Window w, int width, int height) {
+static void resizeWidgets(Window w, int width, int height) {
    Window rroot, parent, *children;
    unsigned int nchildren = 0;
 
    if (XQueryTree(qt_xdisplay(), w, &rroot, &parent, &children, &nchildren)) {
       for (unsigned int i = 0; i < nchildren; i++) {
          XResizeWindow(qt_xdisplay(), children[i], width, height);
-         resizeWidgetTree(children[i], width, height);
       }
       XFree(children);
    }
-
 }
 
 
@@ -877,6 +967,9 @@ void NSPluginInstance::resizePlugin(int w, int h)
    _width = w;
    _height = h;
 
+   XResizeWindow(qt_xdisplay(), XtWindow(_form), w, h);
+   XResizeWindow(qt_xdisplay(), XtWindow(_toplevel), w, h);
+
    Arg args[7];
    Cardinal nargs = 0;
    XtSetArg(args[nargs], XtNwidth, _width); nargs++;
@@ -886,11 +979,15 @@ void NSPluginInstance::resizePlugin(int w, int h)
    XtSetArg(args[nargs], XtNcolormap, QPaintDevice::x11AppColormap()); nargs++;
    XtSetArg(args[nargs], XtNborderWidth, 0); nargs++;
 
+   XtSetValues(_toplevel, args, nargs);
    XtSetValues(_form, args, nargs);
 
-   resizeWidgetTree(XtWindow(_form), _width, _height);
+   resizeWidgets(XtWindow(_form), _width, _height);
 
-   setWindow();
+   if (_firstResize) {
+      _firstResize = false;
+      setWindow();
+   }
 
    kdDebug(1431) << "<- NSPluginInstance::resizePlugin" << endl;
 }
@@ -907,7 +1004,7 @@ void NSPluginInstance::javascriptResult(int id, QString result) {
         _streams.append( s );
 
         int len = result.length();
-        s->create( req->url, QString("text/plain"), req->notify );
+        s->create( req->url, QString("text/plain"), req->notify, req->forceNotify );
         kdDebug(1431) << "javascriptResult has been called with: "<<result<<endl;
         if (len > 0) {
             QByteArray data(len + 1);
@@ -1101,6 +1198,10 @@ NSPluginViewer::NSPluginViewer( QCString dcopId,
    : DCOPObject(dcopId), QObject( parent, name ) 
 {
     _classes.setAutoDelete( true );
+    connect(KApplication::dcopClient(),
+            SIGNAL(applicationRemoved(const QCString&)),
+            this,
+            SLOT(appUnregistered(const QCString&)));
 }
 
 
@@ -1110,6 +1211,27 @@ NSPluginViewer::~NSPluginViewer()
 }
 
 
+void NSPluginViewer::appUnregistered(const QCString& id) {
+   if (id.isEmpty()) {
+      return;
+   }
+
+   QDictIterator<NSPluginClass> it(_classes);
+   NSPluginClass *c;
+   while ( (c = it.current()) ) {
+      QString key = it.currentKey();
+      ++it;
+      if (c->app() == id) {
+         _classes.remove(key);
+      }
+   }
+
+   if (_classes.isEmpty()) {
+      shutdown();
+   }
+}
+
+
 void NSPluginViewer::shutdown()
 {
    kdDebug(1431) << "NSPluginViewer::shutdown" << endl;
@@ -1131,6 +1253,12 @@ DCOPRef NSPluginViewer::newClass( QString plugin )
    if ( !cls ) {
        // create new class
        cls = new NSPluginClass( plugin, this );
+       QCString id = "";
+       DCOPClient *dc = callingDcopClient();
+       if (dc) {
+          id = dc->senderId();
+       }
+       cls->setApp(id);
        if ( cls->error() ) {
            kdError(1431) << "Can't create plugin class" << endl;
            delete cls;
@@ -1276,7 +1404,8 @@ void NSPluginClass::shutdown()
 
 DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
                                     QStringList argn, QStringList argv,
-                                    QString appId, QString callbackId )
+                                    QString appId, QString callbackId,
+                                    bool reload, bool doPost, QByteArray postData )
 {
    kdDebug(1431) << "-> NSPluginClass::NewInstance" << endl;
 
@@ -1294,8 +1423,11 @@ DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
 
    for (unsigned int i=0; i<argc; i++)
    {
-      const char *n = (const char*)argn[i].ascii();
-      const char *v = (const char*)argv[i].ascii();
+      QCString encN = argn[i].utf8();
+      QCString encV = argv[i].utf8();
+
+      const char *n = encN;
+      const char *v = encV;
 
       _argn[i] = strdup(n);
       _argv[i] = strdup(v);
@@ -1341,8 +1473,13 @@ DCOPRef NSPluginClass::newInstance( QString url, QString mimeType, bool embed,
    }
 
    // create source stream
-   if ( !src.isEmpty() )
-      inst->requestURL( src, mimeType, QString::null, 0 );
+   if ( !src.isEmpty() ) {
+       if (doPost) {
+           inst->postURL(src, postData, mimeType, QString::null, 0, KParts::URLArgs(), false);
+       } else {
+           inst->requestURL( src, mimeType, QString::null, 0, false, reload );
+       }
+   }
 
    _instances.append( inst );
    return DCOPRef(kapp->dcopClient()->appId(), inst->objId());
@@ -1363,6 +1500,7 @@ NSPluginStreamBase::NSPluginStreamBase( NSPluginInstance *instance )
    : QObject( instance ), _instance(instance), _stream(0), _tempFile(0L),
      _pos(0), _queue(0), _queuePos(0), _error(false)
 {
+   _informed = false;
 }
 
 
@@ -1422,7 +1560,7 @@ void NSPluginStreamBase::inform()
 
                 // stream into temporary file (use lower() in case the
                 // filename as an upper case X in it)
-                _tempFile = new KTempFile( QString::null, src.fileName().lower() );
+                _tempFile = new KTempFile;
                 _tempFile->setAutoDelete( TRUE );
                 _fileURL = _tempFile->name();
                 kdDebug() << "saving into " << _fileURL << endl;
@@ -1432,7 +1570,7 @@ void NSPluginStreamBase::inform()
 
 }
 
-bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, void *notify )
+bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, void *notify, bool forceNotify)
 {
     if ( _stream )
         return false;
@@ -1444,6 +1582,7 @@ bool NSPluginStreamBase::create( const QString& url, const QString& mimeType, vo
     _onlyAsFile = false;
     _streamType = NP_NORMAL;
     _informed = false;
+    _forceNotify = forceNotify;
 
     // create new stream
     _stream = new NPStream;
@@ -1488,8 +1627,9 @@ int NSPluginStreamBase::process( const QByteArray &data, int start )
           break;
       }
 
-      if (_tempFile)
+      if (_tempFile) {
           _tempFile->dataStream()->writeRawBytes(d, sent);
+      }
 
       to_sent -= sent;
       _pos += sent;
@@ -1511,8 +1651,9 @@ bool NSPluginStreamBase::pump()
 
         // handle AS_FILE_ONLY streams
         if ( _onlyAsFile ) {
-            if ( _tempFile )
+            if (_tempFile) {
                 _tempFile->dataStream()->writeRawBytes( _queue, _queue.size() );
+	    }
             newPos = _queuePos+_queue.size();
         } else {
             // normal streams
@@ -1572,16 +1713,17 @@ void NSPluginStreamBase::finish( bool err )
         }
 
         _instance->NPDestroyStream( _stream, NPRES_DONE );
-        if ( _notifyData )
+        if (_notifyData || _forceNotify)
             _instance->NPURLNotify( _url.url(), NPRES_DONE, _notifyData );
     } else {
         // close temp file
-        if ( _tempFile )
+        if ( _tempFile ) {
             _tempFile->close();
+	}
 
         // destroy stream
         _instance->NPDestroyStream( _stream, NPRES_NETWORK_ERR );
-        if ( _notifyData )
+        if (_notifyData || _forceNotify)
             _instance->NPURLNotify( _url.url(), NPRES_NETWORK_ERR, _notifyData );
     }
 
@@ -1660,7 +1802,7 @@ NSPluginStream::~NSPluginStream()
 
 
 bool NSPluginStream::get( const QString& url, const QString& mimeType,
-                          void *notify )
+                          void *notify, bool reload )
 {
     // create new stream
     if ( create( url, mimeType, notify ) ) {
@@ -1668,6 +1810,9 @@ bool NSPluginStream::get( const QString& url, const QString& mimeType,
         _job = KIO::get(KURL( url ), false, false);
         _job->addMetaData("errorPage", "false");
         _job->addMetaData("AllowCompressedPage", "false");
+        if (reload) {
+            _job->addMetaData("cache", "reload");
+        }
         connect(_job, SIGNAL(data(KIO::Job *, const QByteArray &)),
                 SLOT(data(KIO::Job *, const QByteArray &)));
         connect(_job, SIGNAL(result(KIO::Job *)), SLOT(result(KIO::Job *)));
